/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.checkpoint;

public class CheckpointPathBuilder {

    private final static String CHECKPOINT_ROOT = "checkpoints/";

    enum CheckpointPaths {

        SAVE("save/"), RESTORE("restore/"), BAK("bak/");

        private String path;

        CheckpointPaths(String path) {
            this.path = path;
        }

        @Override
        public String toString() {
            return path;
        }
    }

    public static String getRootSavePath() {
        return buildCheckpointSavePath("");
    }

    public static String getRootRestorePath() {
        return buildCheckpointRecoverPath("");
    }

    public static String getRootBakPath() {
        return buildCheckpointBakPath("");
    }

    public static String buildCheckpointSavePath(String id) {
        return CHECKPOINT_ROOT + CheckpointPaths.SAVE + id;
    }

    public static String buildCheckpointRecoverPath(String id) {
        return CHECKPOINT_ROOT + CheckpointPaths.RESTORE + id;
    }

    public static String buildCheckpointBakPath(String id) {
        return CHECKPOINT_ROOT + CheckpointPaths.BAK  + id;
    }
}
