/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.server;

import beehive.thrift.ResultObject_thrift;
import beehive.util.*;

import java.io.*;
import java.util.HashMap;

public class ResultObject implements Externalizable {
    public int status; // 0 means object is found, 1 means relocated, -1 means not found and no location information.
    public String statuscode;        // TODO MAKE THIS AN ENUM
    public Object node;
    public String nodeId;
    public String location;

    @Override
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        this.status = in.readInt();
        this.statuscode = (String) in.readObject();
        this.nodeId = (String) in.readObject();
        this.location = (String) in.readObject();
        Object obj = in.readObject();
        try {
            this.node = (Node) obj;
        } catch (ClassCastException e) {
            this.node = (NodeData) obj;
        }
    }

    @Override
    public void writeExternal(ObjectOutput out) throws IOException {
        out.writeInt(this.status);
        out.writeObject(this.statuscode);
        out.writeObject(this.nodeId);
        out.writeObject(this.location);
        out.writeObject(this.node);
    }

    public ResultObject() {
        this.node = null;
        this.location = "";
        this.statuscode = "";
    }

    public ResultObject(Node node, String nodeid) {
        status = Constants.SUCCESS;
        this.node = node;
        location = "";
        nodeId = nodeid;
    }

    public ResultObject(NodeData node, String nodeid) {
        status = Constants.SUCCESS;
        this.node = node;
        location = "";
        nodeId = nodeid;
    }

    // Constructor to be used when object is relocated
    public ResultObject(String nodeid, String location) {
        status = Constants.RELOCATED;
        this.node = null;
        this.location = location;
        this.nodeId = nodeid;
        this.statuscode = Constants.NODE_RELOCATED;
    }

    public ResultObject(String nodeid, String location, Node node) {
        status = Constants.RELOCATED;
        this.node = node;
        this.location = location;
        this.nodeId = nodeid;
        this.statuscode = Constants.NODE_RELOCATED;
    }

    public ResultObject(String nodeid, String location, NodeData node) {
        status = Constants.RELOCATED;
        this.node = node;
        this.location = location;
        this.nodeId = nodeid;
        this.statuscode = Constants.NODE_RELOCATED;
    }

    public ResultObject(Node node, String nodeid, NodeMetadata m, String s) {
        this.node = node;
        location = "";
        nodeId = nodeid;
        statuscode = s;
    }

    public ResultObject(NodeData node, String nodeid, NodeMetadata m, String s) {
        this.node = node;
        location = "";
        nodeId = nodeid;
        //		metadata = m;
        statuscode = s;
    }

    public ResultObject(Node node, String nodeId, String s) {
        this.node = node;
        this.nodeId = nodeId;
        statuscode = s;
    }

    public ResultObject(NodeData node, String nodeId, String s) {
        this.node = node;
        this.nodeId = nodeId;
        statuscode = s;
    }

    public ResultObject(Node node, String nodeid, NodeMetadata m, String s, int stat) {
        status = stat;
        this.node = node;
        location = "";
        nodeId = nodeid;
        statuscode = s;
    }

    public ResultObject(NodeData node, String nodeid, NodeMetadata m, String s, int stat) {
        status = stat;
        this.node = node;
        location = "";
        nodeId = nodeid;
        statuscode = s;
    }

    public ResultObject(String nodeID, String s, int stat) {
        status = stat;
        nodeId = nodeID;
        statuscode = s;
    }

    public ResultObject(String nodeid) {
        status = Constants.FAILED;
        this.node = null;
        this.location = "";
        this.nodeId = nodeid;
    }

    public ResultObject(String nodeid, int status) {
        this.status = status;
        this.nodeId = nodeid;
    }

    public ResultObject(int status, Node node, String nodeId, String location) {
        this.status = status;
        this.node = node;
        this.nodeId = nodeId;
        this.location = location;
    }

    public ResultObject(int status, NodeData node, String nodeId, String location) {
        this.status = status;
        this.node = node;
        this.nodeId = nodeId;
        this.location = location;
    }

    public ResultObject(ResultObject_thrift res_thrift, HashMap<Byte, Class> nodeClassMap) {
        this.status = res_thrift.status;
        this.statuscode = res_thrift.statuscode;
        if (res_thrift.isSetN()) {
            this.node = new Node(res_thrift.n);
        } else if (res_thrift.isSetNd()) {
            this.node = new NodeData(res_thrift.nd, nodeClassMap);
        } else {
            this.node = null;
        }
        this.nodeId = res_thrift.nodeId;
        this.location = res_thrift.location;
    }

    @Override
    public String toString() {
        return "ResultObject{" +
                "status=" + status +
                ", statuscode='" + statuscode + '\'' +
                ", node=" + node +
                ", nodeId='" + nodeId + '\'' +
                ", location='" + location + '\'' +
                '}';
    }
}