/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.thrift;

import org.apache.thrift.TProcessor;
import org.apache.thrift.protocol.TCompactProtocol;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.server.TServer;
import org.apache.thrift.server.TThreadPoolServer;
import org.apache.thrift.transport.TServerSocket;
import org.apache.thrift.transport.TServerTransport;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransport;
import org.apache.thrift.transport.TTransportException;

public class BeehiveThriftBuilder{
	public static int timeout = 180000;
	
	public static void setThriftConnectionTimeout(int timeoutValue){
		timeout = timeoutValue;
	}
	
	public static TProtocol getThriftClientProtocol(String thriftHostname, int thriftPort) throws TTransportException{
    	TTransport transport=null;
        TProtocol protocol=null;
        transport = new TSocket(thriftHostname, thriftPort, timeout);
		transport.open();
		protocol = new  TCompactProtocol(transport);
		return protocol;
    }
	
	public static TServer getThriftServer(int thriftPort, TProcessor processor) throws TTransportException{
    	TServerTransport serverTransport = new TServerSocket(thriftPort);
    	TThreadPoolServer.Args a = new TThreadPoolServer.Args(serverTransport).processor(processor).protocolFactory(new TCompactProtocol.Factory());
    	//a.maxWorkerThreads(8);
    	TServer server = new TThreadPoolServer(a);
    	return server;
    }
}
