/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import java.util.*;
import java.io.*;


public class Config {
    protected static Config config = null;

    private String configFile = null;
    private Hashtable<String, Object> configTable;
    private Logger logger = null;

    public Config() {
        configTable = new Hashtable<String, Object>();
        createLogger();
        //  hbaseConfig = new Configuration();
        config = this;
    }

    public Config(String configFile) {
        this.configFile = configFile;
        configTable = new Hashtable<String, Object>();
        readConfigFile();
        createLogger();
        //hbaseConfig = HBaseConfiguration.create();
        config = this;
    }

    private void readConfigFile() {
        try {
            FileReader fr = new FileReader(configFile);
            BufferedReader bread = new BufferedReader(fr);
            String line;
            while (bread.ready()) {
                line = bread.readLine();
                if (line.length() < 3 || line.charAt(0) == '#' || !line.contains("="))
                    continue;
                System.out.println(line);
                StringTokenizer st = new StringTokenizer(line, "=");
                String key = st.nextToken();
                String value = st.nextToken();
                configTable.put(key, value);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void createLogger() {
        if (has("logFile")) {
            String logfile = getStringValue("logFile");
            try {

                logger = new Logger(logfile);

            } catch (Exception e) {

                e.printStackTrace();
            }
        } else {
            logger = new Logger();
        }
        String loglevels = getStringValue("Logger.logLevels");
        if (loglevels.contains("debug"))
            logger.setDebug(true);
        if (loglevels.contains("info"))
            logger.setInfo(true);
        if (loglevels.contains("warning"))
            logger.setWarning(true);

        //if(getBoolValue("printConsole")==false)
        //logger.setPrintConsole(false);
    }

    public Object get(String key) {
        return configTable.get(key);
    }

    public String getStringValue(String key) {
        return (String) get(key);
    }

    public boolean getBoolValue(String key) {
        Boolean boolVal = new Boolean((String) get(key));
        return boolVal;
    }

    public int getIntValue(String key) {
        Integer intVal = new Integer((String) get(key));
        return intVal;
    }

    public long getLongValue(String key) {
        Long longVal = new Long((String) get(key));
        return longVal;
    }

    public float getFloatValue(String key) {
        Float floatVal = new Float((String) get(key));
        return floatVal;
    }

    public boolean has(String key) {
        return configTable.containsKey(key);
    }

    public void add(String key, Object value) {
        configTable.put(key, value);
    }

    public Logger getLogger() {
        return logger;
    }

    /*  public Configuration getHBaseConfig(){
          return hbaseConfig;
      }
     */
    public static Config getInstance() {
        return config;
    }
}
