/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.concurrent.ConcurrentHashMap;

import beehive.thrift.*;
import beehive.server.*;

public class ConvertRMIToThrift {

    public ConvertRMIToThrift() {
    }

    public static ResultObject_thrift convertResultObject(ResultObject res) {

        ResultObject_thrift result = new ResultObject_thrift(res.status, res.statuscode, res.nodeId, res.location);
        if (res.node != null) {
            if (res.node instanceof Node) {
                Node res_node = (Node) res.node;
                Node_thrift node_thrift = convertNode(res_node);
                result.setN(node_thrift);
            } else {
                NodeData res_node = (NodeData) res.node;
                NodeData_thrift node_thrift = convertNodeData(res_node, res_node.nodeClassId);
                result.setNd(node_thrift);
            }
        }
        return result;
    }

    private static Node_thrift convertNode(Node node) {

        Map<String, Edge_thrift> neighbors_thrift = new ConcurrentHashMap<String, Edge_thrift>();
        for (String str : node.neighbors.keySet()) {
            neighbors_thrift.put(str, new Edge_thrift());
        }
        Node_thrift n = new Node_thrift(node.nodeId, node.TS, neighbors_thrift);

        return n;
    }

    static NodeData_thrift convertNodeData(NodeData nd, byte classNameID) {

        Map<String, String> properties = new HashMap<String, String>();
        for (Map.Entry<String, Object> entry : nd.properties.entrySet()) {
            if (entry.getValue() == null) {
                properties.put(entry.getKey(), "");
            } else {
                properties.put(entry.getKey(), entry.getValue().toString());
            }

        }
        NodeData_thrift nd_thrift = new NodeData_thrift(nd.nodeId, nd.TS, classNameID, properties);
        return nd_thrift;

    }

    static Vector<NodeData_thrift> convertNodeDataList(Vector<NodeData> vec) {

        Vector<NodeData_thrift> vec_thrift = new Vector<NodeData_thrift>();

//		ConvertRMIToThrift c = new ConvertRMIToThrift();
        for (NodeData ele : vec) {
            vec_thrift.add(convertNodeData(ele, ele.nodeClassId));
        }

        return vec_thrift;
    }

    public static Vector<ResultObject_thrift> convertResultObjectList(Vector<ResultObject> resList) {

        Vector<ResultObject_thrift> result = new Vector<ResultObject_thrift>();

        for (ResultObject ele : resList) {
            result.add(convertResultObject(ele));
        }
        return result;

    }


    public static LockRequest_thrift convertLockRequest(LockRequest lockRequest) {
        return new LockRequest_thrift(lockRequest.nodeId, lockRequest.ts, lockRequest.lockMode);
    }
}
