/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import java.util.HashMap;
import java.util.Map;
import java.util.Vector;
import java.util.concurrent.Callable;

import beehive.server.HashtableServer;

public class GetRemoteNodes implements Callable<Map<Integer, Vector<Node>>> {

    private int remoteServerId;
    private HashtableServer server;
    private int currentServerID;
    private Vector<String> nodeIds;

    public GetRemoteNodes(HashtableServer hashTableServer, int s, int rs,
                          Vector<String> nodes) {
        this.server = hashTableServer;
        this.currentServerID = s;
        this.remoteServerId = rs;
        this.nodeIds = nodes;
    }

    @Override
    public Map<Integer, Vector<Node>> call() throws Exception {
        Map<Integer, Vector<Node>> res = new HashMap<Integer, Vector<Node>>();
        res.put(remoteServerId,
                server.getNodesForCaching(nodeIds, currentServerID));
        return res;
    }

}