/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import beehive.thrift.LockRequest_thrift;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

public class LockRequest implements Comparable<LockRequest>, Externalizable {

    public String nodeId;
    public long ts;
    public int lockMode;

    public LockRequest(int lockMode, long ts, String nodeId) {
        this.nodeId = nodeId;
        this.ts = ts;
        this.lockMode = lockMode;
    }

    public LockRequest(LockRequest_thrift lrt) {
        this.nodeId = lrt.nodeId;
        this.ts = lrt.ts;
        this.lockMode = lrt.lockMode;
    }

    public LockRequest() {
    }

    @Override
    public int compareTo(LockRequest lockRequest) {
        return nodeId.compareTo(lockRequest.nodeId);
    }

    @Override
    public void writeExternal(ObjectOutput objectOutput) throws IOException {
        objectOutput.writeObject(nodeId);
        objectOutput.writeLong(ts);
        objectOutput.writeInt(lockMode);
    }

    @Override
    public void readExternal(ObjectInput objectInput) throws IOException, ClassNotFoundException {
        nodeId = (String) objectInput.readObject();
        ts = objectInput.readLong();
        lockMode = objectInput.readInt();
    }

    @Override
    public String toString() {
        String lockModeStr = "";
        switch (lockMode) {
            case Constants.WRITE:
                lockModeStr = "write";
                break;
            case Constants.READ:
                lockModeStr = "read";
                break;
            default:
                break;
        }
        return "{" +
                "nodeId='" + nodeId + '\'' +
                ", ts=" + ts +
                ", lockMode=" + lockModeStr +
                '}';
    }
}
