/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import java.io.*;
import java.util.*;

public class Logger {
	
	public static final int INFO = 1;
	public static final int WARNING = 2;
	public static final int DEBUG = 4;
	
	//Buffer size
	public static int BUF_SIZE = 8000;
		
	StringBuilder buf;
	String logFile;
	PrintWriter logWriter;
	int logLevel;
	boolean debug=false,info=false,warning=false,fatal=true;
	boolean printLevel = false;
	boolean printConsole = true;
	boolean printFile = false;
	boolean autoFlush = true;
	boolean fullFormat = false;
		
	/**
	 * @param logfile file path where the log should be written
	 */
	/* **********************************************************
	 *
	 * **********************************************************/	
	public Logger(String logfile) throws IOException {
		this.logFile = logfile;		
		logWriter = new PrintWriter(new FileOutputStream(logFile,true),true);
		buf = new StringBuilder();
		Config configParams = Config.getInstance();
		if(configParams!=null){
			printFile = configParams.getBoolValue("Logger.PrintToFile");	
			printConsole = configParams.getBoolValue("Logger.PrintToConsole");
			BUF_SIZE = configParams.getIntValue("Logger.BUF_SIZE");
		}
	}	
	
	public Logger(){

		printFile = false;
		printConsole = true;
	
	}
	
	public void logDebug(String msg){
		if(debug){
			log(msg,"DEBUG");
		}
	}
	
	public void logInfo(String msg){
		if(info){
			log(msg,"INFO");
		}
	}
	
	public void logWarning(String msg){
		if(warning){
			log(msg,"WARNING");
		}
	}
	
	public void logFatal(String msg){
		if(fatal){
			log(msg,"FATAL_ERROR");
		}
	}

	public synchronized void log(String msg,String level){
		String logLine = msg;	
		
		if(printFile){
			buf.append(msg+"\n");
			if(buf.length() > BUF_SIZE) {
					buf.append("\n***********************************************\n");
					logWriter.println(buf.toString());	
					buf.setLength(0);
			}
		}
		
		if (fullFormat) 
			logLine = new Date().toString() +"|"+System.currentTimeMillis()+"|"+ level +"|"+  msg;
		
		if(printConsole){
			System.out.println(msg);
			System.out.flush();
		}

	}
	
	public void logMsgOnly(String msg){
		if(printConsole){
			System.out.println(msg);
		}
		if(printFile)
			logWriter.println(msg);
	}

	public void setDebug(boolean debug) {
		this.debug = debug;
		System.out.println("Debug level set to"+debug);
	}

	public void setInfo(boolean info) {
		this.info = info;
		System.out.println("Info level set to"+info);
	}

	public void setWarning(boolean warning) {
		this.warning = warning;
		System.out.println("Warning level set to"+warning);
	}
		

	public void setPrintConsole(boolean printConsole) {
		this.printConsole = printConsole;
	}
	
	public void flush(){
		if(printFile){
				buf.append("\n***********************************************\n");
                logWriter.println(buf.toString());
                buf.setLength(0);
				logWriter.flush();
		}
	}
	
}
