/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.List;
import java.util.ArrayList;
/**
 * MetaData class hold metadata of the nodes to know if it is in cache or it is
 * the master copy.
 * 
 * @author thirunavukarasu
 *
 */
public class NodeMetadata implements Externalizable {
	public String nodeID;
	public String masterNodeID;
	public boolean isMaster;
	public List<Integer> cacheCopyNodes;
	public int masterCopySite;
	public volatile long cachingInProgressFlag;
	public int numTimesUsed = 0;
	public long lastAccessedTS = 0;

	public NodeMetadata() {
		cachingInProgressFlag = -1;
		cacheCopyNodes = new ArrayList<Integer>();
	}

	public String getNodeID() {
		return nodeID;
	}

	public void setNodeID(String nodeID) {
		this.nodeID = nodeID;
	}

	public String getMasterNodeID() {
		return masterNodeID;
	}


	public void setMasterNodeID(String masterNodeID) {
		this.masterNodeID = masterNodeID;
	}

	public boolean isMaster() {
		return isMaster;
	}

	public void setMaster(boolean isMaster) {
		this.isMaster = isMaster;
	}

	public List<Integer> getCacheCopyNodes() {
		return cacheCopyNodes;
	}

	public void setCacheCopyNodes(ArrayList<Integer> cacheCopyNodes) {
		this.cacheCopyNodes = cacheCopyNodes;
	}

	public int getMasterCopyNode() {
		return masterCopySite;
	}

	public void setMasterCopyNode(int masterCopyNode) {
		this.masterCopySite = masterCopyNode;
	}

	/*public synchronized boolean testAndSet(boolean nonBlocking, long storageSystemThreadId) {
		if (cachingInProgressFlag >= 0) {
			if(cachingInProgressFlag == storageSystemThreadId) {
				System.out.println("***** NodeMetdata : testAndSet : Storage System Thread " + storageSystemThreadId + " is trying to cache Node " + nodeID + " again*****");
			}
			if (!nonBlocking) {
				try {
					wait();
				} catch (Exception e) {
					e.printStackTrace();
					return false;
				}
			}else {
				cachingInProgressFlag = storageSystemThreadId;
				//there might be some threads waiting on the thread that was caching earlier, notify them
				notifyAll();
			}
			return false;
		} else {
			cachingInProgressFlag = storageSystemThreadId;
			return true;
		}
	}*/
	
	//Simplifying the above function since currently we always have non blocking
	public boolean testAndSet(boolean nonBlocking, long storageSystemThreadId) {
		if (cachingInProgressFlag >= 0) {
			if(cachingInProgressFlag == storageSystemThreadId) {
				System.out.println("***** NodeMetdata : testAndSet : Storage System Thread " + storageSystemThreadId + " is trying to cache Node " + nodeID + " again*****");
			}
			cachingInProgressFlag = storageSystemThreadId;
			return false;
		} else {
			cachingInProgressFlag = storageSystemThreadId;
			return true;
		}
	}
	
	@Override  
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {  
		this.nodeID = (String) in.readObject();
		this.masterNodeID = (String) in.readObject();
		this.isMaster = in.readBoolean();
		this.masterCopySite = in.readInt();
		this.cachingInProgressFlag = in.readLong();
		this.numTimesUsed = in.readInt();
		this.lastAccessedTS = in.readLong();
		
		int[] intcacheCopyNodes = (int[]) in.readObject();
		for(int i=0;i<intcacheCopyNodes.length;i++)
			this.cacheCopyNodes.add(intcacheCopyNodes[i]);
	}  

	@Override  
	public void writeExternal(ObjectOutput out) throws IOException {  
		out.writeObject(this.nodeID);
		out.writeObject(this.masterNodeID);
		out.writeBoolean(this.isMaster);
		out.writeInt(this.masterCopySite);
		out.writeLong(this.cachingInProgressFlag);
		out.writeInt(this.numTimesUsed);
		out.writeLong(this.lastAccessedTS);
		
		int[] intcacheCopyNodes = new int[this.cacheCopyNodes.size()];
		int i=0;
		for(Integer o: this.cacheCopyNodes){
			intcacheCopyNodes[i] = o.intValue();
			i++;
		}
		out.writeObject(intcacheCopyNodes);
	}

}
