/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import java.io.*;
import java.util.*;

//Tara: implements serializable - in case in future local workpools send their stats to master
public class QueueStat implements Serializable{
	public long lastUpdateTime;
	public long startTime;
	public long totalArrivals;
	public long totalDepartures;
	public long currentCount;
	public long totalIdleTime;
	public double cumulativeArea;
	public boolean start;
	
	public QueueStat() 
	{
		startTime = System.nanoTime();
		lastUpdateTime = System.nanoTime();
		totalArrivals = totalDepartures = currentCount = totalIdleTime = 0;
		cumulativeArea = 0;
		start = true;
	}

	public void printStats() {	
	//TODO:
		System.out.println("Average Waiting Time: "+ getAvgWaitingTime());
		System.out.println("Average Pool Length: "+ getAvgQueueLength());
		System.out.println("Utilization \t: "+ getUtilization());
		System.out.println("Total Completions \t: "+ totalDepartures);
		System.out.println("");	
	}

		//TODO: Edge scenario where the curret count is stable for a while and still it is not added to aread because of no report completiion..
	//On Readypool it is the actual average waiting time 
	//On Activepool it is the actual average running time
	public double getAvgWaitingTime() {
		return ( cumulativeArea / totalArrivals);
	}

	public double getUtilization(){
		double currTime = (double)System.nanoTime();
		return (1  - totalIdleTime/(currTime-startTime));
	}
	//On ReadPool it will the average waitig tasks
	//On ActivePool it will be the average concurrency we achieved
	public double getAvgQueueLength() {			
		long currTime = System.nanoTime();
		return (cumulativeArea*1000000)/(currTime-startTime); 
	}
	
	public synchronized void reportArrival() {
		totalArrivals++;
		long currTime = System.nanoTime();
		if(!start)
		cumulativeArea = cumulativeArea + (((double)currTime - lastUpdateTime)/1000000)*currentCount;

  		currentCount++;
		
		if(currentCount==1) { 
			if(start){
				start = false;
				startTime = System.nanoTime();	
			}
			else
			totalIdleTime = totalIdleTime + ( currTime - lastUpdateTime);
		}
		lastUpdateTime = currTime;
	}

	public synchronized void reportDeparture() {
		totalDepartures++;
		long currTime = System.nanoTime();
		cumulativeArea = cumulativeArea + (((double)currTime - lastUpdateTime)/1000000)*currentCount ;
	        currentCount--;	

		//if(currentCount==0)
		
	}

	public synchronized void reportDeparture(int count) {
		totalDepartures = totalDepartures + count;
		long currTime = System.nanoTime();
		cumulativeArea = cumulativeArea + (((double)currTime - lastUpdateTime)/1000000)*currentCount;
		currentCount = currentCount - count;
	}
}
