/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import java.io.*;
import java.util.*;
import beehive.thrift.RWSetInfo_thrift;

public class RWSetInfo implements Externalizable{
	public HashSet<String> readSet;
	public HashSet<String> writeSet;
	public HashSet<String> unusedSet;

	public RWSetInfo()
	{
		this.readSet = new HashSet<String>();
		this.writeSet = new HashSet<String>();
		this.unusedSet = new HashSet<String>();
	}

	public RWSetInfo(RWSetInfo_thrift o){
		this.readSet = (HashSet)o.readSet;
		this.writeSet = (HashSet)o.writeSet;
		this.unusedSet = (HashSet)o.unusedSet;
	}

	@Override
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
		this.readSet = (HashSet<String>) in.readObject();
		this.writeSet = (HashSet<String>) in.readObject();
		this.unusedSet = (HashSet<String>) in.readObject();
	}

	@Override
	public void writeExternal(ObjectOutput out) throws IOException {
		out.writeObject(this.readSet);
		out.writeObject(this.writeSet);
		out.writeObject(this.unusedSet);
	}

	public String toString() {
		return "ReadSet:" + readSet.toString() + "\t" + "WriteSet:"+ writeSet.toString() + "\tUnusedSet:"+ unusedSet.toString();
	}
}
