/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import beehive.server.*;

import java.rmi.server.UnicastRemoteObject;
import java.rmi.*;
import java.rmi.registry.*;
import java.util.*;
import java.io.*;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentHashMap;


public class RemoteRequestExecutorHandler implements Callable {

    public HashtableServer serverHandle;
    public RequestedOp op;
    public Vector<Object> params;
    public Vector<ResultObject> result;
    public StorageSystem storageSys;

    public RemoteRequestExecutorHandler(int hostId, RequestedOp op,
                                        Vector<Object> params, StorageSystem storageSys) {
        this.op = op;
        this.params = params;
        result = null;
        this.storageSys = storageSys;

        if (StorageSystem.RPC_MECHANISM.equals("RMI")) {
            serverHandle = storageSys.servers.get(hostId);
        } else if (StorageSystem.RPC_MECHANISM.equals("THRIFT")) {
            serverHandle = storageSys.serversThrift.get(hostId);
        }

    }

    public Object call() {
        int tries = 0;
        while (true) {
            try {
                if (op == RequestedOp.getNodes) {
                    Vector<String> keyList = (Vector<String>) params.get(0);
                    result = serverHandle.getObjects(keyList);
                    return result;
                } else if (op == RequestedOp.getNodeProperties) {
                    Vector<NodeData> keyList = (Vector<NodeData>) params.get(0);
                    result = serverHandle.getObjectsProperties(keyList);
                    return result;
                } else if (op == RequestedOp.putNodes) {
                    ConcurrentHashMap<String, Node> bucket = (ConcurrentHashMap<String, Node>) params.get(0);
                    result = serverHandle.putObjects(bucket);
                    return result;
                } else if (op == RequestedOp.updateNodeProperties) {
                    Vector<NodeData> bucket = (Vector<NodeData>) params.get(0);
                    result = serverHandle.updateNodePropertiesParallel(bucket);
                    return result;
                }
                break;

            } catch (ConnectException ce) {
                tries++;
                if (tries > 10) {
                    break;
                }
                try {
                    Thread.sleep(10);
                } catch (Exception e) {
                }
            } catch (Exception e) {
                e.printStackTrace();
                break;
            }
        }
        return null;
    }
}
