/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.util;

import java.io.*;

public class Task implements Externalizable {
	//public String taskId;	// globally unique task id, node id string appended with a local unique id generated. 
	public String nodeId; 
	public TransactionInfo txnInfo;
	public String phase; 

	// locality information
	public int affinityLevel;
	public String targetLocation;	

	public int status;

	// TODO: this statistics will need to be populated
	//public int retryCount = 0;
	//public long initTime = 0;
	//public long scheduleTime = 0;

	public Task() {
		this.nodeId = "";
		this.txnInfo = null;
		this.phase = "GENERIC";
		this.affinityLevel = TaskConstants.NO_AFFINITY;
		this.targetLocation = "";
		this.status = TaskConstants.TASK_INIT; 
	}

	public Task(String id ) {
		nodeId = id;
		txnInfo = null;
		status = TaskConstants.TASK_INIT; 
		//initTime = System.nanoTime(); 
		phase = "GENERIC";
	}
	
	public Task(String id, TransactionInfo txnInfo) {
		nodeId = id;
		this.txnInfo = txnInfo;
		affinityLevel = TaskConstants.NO_AFFINITY;
		status = TaskConstants.TASK_INIT; 
		//initTime = System.nanoTime(); 
		phase = "GENERIC";
	}
	
	public Task(String id, TransactionInfo txnInfo, int affinityLevel, String location) {
		nodeId = id;
		this.txnInfo = txnInfo;
		this.affinityLevel = affinityLevel;
		this.targetLocation = location;
		status = TaskConstants.TASK_INIT; 
		//initTime = System.nanoTime(); 
		phase = "GENERIC";
	}

    public void setPhase(String phase) {
		this.phase = phase;
	}


	public String getNodeId() {
		return nodeId;
	}

	public String toString() {
		return nodeId;
	}

	public TransactionInfo getTxnInfo() {
		return txnInfo;
	}

	public void setStatus(int status) {
		this.status = status;
	}

	
	public void setAffinityLevel(int affinityLevel, String location) {
		this.affinityLevel = affinityLevel;
		this.targetLocation = location;
	}

	public boolean equals(Object o){
		Task t = (Task)o;
		if(t==null)
			return false;
		if(this.nodeId.equals(t.nodeId))
			return true;
		else
			return false;
	}
	
	@Override
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
		this.nodeId = (String) in.readObject();
        this.txnInfo = (TransactionInfo) in.readObject();
        this.phase = (String) in.readObject();
        this.affinityLevel = in.readInt();
		this.targetLocation = (String) in.readObject();
        this.status = in.readInt();
	}

	@Override
	public void writeExternal(ObjectOutput out) throws IOException {
		out.writeObject(this.nodeId);
        out.writeObject(this.txnInfo);
        out.writeObject(this.phase);
		out.writeInt(this.affinityLevel);
		out.writeObject(this.targetLocation);
		out.writeInt(this.status);
	}
}
