/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.validation;

import java.io.*;
import java.util.*;

import beehive.thrift.*;
import beehive.workpool.LoadInfo;

public class ValidateResponse implements Externalizable {
    public boolean commit;    // commit or abort status
    public long commitTS;    // commit ts if the txn committed
    public long latestSTS;    // latest value of sts
    public HashSet<String> refetchSet;    // objects in the read-write-set which got modified
    public boolean localResponse = false;    // is the response from local validator
    public boolean speculativeAbort = false;    //in case of abort is the abort speculative
    Vector<LoadInfo> loadVector;
    public long conflictingTS; //largest conflicting time stamp with which transaction conflicted

    @Override
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        this.commit = in.readBoolean();
        this.commitTS = in.readLong();
        this.latestSTS = in.readLong();
        this.refetchSet = (HashSet<String>) in.readObject();
        this.localResponse = in.readBoolean();
        this.speculativeAbort = in.readBoolean();
        this.loadVector = (Vector<LoadInfo>) in.readObject();
        this.conflictingTS = in.readLong();
    }

    @Override
    public void writeExternal(ObjectOutput out) throws IOException {
        out.writeBoolean(this.commit);
        out.writeLong(this.commitTS);
        out.writeLong(this.latestSTS);
        out.writeObject(this.refetchSet);
        out.writeBoolean(this.localResponse);
        out.writeBoolean(this.speculativeAbort);
        out.writeObject(this.loadVector);
        out.writeLong(this.conflictingTS);
    }

    public ValidateResponse() {
        this.commit = false;
        this.commitTS = 0L;
        this.latestSTS = 0L;
        this.refetchSet = new HashSet<String>();
        this.localResponse = false;
        this.speculativeAbort = false;
        this.loadVector = new Vector<LoadInfo>();
        this.conflictingTS = 0L;
    }

    public ValidateResponse(ValidateResponse_thrift o) {
        this.commit = o.commit;
        this.commitTS = o.commitTS;
        this.latestSTS = o.latestSTS;
        this.refetchSet = (HashSet<String>) o.refetchSet;
        this.localResponse = o.localResponse;
        this.speculativeAbort = o.speculativeAbort;
        //
        // Vector<LoadInfo> loadInfoVec = new Vector<LoadInfo>();
        // for(LoadInfo_thrift r : o.loadVector){
        // 	loadInfoVec.add(new LoadInfo(r));
        // }

        // this.loadVector = loadInfoVec;
        this.conflictingTS = o.conflictingTS;
    }

}
