/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.validation;

import beehive.workpool.LoadInfo;

import java.util.*;


public class ValidationDispatcher extends Thread {
    private static final long DELAY = 10;        //10 ms

    private LocalValidator localValidator;
    private ValidationService validationService;
    private LoadInfo currLoadInfo = null;
    private long lastDispatchTime = 0;

    public ValidationDispatcher(ValidationService validationService) {
        this.validationService = validationService;
    }

    void setLocalValidator(LocalValidator validator) {
        localValidator = validator;
    }

    public void run() {
        while (true) {
            // TODO: check if next batch is already ready or if time is elapsed since prev batch
            long timeSpent = System.currentTimeMillis() - lastDispatchTime;
            if (timeSpent < DELAY && !localValidator.isBatchReady()) {
                waitForEvent(timeSpent);
            }
            Queue<ValidationRequest> requestBatch = localValidator.fetchBatch();
            if (requestBatch.size() == 0) {
                continue;
            }
            lastDispatchTime = System.currentTimeMillis();
            // send batch to global validator
            try {
                Hashtable<String, ValidateResponse> responseTable = validationService.validateBatch(requestBatch, currLoadInfo);
                // process responses one by one
                for (ValidationRequest request : requestBatch) {
                    ValidateResponse response = responseTable.get(request.tid);
                    localValidator.processResponse(request.tid, response, request.rwSet);
                }
                localValidator.notifyWaitingThreads();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private synchronized void waitForEvent(long timeSpent) {
        try {
            wait(DELAY - timeSpent);
        } catch (Exception e) {
            e.printStackTrace();

        }
    }

    synchronized void flushBatch(LoadInfo loadInfo) {
        this.currLoadInfo = loadInfo;
        notify();
    }
}
