/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.validation;

import beehive.util.*;
import beehive.workpool.LoadInfo;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.*;

public interface ValidationService extends Remote {

    /* calls for optimistic (transaction-based) scheduling */

    /* get a start timestamp and transaction id */
    long getSTS() throws RemoteException;

    long reportCompletion(long commitTS) throws RemoteException;

    long reportCompletion(Vector<Long> commitTSList) throws RemoteException;

    /* validate the transaction, returns true if transaction has no conflict */
    /* RWSetInfo : structure representing read-write set info :-
                   Vector<nodeId: String> readSet;
                   Vector<nodeId: String> writeSet;
    */
    ValidateResponse validate(long startTS, RWSetInfo rwSet, LoadInfo myLoad) throws RemoteException;

    Hashtable<String, ValidateResponse> validateBatch(Queue<ValidationRequest> requestBatch, LoadInfo myLoad) throws RemoteException;

    void checkPoint() throws RemoteException;
}

