/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.validationService;

import beehive.util.RWSetInfo;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Vector;

/**
 * Created by khand055 on 3/23/17.
 */
public interface PeerInterface extends Remote {

    //Called by peer server to Host 0 (Timestamp Manager running at Host 0)
    public long reportCompletion(long commitTS) throws RemoteException;

    //Called by peer server to Host 0 (Timestamp Manager running at Host 0)
    public long reportCompletion(Vector<Long> commitTSList) throws RemoteException;

    //Calls received from the coordinator
    public PeerResponse validate(long startTS, RWSetInfo rwSet, long transactionId) throws RemoteException;

    //Called by peer server to Host 0 (Timestamp Manager running at Host 0) to get commit timestamp
    public long getGTS() throws RemoteException;

    //Called by peer server to Host 0 (Timestamp Manager running at Host 0) to get stable timestamp
    public long getSTS() throws RemoteException;

    //Called by peer server to Host 0 (Timestamp Manager running at Host 0) to get stable timestamp and commit timestamp in single call
    public long[] getCtsSts() throws RemoteException;

    //Sent to each peer to release the lock and commit
    public PeerResponse commit(long startTS, long commitTS, long transactionId) throws RemoteException;

    //Sent to each peer to release the lock
    public PeerResponse abort(long startTS, long transactionId, boolean bReleaseLock) throws RemoteException;
}
