/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.worker;

import beehive.util.Node;
import beehive.util.NodeData;
import beehive.util.StorageSystem;
import beehive.util.TransactionInfo;

import java.util.Map;
import java.util.Set;

abstract class CommitStrategy {

    StorageSystem dataStorage;

    CommitStrategy(StorageSystem dataStorage) {
        this.dataStorage = dataStorage;
    }

    /**
     * Commits the changes in the write sets for global visibility
     *
     * @param t metadata of the responsible transaction/task for this commit
     * @param writeSet Nodes that have been updated during the transaction
     * @param writeNDSet NodeDatas that have been updated during the transaction
     * @param readSet NodeIDs and their respective timestamps in which they were read during the transaction
     * @return true if the transaction was successfully committed, false otherwise
     */
    abstract boolean commit(TransactionInfo t, Set<Node> writeSet, Set<NodeData> writeNDSet, Map<String, Long> readSet);
}
