/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.worker;

import beehive.util.*;
import org.apache.thrift.TException;

import java.rmi.RemoteException;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

/**
 * Commits transactions that have been validated by a global validation service.
 */
public class ValidatingCommit extends CommitStrategy {

    public ValidatingCommit(StorageSystem dataStorage) {
        super(dataStorage);
    }

    @Override
    boolean commit(TransactionInfo t, Set<Node> writeSet, Set<NodeData> writeNDSet, Map<String, Long> readSet) {
        boolean commit = false;
        if (!writeSet.isEmpty()) {
            try {
                //update the TS of every node
                for (Node n : writeSet) {
                    n.TS = t.commitTS;
                }
                if (writeSet.size() == 1) {
                    Node node = writeSet.iterator().next();
                    commit = dataStorage.putNode(node.nodeId, node);
                } else {
                    dataStorage.putNodesParallel(new Vector<>(writeSet));
                    commit = true;
                }
            } catch (RemoteException e) {
                e.printStackTrace();
                commit = false;
            }
        }
        if (!writeNDSet.isEmpty()) {
            try {
                for (NodeData n : writeNDSet) {
                    n.TS = t.commitTS;
                }
                if (writeNDSet.size() == 1) {
                    NodeData nodeData = writeNDSet.iterator().next();
                    commit = dataStorage.updateNodeProperties(nodeData);
                } else {
                    dataStorage.updateNodePropertiesParallel(new Vector<>(writeNDSet));
                    commit = true;
                }
            } catch (RemoteException | TException e) {
                e.printStackTrace();
                commit = false;
            }
        }
        return commit;
    }
}
