/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.worker;

import beehive.workpool.*;
import beehive.util.*;

import java.io.*;
import java.util.*;

public class WorkerStatus implements Serializable {
    public static final int IDLE = 0;
    public static final int TASK_PICKED = 1;
    public static final int COMPUTE = 2;
    public static final int TASK_VALIDATION = 3;
    public static final int UPDATE = 4;
    public static final int REPORT_COMPLETION = 5;
    public static final int TASK_COMPLETED = 6;

    public static HashMap<Integer, String> statusDesc;
    public int workerId;

    static {
        statusDesc = new HashMap<Integer, String>();
        statusDesc.put(0, "IDLE");
        statusDesc.put(1, "TASK_PICKED");
        statusDesc.put(2, "COMPUTE");
        statusDesc.put(3, "TASK_VALIDATION");
        statusDesc.put(4, "UPDATE");
        statusDesc.put(5, "REPORT_COMPLETION");
        statusDesc.put(6, "TASK_COMPLETED");
    }

    public int status;        // current status
    public TransactionInfo currentTxnInfo;    //pointer to the txn infor for the task currently being executed
    public String taskId = "";

    public WorkerStatus(int workerId) {
        this.workerId = workerId;
        this.status = IDLE;
    }

    public WorkerStatus(int status, TransactionInfo txnInfo) {
        this.status = status;
        this.currentTxnInfo = txnInfo;
    }

    public String toString() {
        String str = "status=" + statusDesc.get(status) + " taskid=" + taskId;
        if (currentTxnInfo != null)
            str = str + " txninfo=" + currentTxnInfo.toString();

        return str;
    }
}
