/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.workpool;

import java.io.*;
import java.util.HashSet;

public class LoadInfo implements Externalizable, Comparable {
	public int load;
	public boolean tsMatch;
	public String hostname;
	public LoadInfo(int load, String myHost){
		this.load = load;
		this.hostname = myHost;
	}
	public LoadInfo(){
		this.load = 0;
		this.hostname = new String();
		this.tsMatch = false;
	}

	public int compareTo(Object obj){
		LoadInfo ldobj = (LoadInfo) obj;
		if(this.load>ldobj.load)
			
			return 1;
		else if(this.load<ldobj.load)
			return -1;
		else 
			return 0;
		
	}
	public String toString(){
		return "Host: "+hostname+" Load: "+load;
	}
	
	@Override  
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {  
		this.load = in.readInt();
		this.tsMatch = in.readBoolean();
		this.hostname = (String) in.readObject();
	}  

	@Override  
	public void writeExternal(ObjectOutput out) throws IOException {  
		out.writeInt(this.load);
		out.writeBoolean(this.tsMatch);
		out.writeObject(this.hostname);
	}

}
