/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package beehive.workpool;

import java.util.LinkedList;


public class TimestampManager {  
	
	long gts = 0;	//GTS
	long sts = 0;	//STS
	
	LinkedList<Long> incompleteTS;	// the list of ts whose completion is not yet reported.
	
	public TimestampManager() {
		incompleteTS = new LinkedList<Long>();
	}

	public TimestampManager(long startTS) {
		this();
		this.gts = startTS;
		this.sts = startTS;
	}

	public synchronized long getGTS(){		
		gts = gts+1;		
		incompleteTS.add(new Long(gts));
		//System.out.println("issued ts "+timestamp);
		return gts;
	}

	public long getSTS() {
		return sts;
	}
	
	// 
	public synchronized void reportDone(long ts){		
		// remove this ts from the list uncmtTS
		incompleteTS.remove(new Long(ts));	
		
		// if the one that is completed is the immediately next to the current STS
		// then avance STS
		if(ts == sts+1){
			sts = ts;
		}

		// see if we can advance STS further if the some of the subsequence transaction
		//  have been completed
		if(incompleteTS.size() > 0 ){
			long first = incompleteTS.getFirst();
			if(first-1 > sts)
				sts = first-1;			
		} else { // no more incomplete transaction 
			// all transactions for which gts was given are committed 
			sts = gts;
		}		
		
		//System.out.println("updated STS to "+sts);	
	}
	
}




