/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.APSP;

import beehive.util.*;

import java.util.*;
import java.io.*;

public class APSPNode extends Node {

    public long[] dist;
    private long numNodes;

    public APSPNode() {
    }

    public APSPNode(int id, int numNodes) {
        super(Integer.toString(id));
        this.dist = new long[numNodes];
        this.numNodes = numNodes;

        for (int i = 0; i < numNodes; i++) {

            this.dist[i] = Integer.MAX_VALUE;
        }

        this.dist[id] = 0;
    }

    @Override
    public String toString() {
        return "APSPNode{" +
                "nodeId='" + nodeId + '\'' +
                ", TS=" + TS +
                ", dist=" + Arrays.toString(dist) +
                ", numNodes=" + numNodes +
                '}';
    }

    public String printNodeEdges() {
        StringBuffer str = new StringBuffer();


        str.append("Node id : " + nodeId + " New dist = ");
        for (int i = 0; i < this.numNodes; i++) {
            str.append(i + ":" + this.dist[i] + " ");
        }

        str.append(" original graph: ");
        Enumeration edges = neighbors.keys();
        while (edges.hasMoreElements()) {
            String key = (String) edges.nextElement();
            APSPEdge e = (APSPEdge) neighbors.get(key);
            str.append(e.toString() + " ");
        }
        return str.toString();
    }

    @Override
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal(in);
        dist = (long[]) in.readObject();
        numNodes = in.readLong();
    }

    @Override
    public void writeExternal(ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(dist);
        out.writeLong(numNodes);
    }
}
