/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.APSP;

import TestProgs.util.BeehiveAppLoader;
import beehive.server.HashtableServer;
import beehive.util.Config;
import beehive.util.StorageSystem;

import java.io.IOException;
import java.util.HashMap;

public class APSPTest extends BeehiveAppLoader {

    private static String fileLoader = "Distributed";

    public static void main(String args[]) throws Exception {

        System.out.println("APSP Program  started");

        if (args.length != 6) {
            throw new RuntimeException("Syntax: AllPairsShortestPathTest <machine list> <config params file> <workpoolserver hostname>  <input filename> <num_nodes> <Exp Desc File>");
        }
        String machineList = args[0];

        //Initializing Config Parameters
        String configParamsFile = args[1];
        Config configParams = new Config(configParamsFile);
        if (configParams.has("Server.FileLoader")) {
            fileLoader = configParams.getStringValue("Server.FileLoader");
        }
        HashMap<Byte, String> nodeClassNameMap = new HashMap<Byte, String>();
        HashMap<Byte, Class> nodeClassMap = new HashMap<Byte, Class>();
        nodeClassNameMap.put((byte) 1, APSPNode.class.getName());
        nodeClassMap.put((byte) 1, APSPNode.class);


        APSPTest loader = new APSPTest();
        loader.init(args, "APSP", nodeClassNameMap, nodeClassMap);

        int numThreads = configParams.getIntValue("Worker.WORKER_COUNT");
        System.out.println("Number of workers:" + numThreads);

        Thread.sleep(5000);
        boolean master_thread = loader.isMaster();
        for (int i = 0; i < numThreads; i++) {
            if (i > 0) {
                master_thread = false;
            }
            APSPWorker worker = new APSPWorker(machineList, loader.getWorkpool(), master_thread, loader.getDataStorage(), i, loader.getNumNodes(), loader.getLocation());

            Thread workerThread = new Thread(worker);
            workerThread.start();
        }
    }

    @Override
    public void loadGraphData(String input_file, HashtableServer server, int numNodes, StorageSystem dataStorage) {
        InputFileReader reader = new InputFileReader(input_file, numNodes, dataStorage);
        try {
            reader.readAndLoadData(numNodes);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
