/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ConnectedComponent;

import beehive.util.*;
import java.util.*;

import java.io.*;

public class CCNode extends Node implements Externalizable  {
	int label;
	HashSet<String> history = new HashSet<String>();
	
	public HashSet<String> removedNeighbors = new HashSet<String>();
	public HashMap<String, Vector<String>> labelClashWithNbr = new HashMap<String, Vector<String>>();

	public CCNode(){
		super();
		label = -1;
	}

	public CCNode(String id) {
		super( id );
		label = -1;
	}

	public CCNode(CCNode n) {
		super(n);
		this.label = n.label;
		this.history = new HashSet<String>(n.history);
	}	

	public String printNode () {
		StringBuffer str =  new StringBuffer();
		str.append("Node id: "+ nodeId +"=" + " label "+label);
		return  str.toString();
	}

	public String toString() {
		return printNode();
	}

	public String printNodeEdges () {
		StringBuffer str =  new StringBuffer();
		str.append("Node id: "+ nodeId +"=" + " label: "+label);
		Enumeration edges = neighbors.keys();
		while (edges.hasMoreElements()) {
			String key = (String) edges.nextElement();
			Edge e = (Edge) neighbors.get(key);
			str.append(e.toString()+" ");
		}
		return  str.toString();
	}

	public HashSet<String> getHistory() {
		return history;
	}

	public void setHistory(HashSet<String> history) {
		this.history = history;
	}

	public HashSet<String> getRemovedNeighbors() {
		return removedNeighbors;
	}

	public void setRemovedNeighbors(HashSet<String> removedNeighbors) {
		this.removedNeighbors = removedNeighbors;
	}

	public HashMap<String, Vector<String>> getLabelClashWithNbr() {
		return labelClashWithNbr;
	}

	public void setLabelClashWithNbr(HashMap<String, Vector<String>> labelClashWithNbr) {
		this.labelClashWithNbr = labelClashWithNbr;
	}

	public Integer getLabel() {
		return label;
	}

	public void setLabel(int c) {
		this.label = c;
	}

	@Override  
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {  
		super.readExternal(in);
		this.label = in.readInt();
		/*String[] keys = (String[]) in.readObject();
		for(int i=0;i<keys.length;i++){
			this.history.add(keys[i]);
		}*/
		this.history = (HashSet<String>) in.readObject();
	}  

	@Override  
	public void writeExternal(ObjectOutput out) throws IOException {  
		super.writeExternal(out);
		out.writeInt(this.label);
		
		/*String[] keys = new String[this.history.size()];
		int i=0;
		for(String s: this.history){
			keys[i] = s;
			i++;
		}
		out.writeObject(keys);*/
		out.writeObject(this.history);
	}

	public NodeData toNodeData(Map map, byte classNameID){
		//System.out.println("----> this :: "+this.nodeId+", keys in map : "+map.keySet().toString());
		NodeData nd = new NodeData(this.nodeId, this.TS, classNameID);
		Iterator it = map.keySet().iterator();
		while(it.hasNext()){
			String key = (String) it.next();
			switch (key){
			case "label": nd.properties.put("label", this.label);
			break;
			case "history": nd.properties.put("history", this.history);
			break;
			default : System.out.println("Error (CCNode.toNodeData()) :: No string matched!!!");
			break;
			}
		}		
		return nd;
	}
}