/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ConnectedComponent;

/***
 * Author: Rahul R Sharma
 * Created : 11/26/2016
 */
import beehive.util.*;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

public class CCTask extends Task{
	public int label;
	public String srcNodeId;
	
	public String getSrcNodeId() {
		return srcNodeId;
	}

	public void setSrcNodeId(String srcNodeId) {
		this.srcNodeId = srcNodeId;
	}

	public int getLabel() {
		return label;
	}

	public void setLabel(int label) {
		this.label = label;
	}

	public CCTask() {
		super();
		label = -1;
		srcNodeId = "-1";
	}
	
	public CCTask(String targetNodeId, String sourceNodeId) {
		super(targetNodeId);
		label = -1;
		srcNodeId = sourceNodeId;
	}
	
	@Override
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
		super.readExternal(in);
    	this.label = in.readInt();
		this.srcNodeId = (String) in.readObject();
	}

	@Override
	public void writeExternal(ObjectOutput out) throws IOException {
		super.writeExternal(out);
		out.writeInt(this.label);
		out.writeObject(this.srcNodeId);
	}
}
