/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ConnectedComponent;

import  beehive.server.*;
import  beehive.util.*;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

import TestProgs.util.BeehiveAppLoader;

public class CCTest extends BeehiveAppLoader {
	public static void main (String args[]) throws Exception {
		if (args.length != 6) {
			throw new RuntimeException ("Syntax: java CCTest <hostlist_file> <config_params_file> <workpoolserver_hostname> <graph_input_filename> <num_nodes> <MsgFile>");
		}
		//All inherited Node classes used
		HashMap<Byte, String> nodeClassNameMap = new HashMap<Byte, String>();
		HashMap<Byte, Class> nodeClassMap = new HashMap<Byte, Class>();
		nodeClassNameMap.put((byte) 1, new CCNode().getClass().getName());
		nodeClassMap.put((byte) 1, new CCNode().getClass());
		nodeClassNameMap.put((byte) 2, new MetaDataNode().getClass().getName());
		nodeClassMap.put((byte) 2, new MetaDataNode().getClass());
		
		CCTest loader = new CCTest();
		loader.init(args, "CCTest", nodeClassNameMap, nodeClassMap);
		
		boolean master_thread = loader.isMaster();
		HashSet<String> unlabeledNodes = new HashSet<String>();
		ConcurrentHashMap<String, HashSet<String>> labeledNodes = new ConcurrentHashMap<String, HashSet<String>>();
		for (int i=0; i< loader.getNumThreads(); i++) {
			if (i> 0) {	
				master_thread = false;
			}
			
			CCWorkerReflection worker = new CCWorkerReflection(loader.getHostlistFile(), loader.getWorkpool(), master_thread, loader.getDataStorage(), i, loader.getLocation(), loader.getNumNodes(), labeledNodes, unlabeledNodes);
			Thread workerThread = new Thread(worker);
			workerThread.start();
		}
	}
	
	public void loadGraphData(String input_file, HashtableServer server, int numNodes, StorageSystem  dataStorage) {
		try{
			InputFileReaderDistributed fileReader = new InputFileReaderDistributed(input_file, server);
			fileReader.readAndLoadData();
		}catch (Exception e) {
			e.printStackTrace();
		}
	}
}
