/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ConnectedComponent;

import java.util.*;
import java.io.*;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;

import beehive.util.*;

public class InputFileReader {

	StorageSystem storage;
	String fileName;	
	// public Vector<String> totalNodes;
	// public Vector<String> nodesCreated;
	Hashtable<String, Node> table;
	boolean debug = false;

	public InputFileReader (String filename, int numNodes, StorageSystem storage) {
		this.fileName = filename;
		this.storage = storage;
        table = new Hashtable<String, Node>();
        for(int i=0; i< numNodes; i++) {
            CCNode n = new CCNode(""+i);
            String nodeKey  =  Integer.toString(i);
            table.put(nodeKey, n);
        }
	}		

	public void readAndLoadData() throws IOException {
		
		BufferedReader br = new BufferedReader(new FileReader(fileName));
		String line = null;

		int ctr = 0;
		while( (line = br.readLine()) != null) {
		  ctr++;
		  if (ctr % 1000==0) {
			System.out.println(new Date()+" total nodes processed:"+ctr);
		  }
			StringTokenizer tok = new StringTokenizer(line," ");
			String currentNodeKey = tok.nextToken();  
		    /*
			nodesCreated.add(currentNodeKey);	
			if (!totalNodes.contains(currentNodeKey)) {
				totalNodes.add(currentNodeKey);	
			}*/
				
			CCNode currentNode  = new CCNode(currentNodeKey);            
			if (table.containsKey(currentNodeKey)) {
				currentNode = (CCNode)table.get(currentNodeKey);

            }
			//update the neighbor list for this node;
			String nbrList = "";
			if (tok.hasMoreTokens()){ 
				nbrList = tok.nextToken();
			} else
				continue;
			
			StringTokenizer tok2 = new StringTokenizer(nbrList,",");
			// loop over the list of neighbours
			while(tok2.hasMoreTokens()) {
				String neighborInfo = tok2.nextToken();
				StringTokenizer tok3 = new StringTokenizer(neighborInfo,":");
				int nbrId = Integer.parseInt(tok3.nextToken());  /* Neighbor Id */
                		
				String nbrKey = Integer.toString( nbrId );
				/*
				if (!totalNodes.contains(currentNodeKey)) { 
					totalNodes.add(currentNodeKey);	
				}
				*/	
				
				int weight = Integer.parseInt(tok3.nextToken());
	            CCEdge toNbr =  new CCEdge();
	            //toNbr.weight = weight; 
				currentNode.addNeighbor(nbrKey, toNbr);

				/* If the neighbor alread has an edge for the current node */				
				CCNode neighborNode = (CCNode) table.get(nbrKey);
				
                CCEdge fromNbr =  (CCEdge) neighborNode.neighbors.get(currentNodeKey) ;
                if (fromNbr == null) {
				  if (debug) 	
	                  System.out.println("No edge was found from" + neighborNode.nodeId + " to " + currentNodeKey);
                  fromNbr =  new CCEdge();
				  //fromNbr.weight = weight;
                  neighborNode.addNeighbor(currentNodeKey, fromNbr);
                  table.put(nbrKey, neighborNode);
                }               
				
			}
            table.put(currentNodeKey, currentNode);				
		    if (debug) 	
			  System.out.println("Added node: "+ currentNodeKey);
			
		}

	System.out.println(new Date()+ " All nodes processed total nodes:"+ ctr);
	ctr=0;
        Enumeration<String> enumKeys = table.keys();
		Vector<Node> batch = new Vector<Node>();	
		while (enumKeys.hasMoreElements()) { 
 			String key = enumKeys.nextElement();
			CCNode node = (CCNode)table.get(key);
			batch.add(node);
			if (batch.size() == 1000) { 
				storage.putNodesParallel(batch);
				batch = new Vector<Node>();
			}	
		    ctr++;
		    if (ctr % 1000==0) {
			  System.out.println(new Date()+" total nodes put:"+ctr);
		    }
        }
		if (batch.size() > 0) {
			storage.putNodesParallel(batch);			
		}
		 System.out.println(new Date()+" total nodes put:"+ctr);
	}

    

	
}

