/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ConnectedComponent;

import beehive.util.*;
import java.util.*;

import java.io.*;

public class MetaDataNode  extends Node implements Externalizable  {
    public HashSet<String> set;
    public HashMap<String, HashSet<String>> map;
    public HashMap<String, Integer> countmap;
	
	public MetaDataNode(){
		super();
		set = new HashSet<String>();
		map = new HashMap<String, HashSet<String>>();
		countmap = new HashMap<String, Integer>();
	}
	
	public MetaDataNode(String nodeid){
		super(nodeid);
		set = new HashSet<String>();
		map = new HashMap<String, HashSet<String>>();
		countmap = new HashMap<String, Integer>();
	}

	public MetaDataNode(String nodeid, HashSet hs){
		super(nodeid);
		set = new HashSet<String>(hs);
		map = new HashMap<String, HashSet<String>>();
		countmap = new HashMap<String, Integer>();
	}
	
	public MetaDataNode(String nodeid, Vector<String> vec){
		super(nodeid);
		set = new HashSet(vec);
		map = new HashMap<String, HashSet<String>>();
		countmap = new HashMap<String, Integer>();
	}
	
	public MetaDataNode(String nodeid, HashMap<String, HashSet<String>> hmap){
		super(nodeid);
		set = new HashSet();
		map = new HashMap<String, HashSet<String>>(hmap);
		countmap = new HashMap<String, Integer>();
	}	

	public synchronized void addToMap(String labelNodeIdCommaSeparatedPair){
		String[] str = labelNodeIdCommaSeparatedPair.split(",");
		addToMap(str[0], str[1]);
	}
	
	public synchronized void addToMap(String label, String nodeId){
		HashSet<String> vec = map.get(label);
		if(vec==null)
			vec = new HashSet<String>();
		vec.add(nodeId);
		map.put(label, vec);
	}
	
	public synchronized void addToCountMap(String labelClashEntry){
		Integer val = countmap.get(labelClashEntry);
		if(val==null){
			val = new Integer(0);
		}
		val=new Integer(val.intValue()+1);
		countmap.put(labelClashEntry, val);
	}
	
	public synchronized void decrementFromCountMap(String labelClashEntry){
		Integer val = countmap.get(labelClashEntry);
		if(val==null){
			val = new Integer(0);
		}
		val=new Integer(val.intValue()-1);
		countmap.put(labelClashEntry, val);
	}
	
	public synchronized void clearMap(){
		map.clear();
	}
	
	public synchronized void clearCountMap(){
		countmap.clear();
	}
	
	public synchronized void clearSet(){
		set.clear();
	}
	
	public synchronized boolean isSetEmpty(){
		return set.isEmpty();
	}
	
	public synchronized void addToSet(String str){
		set.add(str);
	}
	
	public synchronized void addAllToSet(Set s){
		set.addAll(s);
	}
	
	public synchronized void removeFromSet(String str){
		set.remove(str);
	}

	public synchronized void removeAllFromSet(HashSet s){
		set.removeAll(s);
	}

	public synchronized String printNode () {
        StringBuffer str =  new StringBuffer();
        str.append("Node id: "+ nodeId +"=" + " set "+set.toString());
        return  str.toString();
    }

    public synchronized String toString() {
    	return printNode();
    }

    @Override  
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {  
    	super.readExternal(in);
    	this.set = (HashSet) in.readObject();
    	//this.map = (HashMap<String, HashSet<String>>) in.readObject();
    	//this.countmap = (HashMap<String, Integer>) in.readObject();
    	
	}  

	@Override  
	public void writeExternal(ObjectOutput out) throws IOException {  
		super.writeExternal(out);
		out.writeObject(this.set);
		//out.writeObject(this.map);
		//out.writeObject(this.countmap);
	}
}