/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ConnectedComponent;

/***
 * Author: Rahul R Sharma
 * Created : 11/26/2016
 */
import beehive.util.*;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.Serializable;
import java.util.HashSet;

public class RecomputeTask extends Task{
	String deletedEdgeSeedNodeId;
	String deletedEdgeOtherNodeId;	//the neighbor node of the seed node	
	String label;
	String oldLabel;
	String senderId;
	HashSet<String> history;
	HashSet<String> remotelabeledNodes;
	
	public RecomputeTask() {
		super();
		this.deletedEdgeSeedNodeId = "-1";
		this.deletedEdgeOtherNodeId = "-1";
		this.label = null;
		this.oldLabel = null;
		this.senderId = null;
		this.history = new HashSet<String>();
		this.remotelabeledNodes = new HashSet<String>();
	}
	
	public RecomputeTask(String targetNodeId,String nodeId, String nodeId2, String label, String oldLabel, HashSet<String> history, HashSet<String> remotelabeledNodes, String senderId) {
		super(targetNodeId);
		this.deletedEdgeSeedNodeId = nodeId;
		this.deletedEdgeOtherNodeId = nodeId2;
		this.label = label;
		this.oldLabel = oldLabel;
		this.senderId = senderId;
		this.history = history;
		this.remotelabeledNodes = remotelabeledNodes;
	}
	
	@Override
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
		super.readExternal(in);
		this.deletedEdgeSeedNodeId = (String) in.readObject();
		this.deletedEdgeOtherNodeId = (String) in.readObject();
		this.label = (String) in.readObject();
		this.oldLabel = (String) in.readObject();
		this.senderId = (String) in.readObject();
		this.history = (HashSet) in.readObject();
		this.remotelabeledNodes = (HashSet) in.readObject();
	}

	@Override
	public void writeExternal(ObjectOutput out) throws IOException {
		super.writeExternal(out);
		out.writeObject(this.deletedEdgeSeedNodeId);
		out.writeObject(this.deletedEdgeOtherNodeId);
		out.writeObject(this.label);
		out.writeObject(this.oldLabel);
		out.writeObject(this.senderId);
		out.writeObject(this.history);
		out.writeObject(this.remotelabeledNodes);
	}
}
