//java -Xms4096m -Xmx18432m -XX:+UseG1GC  TestProgs.GraphColoring.ClientTest validation-nodes configFile

package TestProgs.GraphColoring;

import beehive.util.Config;
import beehive.util.Logger;
import beehive.util.RWSetInfo;
import beehive.util.StatContainer;
import beehive.workpool.LoadInfo;
import beehive.validation.ValidateResponse;
import beehive.validation.ValidationService;

import java.io.*;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;
import java.util.*;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicLong;

import org.apache.thrift.protocol.TCompactProtocol;
import org.apache.thrift.protocol.TProtocol;
import org.apache.thrift.transport.TSocket;
import org.apache.thrift.transport.TTransport;

import beehive.thrift.*;
import org.apache.thrift.transport.TTransportException;

/**
 * Created by maste on 3/25/2017.
 */
public class ClientTest extends Thread {

    final static AtomicLong totalAborts = new AtomicLong();
    final static AtomicLong totalCommits = new AtomicLong();
    public static LinkedBlockingQueue<Long> pendingReportCompletionQueue;

    //partitioned graph nodes location table
    public static int[] partitionedGraphHostNodesTable;


    //Configuration Parameters
    public static int iterationCount;
    public static int nodeCount;
    public static int threadIncrementCount;
    public static int initialThreadCount;
    public static int rwSetLengthCount;
    public static int acrossCluster;
    public static int threadExecTime; //millisec for throughput
    public static int threadCountUpperLimit;
    public static int clientProgramId;
    static public int clientPort;

    static public int thriftConnectionPoolSize = 20;

    static public Map<Integer, LinkedBlockingQueue<ValidationService_thrift.Client>> serviceThreadPools;

    public static Map<Integer, ValidationService> certifierHandleTable; //HostId to certifier handle mapping
    public static Map<Integer, StatContainer> threadStatContainer;
    public static Config configParams=null;
    public static Map<Integer, ArrayList<Long>> keyRangeServerMap;

    private static PrintWriter writer;

    public static Map<Long, Integer> throughputTracker;

    public void setStartTS(long startTS) {
        if( startTS > this.startTS) {
            this.startTS = startTS;
        }
    }

    public long startTS = 0;
    public static Logger logger;
    public int clientId;


    public static String rpcMechanism;

    public String thriftHostname;
    public static Map<Integer, String> serverGroup = new HashMap<>();


    private List<ValidationService_thrift.Client> client;

    private List<TTransport> transport;

    private List<TProtocol> protocol;

    public int serverId;


    public ClientTest(int clientId) throws TTransportException {
        this.clientId = clientId;
    }


    @Override
    public void run() {


        //Creating Stats container
        StatContainer statContainer = new StatContainer();
        threadStatContainer.put(clientId, statContainer);

        //Selecting random server
        List<Integer> keysAsArray = new ArrayList<Integer>(certifierHandleTable.keySet());
        Random r = new Random();


        ValidateResponse validateResponse;
        Vector<Long> reportCompletionTS = new Vector<Long>();
        List<Long> reportCompletionTS_Thrift = new ArrayList<Long>();
        long startTimestamp = System.currentTimeMillis();
        long i=1;
        long decider=1;
        while( System.currentTimeMillis() < startTimestamp + threadExecTime ){
            //this.serverId = clientProgramId;//keysAsArray.get(new Random().nextInt(keysAsArray.size()-2)+2);

        //for( int i=1; i<=iterationCount; i++){
            i++;
            decider++;
            long startTime = System.currentTimeMillis();
            this.serverId = keysAsArray.get(new Random().nextInt(keysAsArray.size()-2)+2);

            //Randomly get the length of read/write set
            //int rwSetLength = r.nextInt(100) + 1; // Random between 1 to 10
            int rwSetLength = rwSetLengthCount;
            int readLength = r.nextInt(rwSetLength) + 1;
            int writeLength = rwSetLength - readLength;

            //logger.logDebug(" rwSetLength : "+rwSetLength+" readLength : "+readLength+" writeLength "+writeLength);

            RWSetInfo rwSetInfo = new RWSetInfo();
                if (decider <= acrossCluster) {
                    while (readLength != 0) {
                        int randomeNode = r.nextInt(nodeCount);
                        if (rwSetInfo.readSet.contains(Integer.toString(randomeNode))) {
                            continue;
                        } else {
                            rwSetInfo.readSet.add(Integer.toString(randomeNode));
                            readLength--;
                        }
                    }

                    while (writeLength != 0) {
                        int randomeNode = r.nextInt(nodeCount);
                        if (rwSetInfo.writeSet.contains(Integer.toString(randomeNode))) {
                            continue;
                        } else {
                            rwSetInfo.writeSet.add(Integer.toString(randomeNode));
                            writeLength--;
                        }
                    }
                } else {
                    //int totalClusterSize = certifierHandleTable.size();
                    //int randomServer = r.nextInt(totalClusterSize);
                    long startId = keyRangeServerMap.get(serverId).get(0);
                    long endId = keyRangeServerMap.get(serverId).get(1);
                    int dataItem = (int) (endId - startId + 1);

                    while (readLength != 0) {
                        int randomeNode = (int) startId + r.nextInt(dataItem);
                        if (rwSetInfo.readSet.contains(Integer.toString(randomeNode))) {
                            continue;
                        } else {
                            rwSetInfo.readSet.add(Integer.toString(randomeNode));
                            readLength--;
                        }
                    }

                    while (writeLength != 0) {
                        int randomeNode = (int) startId + r.nextInt(dataItem);
                        if (rwSetInfo.writeSet.contains(Integer.toString(randomeNode))) {
                            continue;
                        } else {
                            rwSetInfo.writeSet.add(Integer.toString(randomeNode));
                            writeLength--;
                        }
                    }
                }



            if( decider >= 100){
                decider = 1;
            }
            //logger.logDebug(" rwSetLength : "+rwSetLength+" readLength : "+readLength+" writeLength "+writeLength);

            if (rpcMechanism.equals("RMI")) {
                ValidationService certifierHandle = (ValidationService) certifierHandleTable.get(serverId);
                try {

                    logger.logDebug("---------------------------------------------------------------------------");
                    logger.logDebug(" Thread ID : " + clientId + " Iteration Count : " + i + " STS " + startTS);
                    logger.logDebug(" RWSetLength : " + rwSetLength);
                    logger.logDebug(" ReadSet : " + rwSetInfo.readSet.toString());
                    logger.logDebug(" WriteSet : " + rwSetInfo.writeSet.toString());

                    LoadInfo loadInfo = new LoadInfo(); //Dummy Object for backward compatibility
                    validateResponse = certifierHandle.validate(startTS, rwSetInfo, loadInfo);
                    logger.logDebug("Response CLient Id: " + clientId + " iteration " + i);
                    //                if( validateResponse.latestSTS < startTS ){
                    //                    System.out.println("Problem: Currrent: "+startTS + " Obtained :"+validateResponse.latestSTS);
                    //                }
                    setStartTS(validateResponse.latestSTS);// startTS = validateResponse.latestSTS;
                    //System.out.println(startTS);
                    statContainer.add(System.currentTimeMillis() - startTime);

                    if (validateResponse.commit) {
                        totalCommits.incrementAndGet();
                        //ps.print(" "+validateResponse.commitTS);
                        //Write Data to the Storage
                        //Thread.sleep(100);
                        logger.logDebug(" Status : COMMIT    CTS : " + validateResponse.commitTS);
                        //reportCompletionTS.clear();
                        //if( clientId == 1) {
                        //pendingReportCompletionQueue.add(validateResponse.commitTS);
                        reportCompletionTS.add(validateResponse.commitTS);
                        //}
                        if (reportCompletionTS.size() >= 1) {

                            long sts = certifierHandleTable.get(0).reportCompletion(reportCompletionTS);
                            setStartTS(sts);
                            reportCompletionTS.clear();
                        }

                        //startTS = certifierHandleTable.get(0).reportCompletion(reportCompletionTS);
                    } else {
                        //psa.print(" " + validateResponse.commitTS);
                        totalAborts.incrementAndGet();
                        logger.logDebug(" Status : ABORT");

                    }
                    //System.out.println(startTS);

                } catch (RemoteException e) {
                    e.printStackTrace();
                    //e.printStackTrace(ps);
                }
            }

            else {

//                      try{
//                          connectionLock.lock();
//                            System.out.println("clientId: "+clientId + " " + client1.ping());
//                          connectionLock.unlock();
//                      }catch (Exception e){
//                          e.printStackTrace();
//                      }


                    try {

                        //System.out.println("Calling ValidateResponse_thrift StartTS : " + startTS+ " ReadSet: " + rwSetInfo.readSet.toString() + " WriteSet: " + rwSetInfo.writeSet.toString());
                        ValidationService_thrift.Client client = serviceThreadPools.get(serverId).take();
                        ValidateResponse_thrift tempRes = client.validate_thrift(startTS, new RWSetInfo_thrift(rwSetInfo.readSet,rwSetInfo.writeSet,rwSetInfo.unusedSet));
                        serviceThreadPools.get(serverId).add(client);
                        //System.out.println("Receiving ValidateResponse_thrift StartTS : " + startTS);
                        validateResponse = new ValidateResponse(tempRes);
                         long tempstartst = startTS;
                        startTS = validateResponse.latestSTS;
                        //System.out.println(startTS);
                        statContainer.add(System.currentTimeMillis() - startTime);

                        if( validateResponse.commit ){
                            totalCommits.incrementAndGet();

                            //Write Data to the Storage
                            //Thread.sleep(100);
                            logger.logDebug(" Status : COMMIT    CTS : "+validateResponse.commitTS);
                            reportCompletionTS_Thrift.clear();
                            reportCompletionTS_Thrift.add(validateResponse.commitTS);

                            //System.out.println("Sending reportCompletionBatch_thrift StartTS : " + tempstartst);
                            ValidationService_thrift.Client stsClient = serviceThreadPools.get(0).take();
                            startTS = stsClient.reportCompletionBatch_thrift(reportCompletionTS_Thrift);
                            serviceThreadPools.get(0).add(stsClient);
                            //System.out.println("Receiving reportCompletionBatch_thrift StartTS : " + tempstartst);

                        } else {
                            totalAborts.incrementAndGet();
                            logger.logDebug(" Status : ABORT");
                        }

                        //System.out.println(startTS);
                    } catch (Exception e){
                        e.printStackTrace();
                    }



            }
        }
    }

    public static void main(String args[]) throws Exception{

        logger = new Logger();
        logger.setDebug(false);
        logger.setInfo(true);

        pendingReportCompletionQueue = new LinkedBlockingQueue<Long>();

        certifierHandleTable = new HashMap<Integer, ValidationService>();
        threadStatContainer = new ConcurrentHashMap<Integer, StatContainer>();
        configParams = new Config(args[1]);
        //nodeCount = configParams.getIntValue("Validator.nodeSize");
        nodeCount = configParams.getIntValue("Validator.nodeSize");
        //iterationCount = 1000;
        initialThreadCount = 1;
        threadIncrementCount = 0;
        rwSetLengthCount = configParams.getIntValue("Validator.rwSetLengthCount");
        acrossCluster = configParams.getIntValue("Validator.acrossCluster");
        thriftConnectionPoolSize = configParams.getIntValue("Validator.clientThriftConnectionPool");
        clientPort = configParams.getIntValue("Validator.thrift.clientPort");
        threadExecTime = 10000;
        threadCountUpperLimit = 40;
        clientProgramId = Integer.parseInt(args[2]);

        StringBuilder sb = new StringBuilder();
        sb.append("_MSI_").append(configParams.getIntValue("Validator.MSI"));
        sb.append("_Replica_").append(configParams.getIntValue("Validator.ServerReplica"));
        sb.append("_ClientId_").append(clientProgramId);
        sb.append("_RWSet_").append(rwSetLengthCount);
        sb.append("_Cross_").append(acrossCluster);
        sb.append("_NodeCount_").append(nodeCount);
        sb.append("_IniThrdCnt_").append(initialThreadCount);
        sb.append("_ThrdIncrCnt_").append(threadIncrementCount);
        sb.append("_thrdExcTm_").append(threadExecTime);
        sb.append("_thrdCntUprLmt_").append(threadCountUpperLimit);

        writer = new PrintWriter(new FileWriter(sb.toString()),true);
        keyRangeServerMap = new HashMap<Integer, ArrayList<Long>>();

        prepareCertifierHandle(args[0]);
        int threadCount = initialThreadCount;

        initKeyRangeServerMap(nodeCount);

        rpcMechanism = configParams.getStringValue("Validator.RPC_MECHANISM");
        serviceThreadPools = new ConcurrentHashMap<Integer, LinkedBlockingQueue<ValidationService_thrift.Client>>();

        initThriftServerHandle();

        //Response Time Test
        while( threadCount <= threadCountUpperLimit ){
            startTest(threadCount);
            threadCount += threadIncrementCount;
            Thread.sleep(1000);
        }

        //Throughput Test
        /*while( threadCount <= 50 ){
            startTest(threadCount);
            threadCount += threadIncrementCount;
            Thread.sleep(1000);
        }*/

        writer.close();
    }

    public static void initThriftServerHandle(){

        try {
            for (int i = 0; i < serverGroup.size(); i++) {

                TTransport transport;
                TProtocol protocol;
                ValidationService_thrift.Client client;

                //Trying to open connection to each peer
                System.out.println("Looking Up Peer : " + serverGroup.get(i) + ":" + (clientPort + i));
                while (true) {
                    try {
                        transport = new TSocket(serverGroup.get(i), clientPort + i);
                        transport.open();
                        transport.close();
                        System.out.println("Peer Up : " + serverGroup.get(i) + ":" + (clientPort + i));
                        break;
                    } catch (Exception e) {
                        try {
                            Thread.sleep(10);
                        } catch (Exception e1) {
                        }
                    }
                }

                LinkedBlockingQueue<ValidationService_thrift.Client> clientsPool = new LinkedBlockingQueue<ValidationService_thrift.Client>();

//                int poolSize = thriftConnectionPoolSize;
//                if( i == 0){
//                    poolSize = poolSize*(serverGroup.size()-2);
//                }
                for (int j = 0; j < thriftConnectionPoolSize; j++) {

                    transport = new TSocket(serverGroup.get(i), clientPort + i);
                    transport.open();
                    protocol = new TCompactProtocol(transport);
                    client = new ValidationService_thrift.Client(protocol);
                    clientsPool.add(client);
                }
                serviceThreadPools.put(i, clientsPool);
            }
        }catch (Exception e){
            e.printStackTrace();
        }

    }

    public static void startTest(int threadCount) throws InterruptedException, TTransportException {

        long startTimestamp = System.currentTimeMillis();

        ExecutorService executor = Executors.newCachedThreadPool();
        for (int i = 1; i <= threadCount; i++) {
            executor.execute(new ClientTest(i));
        }

        executor.shutdown();
        try {
            while (!executor.awaitTermination(11, TimeUnit.SECONDS)) {
                logger.logInfo("Awaiting completion of threads.");
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
            //e.printStackTrace(ps);
        }

//        logger.logInfo("---------------------------------------------------------------------------");
//        logger.logInfo("Total COMMIT :" + totalCommits.get());
//        logger.logInfo("Total ABORT :" + totalAborts.get());
//        logger.logInfo("Percentage of ABORT :" + (totalAborts.get()*100)/(totalAborts.get() + totalCommits.get())+"%");
//        logger.logInfo("Total Execution time (millisec) :" + (System.currentTimeMillis() - startTimestamp));
//        writer.println();
//        writer.print("Total COMMIT :" + totalCommits.get() + " \t\t Total ABORT :" + totalAborts.get() );
//        try {
//            writer.print(" \tPercentage of ABORT :" + (totalAborts.get() * 100) / (totalAborts.get() + totalCommits.get()) + "%");
//        }catch(Exception e){
//            // Divide by zero
//        }
        //writer.println("Total Execution time (millisec) :" + (System.currentTimeMillis() - startTimestamp));

        //logger.logInfo("--------------------------------KEY STATISTICS-------------------------------");

        StatContainer finalStatistic = new StatContainer();
        StatContainer threadStatHandle;

        for( int threadId : threadStatContainer.keySet()) {

            threadStatHandle = threadStatContainer.get(threadId);
            finalStatistic.add(threadStatHandle.getValues());

//            System.out.format("Thread ID : %d \t Response Time Stats(ms)::\tAVG : %7.3f \t\t MAX : %7.3f \t\t MIN : %7.3f",
//                    threadId, threadStatHandle.getMean(), threadStatHandle.getMax(), threadStatHandle.getMin());
//            System.out.println();

//            logger.logInfo("Thread ID : " + threadId + " Stats(ms)::  AVG : " + threadStatHandle.getMean() + " MAX : "
//                    + threadStatHandle.getMax() + " MIN : " + threadStatHandle.getMin());
        }

        //writer.println();
        System.out.println();
//      logger.logInfo("--------------------------------OVERALL STATISTICS-------------------------------");
        writer.printf("%7d\t%10d\t%10.3f\t%10.3f\t%10d\t%10d",threadCount, finalStatistic.getValues().size()/(threadExecTime/1000),
                finalStatistic.getMean(),finalStatistic.getMax(),totalCommits.get(),totalAborts.get());
        System.out.format("Client Id: %2d \t ThreadCount : %7d \t\t Response Time Stats(ms)::\tAVG : %10.3f \t\t MAX : %10.3f \t\t MIN : %10.3f" +
                        "\t\t Throughput (per sec) : %10d",clientProgramId,threadCount, finalStatistic.getMean(), finalStatistic.getMax(),
                finalStatistic.getMin(), finalStatistic.getValues().size()/(threadExecTime/1000));
//      System.out.println();
        writer.println();
       // ps.close();

        threadStatContainer.clear();
        totalAborts.set(0);
        totalCommits.set(0);
    }


    private static void prepareCertifierHandle(String configFile) throws IOException {

        BufferedReader bufferedReader = new BufferedReader(new FileReader(configFile));
        String line;
        String hostname;
        int hostId;
        int hostIdCounter = 0;
        //int i=0;
        while ((line = bufferedReader.readLine()) != null) {
            if( line.charAt(0) == '#') continue;
            String[] splited = line.split("\\s+");
            hostname = splited[0];
            hostId = hostIdCounter++;//Integer.parseInt(splited[1]);
            //serverList.add(hostId);
            String url  = "//" + hostname + ":" + Registry.REGISTRY_PORT  + "/ValidationService" + hostId;

            ValidationService certifierHandle = lookup(url);
            certifierHandleTable.put(hostId, certifierHandle);
            serverGroup.put(hostId, hostname);

        }
    }

    public static ValidationService lookup(String url) {
        System.out.println("Looking up certifier:"+url);
        while(true) {
            try {
                ValidationService certifierHandle = (ValidationService) Naming.lookup(url);
                System.out.println("certifier up");
                return certifierHandle;
            } catch(Exception e){
                try { Thread.sleep(10); } catch(Exception e1){}
            }
        }
    }


    private static void initKeyRangeServerMap(long numOfNodes){

        Config configParams = Config.getInstance();
        String loaderType = configParams.getStringValue("Server.FileLoader").trim();
        String hashingScheme = configParams.getStringValue("Server.NODE_HASHING_SCHEME").trim();
        if(loaderType.equals("Bucketized") && hashingScheme.equals("GRAPH_PARTITION")){
            initGraphNodeHostTable();
            setGraphPartitionedServerMap();
        } else {
            setKeyRangeServerMap(numOfNodes);
        }
    }

    private static void initGraphNodeHostTable(){

        Config configParams = Config.getInstance();
        List<Integer> hostArrList = new ArrayList<Integer>();

        try{
            String input_filename = configParams.getStringValue("Validator.clusteredGraph") + "/" + "metadata";
            BufferedReader input = new BufferedReader(new FileReader(input_filename));
            String line = "";
            int ctr = 0;

            while( (line = input.readLine()) != null ){
                if( ctr > 0 ) {
                    StringTokenizer lineTok = new StringTokenizer(line, " ");
                    String hostKey = lineTok.nextToken();
                    String fromNode = lineTok.nextToken();
                    String toNode = lineTok.nextToken();
                    int hostid = Integer.parseInt(hostKey);
                    int lastNodeStored = Integer.parseInt(toNode);
                    hostArrList.add(lastNodeStored);
                    //hostArr[hostid]=lastNodeStored;
                }
                ctr++;
            }
            input.close();
        } catch(Exception e){
            e.printStackTrace();
            System.err.println("ERROR :: Failed to read metadata file");
        }
        int hostArr[] = new int[hostArrList.size()];
        for( int i=0; i<hostArrList.size(); i++){
            hostArr[i] = hostArrList.get(i);
        }

        partitionedGraphHostNodesTable = hostArr;
    }

    private static void setGraphPartitionedServerMap(){

        int validationClusterSize = certifierHandleTable.size();
        validationClusterSize = validationClusterSize - 2; //Node 0 will be for timestamp manager STS and 1 for CTS

        int[] clusterPerServiceReplica = new int[validationClusterSize];
        int graphClusterSize = partitionedGraphHostNodesTable.length;
        int iterator = 0;

        while (graphClusterSize > 0){
            clusterPerServiceReplica[iterator]++;
            iterator = (iterator+1)%validationClusterSize;
            graphClusterSize--;
        }

        //Initialize range to -1 for all
        for( int i=2; i<certifierHandleTable.size(); i++ ){
            keyRangeServerMap.put(i, new ArrayList<Long>(Arrays.asList(-1L,-1L)));
        }

        int firstIndex = 0;
        int lastIndex = 0;
        int clusterIndex = 0;


        for( int i=0; i<certifierHandleTable.size()-2; i++ ){

            if( i < clusterPerServiceReplica.length) {
                clusterIndex = clusterIndex + clusterPerServiceReplica[i];
                lastIndex = partitionedGraphHostNodesTable[clusterIndex-1];
                keyRangeServerMap.put(i+2, new ArrayList<Long>(Arrays.asList((long) firstIndex, (long) lastIndex)));
                firstIndex = lastIndex + 1;
            }
        }

    }

    public static void setKeyRangeServerMap(long numOfNodes){

        int totalClusterSize = certifierHandleTable.size();

        totalClusterSize = totalClusterSize - 2; //Node 0 will be for timestamp manager STS and 1 for CTS

        long nodePerCluster = numOfNodes/totalClusterSize;
        long residue = numOfNodes%totalClusterSize; // shall be added to last Host

        long start = 0;
        long startIndex = 0;
        long endIndex = 0;

        //Initialize range to -1 fpr all
        for( int i=2; i<certifierHandleTable.size(); i++ ){
            keyRangeServerMap.put(i, new ArrayList<Long>(Arrays.asList(-1L,-1L)));
        }

        if( nodePerCluster > 0){ //atleast one node per cluster
            for( int i=2; i<certifierHandleTable.size(); i++ ){
                startIndex = start;
                endIndex = start + nodePerCluster - 1;
                if( i == certifierHandleTable.size()-1 ) {// last item
                    endIndex += residue;
                }
                keyRangeServerMap.put(i, new ArrayList<Long>(Arrays.asList(startIndex, endIndex)));
                start += nodePerCluster;
            }
        } else if (residue > 0) {
            keyRangeServerMap.put(2, new ArrayList<Long>(Arrays.asList(startIndex, endIndex))); //Put residue in first node
        }

    }

}
