package TestProgs.GraphColoring;

import  beehive.server.*;
import  beehive.util.*;

import java.util.*;

import TestProgs.util.BeehiveAppLoader;

public class GCTest extends BeehiveAppLoader {
	static String fileLoader = "Distributed";

	public static void main (String args[]) throws Exception {
		if (args.length != 6) {
			throw new RuntimeException ("Syntax: java GCTest <hostlist_file> <config_params_file> <workpoolserver_hostname> <graph_input_filename> <num_nodes> <MsgFile>");
		}

		String configParamsFile = args[1];
		Config configParams = new Config(configParamsFile);
		if(configParams.has("Server.FileLoader"))
			fileLoader=configParams.getStringValue("Server.FileLoader");

		//All inherited Node classes used
		HashMap<Byte, String> nodeClassNameMap = new HashMap<Byte, String>();
		HashMap<Byte, Class> nodeClassMap = new HashMap<Byte, Class>();
		nodeClassNameMap.put((byte) 1, new GCNode().getClass().getName());
		nodeClassMap.put((byte) 1, new GCNode().getClass());
		
		
		GCTest loader = new GCTest();
		loader.init(args, "GCTest", nodeClassNameMap, nodeClassMap);

		boolean master_thread = loader.isMaster();
		for (int i=0; i< loader.getNumThreads(); i++) {
			if (i> 0) {	
				master_thread = false;
			}
			GCWorker worker;
			//Create a worker with or without reflection properties
			if(loader.isUseNodeData()) {
				worker = new GCWorkerReflection(loader.getHostlistFile(), loader.getWorkpool(), master_thread, loader.getDataStorage(), i, loader.getLocation(), loader.getNumNodes());
			}else {
				worker = new GCWorker(loader.getHostlistFile(), loader.getWorkpool(), master_thread, loader.getDataStorage(), i, loader.getLocation(), loader.getNumNodes());
			}
			Thread workerThread = new Thread(worker);
			workerThread.start();
		}
	}

	public void loadGraphData(String input_file, HashtableServer server, int numNodes, StorageSystem  dataStorage) {
		try{
			if(fileLoader.equals("Distributed")){
				InputFileReaderDistributed fileReader = new InputFileReaderDistributed(input_file, server);
				fileReader.readAndLoadData();
			}
			else if(fileLoader.equals("Bucketized")){
				InputFileReaderBucketed fileReader = new InputFileReaderBucketed(input_file, server);
				fileReader.readAndLoadData();
			}
			else{//Parallel loader
				InputFileReaderParallel fileReader = new InputFileReaderParallel(input_file, numNodes, dataStorage);
				fileReader.readAndLoadData();
			}

		}catch (Exception e) {
			e.printStackTrace();
		}
	}
}
