package TestProgs.GraphColoring;

import beehive.util.*;
import beehive.validation.ValidateResponse;
import beehive.workpool.*;
import beehive.worker.*;

import java.util.*;
import java.io.*;
import java.nio.channels.*;
import java.lang.*;

public class GCWorker extends Worker implements Runnable {

    boolean debug = false;
    long ioStartTime = 0;
    int calls = 0;
    long ioTime = 0;
    String myStorageURL;
    int numNodes = 0;
    public static Object mstobjLock = new Object();
    //TODO : This is application specific, remove mstEdges references
    public static Vector<GCEdge> mstEdges = new Vector<GCEdge>();
    public Vector<String> mergedVertices;

    public GCWorker(String configFile, LocalWorkpoolImpl wpool, boolean master, StorageSystem datastorage, int workerId, String myStorageURL, int numOfNodes) throws java.rmi.RemoteException {
        super(configFile, wpool, workerId, datastorage, master);
        this.myStorageURL = myStorageURL;
        this.numNodes = numOfNodes;
    }

    public void compute(Task t, Node node, Vector<Node> neighborNodes) {
        try {
            GCNode u = new GCNode((GCNode) node);    //you need a copy of orig node
            if (u.marked) {
                return;
            }
            HashSet<Integer> neighborColors = new HashSet<Integer>();
            for (Node V : neighborNodes) {
                GCNode v = (GCNode) V;
                //	rwsetinfo.unusedSet.remove(Integer.toString(v.nodeId)); keeping unused set empty
                //read set should be minimized
                if (!v.marked) {
                    rwsetinfo.readSet.add(v.nodeId);
                    readsetTS.put(v.nodeId, v.TS);
                }
                if (v.marked) {
                    neighborColors.add(v.color);
                } else {

                }

            }
            Vector<Integer> uniqColors = new Vector<Integer>(neighborColors);
            Collections.sort(uniqColors);
            int targetColor = 1;

            for (Integer i : uniqColors) {

                if (i.intValue() > targetColor)
                    break;
                else if (i.intValue() == targetColor)
                    targetColor++;

            }
            u.marked = true;
            u.color = targetColor;
            updatedNodes.put(u.nodeId, u);
            rwsetinfo.writeSet.add(u.nodeId);

        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }


    }


    public void doTask(Task tsk) throws Exception {
        boolean commit = false;

        boolean valid = false;
        Vector<Node> verificationList;
        Node origNode = null;
        Vector<Node> neighborNodes = null;
        TransactionInfo txnInfo = null;
        while (!valid) {
            txnInfo = beginTransaction();
            tsk.txnInfo = txnInfo;
            origNode = dataStorage.getNode(tsk.getNodeId());
            neighborNodes = getNeighbors(origNode);
            verificationList = new Vector<Node>();
            verificationList.add(origNode);
            verificationList.addAll(neighborNodes);
            valid = earlyValidation(tsk, verificationList, null);
            if (!valid) {
                Thread.sleep(STS_DELAY);
            }
        }
        while (!commit) {

            try {
                compute(tsk, origNode, neighborNodes);

                if (rwsetinfo.writeSet.isEmpty() && rwsetinfo.readSet.isEmpty()) {
                    commit = true;
                    workpool.removeTask(tsk);
                    return; // nothing to do
                }

                ValidateResponse res = workpool.validate(tsk, rwsetinfo);
                commit = res.commit;
                if (commit) {
                    tsk.txnInfo.commitTS = res.commitTS;
                } else {
                    boolean firstTry = true;
                    valid = false;
                    while (!valid) {
                        txnInfo = beginTransaction();
                        tsk.txnInfo = txnInfo;
                        if (!res.localResponse && !res.speculativeAbort && firstTry) {
                            //update STS is with latest STS
                            tsk.txnInfo.startTS = res.latestSTS;
                            firstTry = false;
                        }
                        origNode = dataStorage.getNode(tsk.getNodeId());
                        neighborNodes = getNeighbors(origNode);
                        verificationList = new Vector<Node>();
                        verificationList.add(origNode);
                        verificationList.addAll(neighborNodes);
                        valid = earlyValidation(tsk, verificationList, null);
                        if (!valid) {
                            Thread.sleep(STS_DELAY);
                        }
                    }
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        } //Transaction committed
        commitTask(tsk);

    }


    public void cleanUp() {
        endTime = System.currentTimeMillis();
        long duration = endTime - startTime;

        System.out.println("*******************************");
        System.out.println("Time taken : " + duration);
        String systemStats = "";
        try {
            systemStats = dataStorage.getStats(true);
            FileOutputStream fout = new FileOutputStream("result.txt", true);
            FileLock lock = fout.getChannel().lock();
            PrintWriter out = new PrintWriter(fout);
            out.println("==================================");
            out.println("Time taken: " + duration + " msec");
            out.println("==================================");
            out.println(systemStats);
            out.println("==================================");
            out.flush();
            lock.release();
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

        System.out.println("Worker " + workerId + ":  Finished cleanup");

        System.out.println("*****************************");

        System.out.println(systemStats);
    }

    public void createTasksAllVertices(Task tsk) throws Exception {

        Vector<String> myLocalKeys = dataStorage.getAllKeys(myStorageURL);
        Vector<Task> taskVector = new Vector<Task>();
        int batch_size = 1000;
        for (String key : myLocalKeys) {
            if (key.equals(Integer.toString(numNodes)))
                continue;
            Task t = new Task(key);
            t.phase = "GENERIC";
            taskVector.add(t);
            if (taskVector.size() == batch_size) {
                workpool.addTask(taskVector);
                taskVector.clear();
            }
            //Thread.sleep(1);
        }
        if (taskVector.size() > 0) {
            workpool.addTask(taskVector);
        }
        workpool.removeTask(tsk);
    }


    @Override
    public void run() {
        startTime = System.currentTimeMillis();

        Task myTask;
        WorkStatus ws;

        boolean terminate = false;

        boolean verifyResult = false;
        if (configParams.has("GCWorker.VERIFY_RESULT"))
            verifyResult = configParams.getBoolValue("GCWorker.VERIFY_RESULT");

        if (master) {
            try {
                Task t = new Task();
                t.phase = "STARTUP";
                t.nodeId = "STARTUP_TASK";
                workpool.broadcast(t);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        String myURL = dataStorage.myURL;
        while (true) {
            try {
                ws = workpool.getTask();
                if (ws.status == StatusType.VALID_TASK) {
                    myTask = ws.task;

                    if (myTask == null) {
                        System.out.println("Worker " + workerId + ":  Error: Valid but Null task");
                        continue;
                    }
                    if (myTask.phase.equals("STARTUP")) {
                        if (master)
                            System.out.println("Worker " + workerId + ":  Startup task picked up by master itself");
                        else
                            System.out.println("Worker " + workerId + ":  Startup task picked up by a non master.");
                        createTasksAllVertices(myTask);
                        System.out.println("Worker " + workerId + ":  Created initial tasks");
                    } else if (myTask.phase.equals("GENERIC")) {
                        doTask(myTask);
                        tasksCreated.clear();
                    } else if (myTask.phase.equals("TERMINATION")) {
                        //Stop task

                        System.out.println("Worker " + workerId + ":  Termination task");
                        System.gc();
                        //before exiting, do the verification step
                        workpool.progressStatusThread.stopPrinting = true;
                        if (verifyResult) {
                            System.out.println("Worker " + workerId + ":  Calling verify()");
                            verify();
                        }
                        //System.exit(0);
                    } else {

                        System.out.println("Worker " + workerId + ":  Unknown phase");
                    }


                } else if (ws.status == StatusType.NO_READY_TASK) {
                    // Thread.sleep(configParams.getIntValue("Worker.THREADSLEEPTIME"));
                } else if (ws.status == StatusType.WORK_TERMINATED) {
                    if (master && !terminate) {
                        System.out.println("Worker " + workerId + ":  Master creating TERMINATE TASK - Worker thread proceeding to cleanUp");

                        cleanUp();

                        Task t = new Task();
                        t.nodeId = "TERMINATE_TASK";
                        t.phase = "TERMINATION";
                        terminate = true;
                        workpool.broadcast(t);

                    }
                    workpool.progressStatusThread.stopPrinting = true;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    public void verify() {
        try {
            //get all local nodes from the storage system
            Vector<String> localKeys = dataStorage.getAllKeys(myStorageURL);
            //for each node, get neighbors and verify solution
            Vector<Node> neighborNodes;
            Node n;
            boolean correct = true;
            int count = 0;
            int irregularNodes = 0;
            for (String key : localKeys) {
                n = dataStorage.getNode(key);
                neighborNodes = getNeighbors(n);
                boolean fault = false;
                for (Node nbr : neighborNodes) {
                    if (((GCNode) n).color == ((GCNode) nbr).color) {
                        correct = false;
                        count++;
                        fault = true;
                    }
                }
                if (fault == true) {
                    irregularNodes++;
                }
            }

            if (correct)
                System.out.println("All ok in Server : " + this.myStorageURL);
            else
                System.out.println("Incorrect solution in Server : " + this.myStorageURL);
            System.out.println("Number of irregularities found : " + count + ", irregular Nodes : " + irregularNodes);
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
