package TestProgs.GraphColoring;

import beehive.util.*;
import beehive.validation.ValidateResponse;
import beehive.workpool.*;

import java.util.*;
import java.lang.*;

public class GCWorkerReflection extends GCWorker implements Runnable {

	public GCWorkerReflection (String configFile, LocalWorkpoolImpl wpool, boolean master, StorageSystem datastorage, int workerId, String myStorageURL, int numOfNodes )  throws java.rmi.RemoteException {
		super(configFile, wpool, master, datastorage, workerId, myStorageURL, numOfNodes);
	}

	public void computeGC(Task t, Node node, Vector<Node> neighborNodes, Vector<NodeData> remoteNeighborNodesData) { 
		try{
			if(((GCNode)node).marked)
			{
				return;
			}

			GCNode u = new GCNode((GCNode)node);
			HashSet<Integer> neighborColors = new HashSet<Integer>();
			for(Node V: neighborNodes){
				if(!((GCNode)V).marked) {
					rwsetinfo.readSet.add(V.nodeId);
				}
				if(((GCNode)V).marked) {
					neighborColors.add(((GCNode)V).color);
				}
			}

			for(NodeData Vnd : remoteNeighborNodesData) {
				if(!((Boolean)Vnd.getProperty("marked")).booleanValue()) {
					rwsetinfo.readSet.add(Vnd.nodeId);
				}
				if(((Boolean)Vnd.getProperty("marked")).booleanValue()) {
					neighborColors.add((Integer)Vnd.getProperty("color"));	
				}
			}

			Vector<Integer> uniqColors = new Vector<Integer>(neighborColors);
			Collections.sort(uniqColors);
			int targetColor = 1;

			for(Integer i: uniqColors){

				if(i.intValue() > targetColor)
					break;
				else if ( i.intValue()==targetColor)
					targetColor++;

			}
			u.marked = true;
			u.color = targetColor;
			updatedNodes.put(u.nodeId, u);
			rwsetinfo.writeSet.add(u.nodeId);	
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}
	}

	@Override
	public void doTask(Task tsk) throws Exception
	{
		boolean commit = false;
		boolean valid = false;
		TransactionInfo txnInfo = null;
		Vector<Node> localNeighborNodes = null;
		Vector<NodeData> remoteNodesData = null;
		Node origNode = null;
		HashSet<String> nbrKeys = null;
		HashSet<String> alreadyFetchedKeys = null;
		Vector<Node> verificationList = null;

		while(!valid) {
			txnInfo = beginTransaction();
			tsk.txnInfo = txnInfo;
			origNode = dataStorage.getNode(tsk.getNodeId());
			//populate the remote and local neighbor list
			try {
				nbrKeys  = new HashSet<String>(getNeighborIds(origNode));
				//first get the local neighbour nodes (complete node)
				localNeighborNodes = getLocalNeighbors(origNode);
				alreadyFetchedKeys = new HashSet<String>();
				for(Node n : localNeighborNodes) {
					alreadyFetchedKeys.add(n.nodeId);
				}
				nbrKeys.removeAll(alreadyFetchedKeys);
				remoteNodesData = getRemoteNeighborsData(new Vector<String>(nbrKeys), origNode.getClass().getName());
				verificationList = new Vector<Node>();
				verificationList.add(origNode);
				verificationList.addAll(localNeighborNodes);

				valid = earlyValidation(tsk, verificationList, remoteNodesData);
				if(!valid) {
					Thread.sleep(STS_DELAY);
				}
			}catch (Exception e) {
				e.printStackTrace();
			}
		}

		while(!commit)
		{

			try{
				computeGC(tsk, origNode, localNeighborNodes, remoteNodesData);

				if(rwsetinfo.writeSet.isEmpty() && rwsetinfo.readSet.isEmpty()){
					commit = true;
					workpool.removeTask(tsk);
					return ; // nothing to do 
				}

				ValidateResponse res=  workpool.validate(tsk,rwsetinfo);
				commit = res.commit;	
				if(commit){
					tsk.txnInfo.commitTS = res.commitTS;
				}	
				else{
					boolean firstTry = true;
					valid = false;
					while(!valid) {
						txnInfo = beginTransaction();			
						tsk.txnInfo = txnInfo;
						if (!res.localResponse && !res.speculativeAbort && firstTry) {
							tsk.txnInfo.startTS = res.latestSTS;	
							firstTry = false;
						}
						origNode = dataStorage.getNode(tsk.getNodeId());
						nbrKeys  = new HashSet<String>(getNeighborIds(origNode));
						//first get the local neighbour nodes (complete node)
						localNeighborNodes = getLocalNeighbors(origNode);
						alreadyFetchedKeys = new HashSet<String>();
						for(Node n : localNeighborNodes) {
							alreadyFetchedKeys.add(n.nodeId);
						}
						nbrKeys.removeAll(alreadyFetchedKeys);
						remoteNodesData = getRemoteNeighborsData(new Vector<String>(nbrKeys), origNode.getClass().getName());
						verificationList = new Vector<Node>();
						verificationList.add(origNode);
						verificationList.addAll(localNeighborNodes);
						valid = earlyValidation(tsk, verificationList, remoteNodesData);
						if(!valid) {
							Thread.sleep(STS_DELAY);
						}
					}
				}
				//update STS is with latest STS
			}catch(Exception e){e.printStackTrace();}
		} //Transaction committed
		commitTask(tsk);
	}

	private Vector<NodeData> getRemoteNeighborsData(Vector<String> nbrKeys, String className) throws Exception {

		Vector<NodeData> reqList = new Vector<NodeData>();
		NodeData nd;
		for(String key : nbrKeys) {
			nd = new NodeData(key, dataStorage.nodeClassNametoIDMap.get(className));
			nd.addProperty("color", null);
			nd.addProperty("marked", null);
			reqList.add(nd);
		}

		return dataStorage.getNodePropertiesParallel(reqList);
	}

	@Override
	public void run ()  {
		startTime  = System.currentTimeMillis();

		Task  myTask;
		WorkStatus ws;

		boolean terminate = false;
		boolean relocated = false;
		
		boolean verifyResult=false;
		if(configParams.has("GCWorker.VERIFY_RESULT"))
			verifyResult=configParams.getBoolValue("GCWorker.VERIFY_RESULT");

		if(master) {
			try{
				Task t = new Task();
				t.phase = "STARTUP";
				t.nodeId="STARTUP_TASK";
				workpool.broadcast(t);
				
				/*Task t = new Task();
				t.phase = "RELOCATE";
				t.nodeId="RELOCATE_TASK";
				workpool.broadcast(t);*/
			} catch(Exception e) {
				e.printStackTrace();	
			}	
		}


		while (true)  {
			try {
				String myURL = dataStorage.myURL;
				ws = workpool.getTask();
				if (ws.status ==  StatusType.VALID_TASK) {
					myTask = ws.task;

					if (myTask == null) {
						System.out.println("Worker "+workerId+":  Error: Valid but Null task");
						continue;
					}
					if(myTask.phase.equals("STARTUP")){
						if(master)
							System.out.println("Worker "+workerId+":  Startup task picked up by master itself");
						else
							System.out.println("Worker "+workerId+":  Startup task picked up by a non master.");
						createTasksAllVertices(myTask);
						System.out.println("Worker "+workerId+":  Created initial tasks");
					}
					/*else if(myTask.phase.equals("RELOCATE")){
						if(master)
							System.out.println("Worker "+workerId+":  Relocate task picked up by master itself");
						else
							System.out.println("Worker "+workerId+":  Relocate task picked up by a non master.");
						
						int nodeIdForRelocate = (dataStorage.myId+dataStorage.numServers-1)%dataStorage.numServers;
						dataStorage.relocate(""+nodeIdForRelocate, true);
						
						System.out.println("Worker "+workerId+":  Completed relocation task");
						Thread.sleep(10000);
						
						Vector<Node> vec = dataStorage.myHashTableServer.getAllLocalNodes();
						String str = new String();
						for(Node nd : vec){
							str = str+nd.nodeId+", ";
						}
						System.out.println("Worker "+workerId+":: Nodes : "+str);
						
						Thread.sleep(10000);
						workpool.removeTask(myTask);
						
						if(master) {
							try{
								Task t = new Task();
								t.phase = "STARTUP";
								t.nodeId="STARTUP_TASK";
								workpool.broadcast(t);
							} catch(Exception e) {
								e.printStackTrace();	
							}	
						}
						
					}*/
					else if(myTask.phase.equals("GENERIC")){
						doTask( myTask );
						tasksCreated.clear();
					}
					else if (myTask.phase.equals("TERMINATION")) {
						//Stop task

						System.out.println("Worker "+workerId+":  Termination task");
						
						System.gc();
						workpool.progressStatusThread.stopPrinting = true;
						if(verifyResult){
							System.out.println("Worker "+workerId+":  Calling verify()");
							verify();
						}						
						//System.exit(0);
					}
					else{
						System.out.println("Worker "+workerId+":  Unknown phase");
					}


				}
				else if (ws.status == StatusType.NO_READY_TASK) {
					// Thread.sleep(configParams.getIntValue("Worker.THREADSLEEPTIME"));
				}
				else if (ws.status == StatusType.WORK_TERMINATED) {
					
					if(master && !terminate){
						System.out.println("Worker "+workerId+":  Master creating TERMINATE TASK - Worker thread proceeding to cleanUp");

						cleanUp();
						Task t = new Task();
						t.nodeId = "TERMINATE_TASK";
						t.phase = "TERMINATION";
						terminate = true;
						workpool.broadcast(t);
						workpool.progressStatusThread.stopPrinting = true;
					}
					/*else if(master && !relocated){
						System.out.println("Worker "+workerId+":  Master detected termination of relocation phase");
						relocated = true;
						
						try{
							Task t = new Task();
							t.phase = "STARTUP";
							t.nodeId="STARTUP_TASK";
							workpool.broadcast(t);
						} catch(Exception e) {
							e.printStackTrace();	
						}	
					}*/
					
				}				
			} catch (Exception e) {
				e.printStackTrace();
			}
		}   
	}

}
