package TestProgs.GraphColoring;

import java.util.*;
import java.io.*;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;

import beehive.util.*;

public class InputFileReader {

	StorageSystem storage;
	String fileName;	
	// public Vector<String> totalNodes;
	// public Vector<String> nodesCreated;
	Hashtable<String, Node> table;
	boolean debug = false;

	public InputFileReader (String filename, int numNodes, StorageSystem storage) {
		this.fileName = filename;
		this.storage = storage;
        table = new Hashtable<String, Node>();
        for(int i=0; i< numNodes; i++) {
            GCNode n = new GCNode(""+i);
            String nodeKey  =  Integer.toString(i);
            table.put(nodeKey, n);
        }
	}		

	public void readAndLoadData() throws IOException {
		
		BufferedReader br = new BufferedReader(new FileReader(fileName));
		String line = null;

		int ctr = 0;
		while( (line = br.readLine()) != null) {
		  ctr++;
		  if (ctr % 1000==0) {
			System.out.println(new Date()+" total nodes processed:"+ctr);
		  }
			StringTokenizer tok = new StringTokenizer(line," ");
			int currentNodeId = Integer.parseInt(tok.nextToken());
            		String currentNodeKey = Integer.toString( currentNodeId);  
		    /*
			nodesCreated.add(currentNodeKey);	
			if (!totalNodes.contains(currentNodeKey)) {
				totalNodes.add(currentNodeKey);	
			}*/
				
			GCNode currentNode  = new GCNode(currentNodeKey);            
			if (table.containsKey(currentNodeKey)) {
				currentNode = (GCNode)table.get(currentNodeKey);

            }
			//update the neighbor list for this node;
			String nbrList = "";
			if (tok.hasMoreTokens()){ 
				nbrList = tok.nextToken();
			} else
				continue;
			
			StringTokenizer tok2 = new StringTokenizer(nbrList,",");
			// loop over the list of neighbours
			while(tok2.hasMoreTokens()) {
				String neighborInfo = tok2.nextToken();
				StringTokenizer tok3 = new StringTokenizer(neighborInfo,":");
				int nbrId = Integer.parseInt(tok3.nextToken());  /* Neighbor Id */
                		
				String nbrKey = Integer.toString( nbrId );
				/*
				if (!totalNodes.contains(currentNodeKey)) { 
					totalNodes.add(currentNodeKey);	
				}
				*/	
				
				int weight = Integer.parseInt(tok3.nextToken());
	            GCEdge toNbr =  new GCEdge();
	            //toNbr.weight = weight; 
				currentNode.addNeighbor(nbrKey,toNbr);

				/* If the neighbor alread has an edge for the current node */				
				GCNode neighborNode = (GCNode) table.get(nbrKey);
				
                GCEdge fromNbr =  (GCEdge) neighborNode.neighbors.get(currentNodeKey) ;
                if (fromNbr == null) {
				  if (debug) 	
	                  System.out.println("No edge was found from" + neighborNode.nodeId + " to " + currentNodeKey);
                  fromNbr =  new GCEdge();
				  //fromNbr.weight = weight;
                  neighborNode.addNeighbor(currentNodeKey,fromNbr);
                  table.put(nbrKey, neighborNode);
                }               
				
			}
            table.put(currentNodeKey, currentNode);				
		    if (debug) 	
			  System.out.println("Added node: "+ currentNodeKey);
			
		}

	System.out.println(new Date()+ " All nodes processed total nodes:"+ ctr);
	ctr=0;
        Enumeration<String> enumKeys = table.keys();
		Vector<Node> batch = new Vector<Node>();	
		while (enumKeys.hasMoreElements()) { 
 			String key = enumKeys.nextElement();
			GCNode node = (GCNode)table.get(key);
			batch.add(node);
			if (batch.size() == 1000) { 
				storage.putNodesParallel(batch);
				batch = new Vector<Node>();
			}	
		    ctr++;
		    if (ctr % 1000==0) {
			  System.out.println(new Date()+" total nodes put:"+ctr);
		    }
        }
		if (batch.size() > 0) {
			storage.putNodesParallel(batch);			
		}
		 System.out.println(new Date()+" total nodes put:"+ctr);
	}

    

	
}

