package TestProgs.GraphColoring;

import java.util.*;
import java.io.*;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;

import beehive.util.*;

public class InputFileReaderBatch {

	StorageSystem storage;
	String fileName;	
	// public Vector<String> totalNodes;
	// public Vector<String> nodesCreated;
	Hashtable<String, Node> table;
	boolean debug = false;
	int lastFlushCtr = 0;
	static int BATCH_SIZE = 1000;
	public InputFileReaderBatch (String filename, int numNodes, StorageSystem storage) {
		this.fileName = filename;
		this.storage = storage;
		table = new Hashtable<String, Node>();
	}		

	public void readAndLoadData() throws IOException {

		BufferedReader br = new BufferedReader(new FileReader(fileName));
		String line = null;

		int ctr = 0;
		while( (line = br.readLine()) != null) {
			ctr++;
			if ((ctr-1) % BATCH_SIZE==0 && ctr!=1 ) {
				System.out.println(new Date()+" total nodes processed:"+ (ctr-1));
				flushTable();
			}
			StringTokenizer tok = new StringTokenizer(line," ");
			int currentNodeId = Integer.parseInt(tok.nextToken());
			String currentNodeKey = Integer.toString( currentNodeId);  
			/*
			nodesCreated.add(currentNodeKey);	
			if (!totalNodes.contains(currentNodeKey)) {
				totalNodes.add(currentNodeKey);	
			}*/

			GCNode currentNode  = new GCNode(currentNodeKey);            
			if (table.containsKey(currentNodeKey)) {
				currentNode = (GCNode)table.get(currentNodeKey);
			}
			//update the neighbor list for this node;
			String nbrList = "";
			if (tok.hasMoreTokens()){ 
				nbrList = tok.nextToken();
			} else
				continue;

			StringTokenizer tok2 = new StringTokenizer(nbrList,",");
			// loop over the list of neighbours
			while(tok2.hasMoreTokens()) {
				String neighborInfo = tok2.nextToken();
				StringTokenizer tok3 = new StringTokenizer(neighborInfo,":");
				int nbrId = Integer.parseInt(tok3.nextToken());  /* Neighbor Id */

				String nbrKey = Integer.toString( nbrId );
				/*
				if (!totalNodes.contains(currentNodeKey)) { 
					totalNodes.add(currentNodeKey);	
				}
				 */	

				int weight = Integer.parseInt(tok3.nextToken());
				GCEdge toNbr =  new GCEdge();
				//toNbr.weight = weight; 
				currentNode.addNeighbor(nbrKey,toNbr);

				/* If the neighbor alread has an edge for the current node */				
				GCNode neighborNode = (GCNode) table.get(nbrKey);
				if (neighborNode == null) {
					neighborNode = new GCNode(nbrKey);
				}
				GCEdge fromNbr =  (GCEdge) neighborNode.neighbors.get(currentNodeKey) ;
				if (fromNbr == null) {
					if (debug) 	
						System.out.println("No edge was found from" + neighborNode.nodeId + " to " + currentNodeKey);
					fromNbr =  new GCEdge();
					//fromNbr.weight = weight;
					neighborNode.addNeighbor(currentNodeKey,fromNbr);
					table.put(nbrKey, neighborNode);
				}              

			}
			table.put(currentNodeKey, currentNode);				
			if (debug) 	
				System.out.println("Added node: "+ currentNodeKey);

		}

		System.out.println(new Date()+ " All nodes processed total nodes:"+ ctr);
		if (!table.isEmpty()) {
			Vector<Node> batch = new Vector<Node>();
			batch.addAll(table.values());
			System.out.println("Flushing the last batch of "+batch.size()+" nodes");
			try {
				//				for(Node n : batch) {
				//					storage.putNode(n.nodeId+"", n);
				//				}
				storage.putNodesParallel(batch);
			} catch(Exception e) {e.printStackTrace();}
		}
	}	


	private void flushTable() {			
		Enumeration<String> enumKeys = table.keys();
		Vector<Node> batch = new Vector<Node>();
		int ctr = lastFlushCtr;	
		for(int i = 0; i<BATCH_SIZE; i++) {
			String key = Integer.toString(ctr);
			GCNode node = (GCNode)table.remove(key);
			batch.add(node);
			ctr++;
		}
		System.out.println("Flushing from "+lastFlushCtr+" to "+ (ctr-1));
		if (batch.size() > 0) {
			try {
				//				for(Node n : batch) {
				//					storage.putNode(n.nodeId+"", n);
				//				}
				storage.putNodesParallel(batch);			
			} catch(Exception e) { e.printStackTrace();}
		}
		System.out.println(new Date()+" total nodes put till now:"+ctr);
		lastFlushCtr = ctr;
	}



}

