package TestProgs.GraphColoring;

import java.util.*;
import java.io.*;
import beehive.server.*;
import beehive.util.*;

public class InputFileReaderDistributed{

	String input_filename;
	HashtableServer server;
	int BATCH_SIZE = 100000;

	public InputFileReaderDistributed(String filename, HashtableServer server){
		input_filename = filename;
		this.server = server;
	}

	public void readAndLoadData() throws IOException, InterruptedException{
		try{
			BufferedReader input = new BufferedReader(new FileReader(input_filename));
			String line = "";

			Vector<String> batch = new Vector<String>();

			int ctr = 0;
			while( (line = input.readLine()) != null ){
				//add to batch only if this is local data to be loaded
				StringTokenizer lineTok = new StringTokenizer(line, " ");
				String currentNodeKey = lineTok.nextToken();
				int currentNodeId = Integer.parseInt(currentNodeKey);
				if(currentNodeId % server.getNumServers() != server.getServerId()) {
					continue;
				}
				batch.add(line);
				ctr++;
				if(ctr % BATCH_SIZE == 0){
					processBatch(batch);
					batch.clear();
				}
			}
			//process the remaining ones
			processBatch(batch);

			input.close();
		} catch(Exception e){
			e.printStackTrace();
			System.err.println("Failed to read file");
		}
	}
	
	private void processBatch(Vector<String> batch) {
		Vector<Node> node_batch = new Vector<Node>();
		for(String node_string: batch){
			GCNode node = processNode(node_string);
			node_batch.add(node);
		}
		try {
			server.loadNodes(node_batch);
		}catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private GCNode processNode(String str){
		StringTokenizer lineTok = new StringTokenizer(str, " ");

		String currentNodeKey = lineTok.nextToken();

		GCNode currentNode = new GCNode(currentNodeKey);

		if(!lineTok.hasMoreTokens()){
			return currentNode;
		}
		String nbrList = lineTok.nextToken();

		StringTokenizer nbrTok = new StringTokenizer(nbrList, ",");

		while(nbrTok.hasMoreTokens()){
			String nbrInfo = nbrTok.nextToken();

			StringTokenizer edgeTok = new StringTokenizer(nbrInfo, ":");

			String nbrKey = edgeTok.nextToken();
			//int nbrId = Integer.parseInt(nbrKey);

			//int weight = Integer.parseInt(edgeTok.nextToken());

			GCEdge toNbr = new GCEdge();
			//toNbr.weight = weight;

			currentNode.addNeighbor(nbrKey,toNbr);
		}

		return currentNode;
	}

}
