/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.KNearestNeighborsV2;

import java.util.*;
import java.util.concurrent.LinkedBlockingQueue;
import java.io.*;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;

import beehive.util.*;

public class InputFileReaderParallel{

	String input_filename;
	int num_nodes;
	int BATCH_SIZE = 1000;
	StorageSystem storage;

	boolean reader_done;

	LinkedBlockingQueue<String> queue;

	public InputFileReaderParallel(String filename, int numNodes, StorageSystem storage){
		input_filename = filename;
		num_nodes = numNodes;
		this.storage = storage;
		queue = new LinkedBlockingQueue<String>();
	}

	public void readAndLoadData(int k) throws IOException, InterruptedException{

		reader_done = false;

		NodeFileReader fr = new NodeFileReader();
		fr.start();

		NodeWriter[] writers = new NodeWriter[10];
		for(int i = 0; i < 10; i++){
			writers[i] = new NodeWriter(k);
			writers[i].start();
		}

		fr.join();
		for(int i = 0; i < 10; i++){
			writers[i].join();
		}
	}


	private class NodeFileReader extends Thread{

		public void run(){
			try{
				BufferedReader input = new BufferedReader(new FileReader(input_filename));
				String line = "";

				Vector<String> batch = new Vector<String>();

				int ctr = 0;
				while( (line = input.readLine()) != null ){
					batch.add(line);
                    //System.out.println("Line" + ctr +  ":" + line);  

					ctr++;
					if( (ctr % BATCH_SIZE == 0) && ctr != 1 ){
				        //System.out.println("Flusing line batch of size = " + ctr);
						flushToQueue(batch);
					}
				}

			    flushToQueue(batch);  // July 18, 2016
				input.close();
			} catch(Exception e){
				System.err.println("Failed to read file");
			}
			reader_done = true;
		}

		public void flushToQueue(Vector<String> batch){
			queue.addAll(batch);
			batch.clear();
		}
	}

	private class NodeWriter extends Thread{
		int k;
		
		public NodeWriter(int k){
			this.k = k;
		}
		
		public void run(){
			Vector<String> str_batch = new Vector<String>();
			Vector<Node> node_batch = new Vector<Node>();

			while( (queue.drainTo(str_batch, 100) > 0) || !reader_done ){
                //System.out.println("Processing line batch size=" + str_batch.size() );
				for(String node_string: str_batch){
					KNNNode node = processNode(node_string,k);
					//System.out.println("Created node id = " + node.nodeId +" Edges: " +  node.printNodeEdges());
					node_batch.add(node);
				}
				str_batch.clear();

				try{
                    // System.out.println("Adding nodes to storage systems = " + node_batch.size()); 
					storage.putNodesParallel(node_batch);
				}
				catch(Exception re){
					System.err.println("Failed to write to storage system");
				}	
				node_batch.clear();
			}
		}

		public KNNNode processNode(String str, int k){
			StringTokenizer lineTok = new StringTokenizer(str, " ");

			String currentNodeKey = lineTok.nextToken();
			KNNNode currentNode = new KNNNode(currentNodeKey, k);

			if(!lineTok.hasMoreTokens()){
				return currentNode;
			}
			String nbrList = lineTok.nextToken();

			StringTokenizer nbrTok = new StringTokenizer(nbrList, ",");

			while(nbrTok.hasMoreTokens()){
				String nbrInfo = nbrTok.nextToken();

				StringTokenizer edgeTok = new StringTokenizer(nbrInfo, ":");

				String nbrKey = edgeTok.nextToken();
				int weight = Integer.parseInt(edgeTok.nextToken());

				KNNEdge toNbr = new KNNEdge();
				toNbr.weight = weight;

				currentNode.addNeighbor(nbrKey, toNbr);
			}

			return currentNode;
		}
	}	
}
