/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.KNearestNeighborsV2;

import beehive.util.*;

import java.util.*;
import java.io.*;


public class KNNNode extends Node implements Externalizable  {

	public KNNSortedSet sortedNeighbors;
	
	public KNNSortedSet getSortedNeighbors() {
		return sortedNeighbors;
	}

	public void setSortedNeighbors(KNNSortedSet sortedNeighbors) {
		this.sortedNeighbors = sortedNeighbors;
	}

	public KNNNode(){ // for externalizable
		super();
		sortedNeighbors = new KNNSortedSet(-1,"-1");
	}
	
	public KNNNode(String id, int k) {
		super( id );
		this.sortedNeighbors = new KNNSortedSet(k,id);
	}
	
	public KNNNode(KNNNode n) {
		super(n, false);

		Enumeration edgeKeys = n.neighbors.keys();
		while(edgeKeys.hasMoreElements())
		{
			String ekey = (String)edgeKeys.nextElement();
			KNNEdge e = (KNNEdge)n.neighbors.get(ekey);
			KNNEdge f = new KNNEdge(e);
			neighbors.put(ekey,f);
		}
		this.sortedNeighbors = new KNNSortedSet(n.sortedNeighbors);
	}	
	

	@Override  
   	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {  
       	super.readExternal(in);
       	this.sortedNeighbors = (KNNSortedSet) in.readObject();
   	}
   	@Override  
   	public void writeExternal(ObjectOutput out) throws IOException {  
   		super.writeExternal(out);
   		out.writeObject(this.sortedNeighbors);
   	}
}
