/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.KNearestNeighborsV2;

import beehive.util.*;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;

public class KNNTask extends Task {
	
	HashMap<String, KNNSortedSet> sortedNeighborsMap;
	HashMap<String, Integer> edgeLengthMap;
	
	public KNNTask(){
		super();
	}
	
	public KNNTask(String nodeId, KNNSortedSet sortedNeighbors, String senderId, int edgeLength){
		super(nodeId);
		this.sortedNeighborsMap = new HashMap<String, KNNSortedSet>();
		this.edgeLengthMap = new HashMap<String, Integer>();
		
		KNNSortedSet srtNeighbors = new KNNSortedSet(sortedNeighbors);
		this.sortedNeighborsMap.put(senderId, srtNeighbors);
		this.edgeLengthMap.put(senderId, edgeLength);
	}
	
	public KNNTask(int k, String nodeId, ConcurrentHashMap<String,Edge> neighbors, String senderId, int edgeLength){
		super(nodeId);
		this.sortedNeighborsMap = new HashMap<String, KNNSortedSet>();
		this.edgeLengthMap = new HashMap<String, Integer>();
		
		KNNSortedSet srtNeighbors = new KNNSortedSet(k,nodeId,neighbors);
		this.sortedNeighborsMap.put(senderId, srtNeighbors);
		this.edgeLengthMap.put(senderId, edgeLength);
	}
	
	@Override
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
		super.readExternal(in);
    	this.sortedNeighborsMap = (HashMap<String, KNNSortedSet>) in.readObject();
		this.edgeLengthMap = (HashMap<String, Integer>) in.readObject();
	}

	@Override
	public void writeExternal(ObjectOutput out) throws IOException {
		super.writeExternal(out);
		out.writeObject(this.sortedNeighborsMap);
		out.writeObject(this.edgeLengthMap);
	}
}
