/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.KNearestNeighborsV2;

import  beehive.server.*;
import  beehive.util.*;
import beehive.worker.*;

import java.util.*;

import TestProgs.util.BeehiveAppLoader;

public class KNNTest extends BeehiveAppLoader {

	static int k; 
	
	public static void main (String args[]) throws Exception {

		if (args.length != 7) {
			// old :   ("Syntax: KNNTest <hostListFile> <config params file> <workpoolserver hostname> <input filename> <k> <num_nodes> <MsgFile>");
			throw new RuntimeException ("Syntax: KNNTest <hostListFile> <config params file> <workpoolserver hostname> <input filename> <num_nodes> <MsgFile> <k> "); // Corrected Format for AppLoader
		}
		
		k = Integer.parseInt(args[6]);
		
		//All inherited Node classes used
		HashMap<Byte, String> nodeClassNameMap = new HashMap<Byte, String>();
		HashMap<Byte, Class> nodeClassMap = new HashMap<Byte, Class>();
		nodeClassNameMap.put((byte) 1, new KNNNode().getClass().getName());
		nodeClassMap.put((byte) 1, new KNNNode().getClass());
		
		
		KNNTest loader = new KNNTest();
		loader.init(args, "KNNTest", nodeClassNameMap, nodeClassMap);

		String configParamsFile = args[1];
		Config configParams = new Config(configParamsFile);
		boolean taskAggrReplaceAndEnsureKItems = false;
		taskAggrReplaceAndEnsureKItems = configParams.getBoolValue("KNNWorker.TASK_AGGREGATOR_REPLACE_AND_ENSURE_K_ITEMS");
		
		loader.workpool.taskComparator = new KNNTaskComparator(taskAggrReplaceAndEnsureKItems);
		System.out.println(loader.workpool.myHost+" completed Initialization...");
		Thread.sleep(10000);
		
		boolean master_thread = loader.isMaster();
		for (int i=0; i< loader.getNumThreads(); i++) {
			if (i> 0) {	
				master_thread = false;
			}
			Worker worker = null;
			//Create a worker with or without reflection properties
			if(loader.isUseNodeData()) {
				//  worker using reflection
			}else {
				worker = new KNNWorker(loader.getHostlistFile(), loader.getWorkpool(), master_thread,  loader.getDataStorage(), i, loader.getLocation(), Integer.parseInt(args[6]), loader.getNumNodes());
			}
			Thread workerThread = new Thread(worker);
			workerThread.start();
		}
	}

	@Override
	public void loadGraphData(String input_file, HashtableServer server, int numNodes, StorageSystem  dataStorage) {
		// TODO Auto-generated method stub
		try{
			InputFileReaderDistributed fileReader = new InputFileReaderDistributed(input_file, server, k);
			//InputFileReaderParallel fileReader = new InputFileReaderParallel(input_file, loader.getNumNodes(), server);
			fileReader.readAndLoadData();
		}catch (Exception e) {
			e.printStackTrace();
		}

	}

}
