/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.KNearestNeighborsV2;

import beehive.util.*;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

public class RecomputeTask extends Task {
	
	KNNSortedSet sortedNeighbors;
	String senderId;
	Set<String> deletedSet;
	
	public RecomputeTask(String nodeId, KNNSortedSet sortedNeighbors, String senderId, HashSet<String> deletedSet){
		super(nodeId);
		this.sortedNeighbors = new KNNSortedSet(sortedNeighbors);
		this.senderId = senderId;
		this.deletedSet = deletedSet;
	}
	
	public RecomputeTask(int k, String nodeId, ConcurrentHashMap<String,Edge> neighbors, String senderId, HashSet<String> deletedSet){
		super(nodeId);
		this.sortedNeighbors = new KNNSortedSet(k,nodeId, neighbors);
		this.senderId = senderId;
		this.deletedSet = deletedSet;
	}
	
	public RecomputeTask(){
		super();
	}
	
	@Override
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
		super.readExternal(in);
    	this.sortedNeighbors = (KNNSortedSet) in.readObject();
		this.senderId = (String) in.readObject();
		this.deletedSet = (Set<String>) in.readObject();
	}

	@Override
	public void writeExternal(ObjectOutput out) throws IOException {
		super.writeExternal(out);
		out.writeObject(this.sortedNeighbors);
		out.writeObject(this.senderId);
		out.writeObject(this.deletedSet);
	}
}
