/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.MST.MSTAsync;

import TestProgs.util.BeehiveAppLoader;
import beehive.server.*;
import beehive.util.*;

import java.util.*;
import java.io.*;

public class MSTTest extends BeehiveAppLoader {

    public static void main(String args[]) throws Exception {

        if (args.length != 6) {
            throw new RuntimeException("Syntax: MSTTreeTest <hostlist> <config params file> <workpoolserver hostname> <input filename> <num_nodes> <MsgFile>");
        }

        String configParamsFile = args[1];
        Config configParams = new Config(configParamsFile);

        //All inherited Node classes used
        HashMap<Byte, String> nodeClassNameMap = new HashMap<Byte, String>();
        HashMap<Byte, Class> nodeClassMap = new HashMap<Byte, Class>();
        nodeClassNameMap.put((byte) 1, MSTNode.class.getName());
        nodeClassMap.put((byte) 1, MSTNode.class.getClass());



        MSTTest loader = new MSTTest();
        loader.init(args, "MSTTest", nodeClassNameMap, nodeClassMap);

        boolean master_thread = loader.isMaster();
        for (int i=0; i< loader.getNumThreads(); i++) {
            if (i> 0) {
                master_thread = false;
            }
            MSTWorker worker = new MSTWorker(configParamsFile, loader.getWorkpool(), master_thread, loader.getDataStorage(), loader.getLocation(), loader.getNumNodes());
            Thread workerThread = new Thread(worker);
            workerThread.start();
        }
    }

    @Override
    public void loadGraphData(String input_file, HashtableServer server, int numNodes, StorageSystem dataStorage) {
        InputFileReader reader = new InputFileReader(input_file, numNodes, dataStorage);
        try {
            reader.readAndLoadData();
            if (reader.totalNodes.size() != reader.nodesCreated.size()) {
                System.out.println("Mismatch in number of nodes created and total nodes");
                System.out.println("Total Nodes: " + reader.totalNodes.size() + " Nodes Created: " + reader.nodesCreated.size());
                System.exit(1);
            }
            reader.printTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
