/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.MST.MSTOptimized;

import beehive.server.HashtableServer;
import beehive.util.Node;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;

public class InputFileReaderDistributed {

    String input_filename;
    HashtableServer server;
    int BATCH_SIZE = 100000;

    public InputFileReaderDistributed(String filename, HashtableServer server) {
        input_filename = filename;
        this.server = server;
    }

    public void readAndLoadData() throws IOException, InterruptedException {
        try {
            BufferedReader input = new BufferedReader(new FileReader(input_filename));
            String line = "";

            Vector<String> batch = new Vector<String>();

            int ctr = 0;
            while ((line = input.readLine()) != null) {
                //add to batch only if this is local data to be loaded
                StringTokenizer lineTok = new StringTokenizer(line, " ");
                String currentNodeKey = lineTok.nextToken();
                int currentNodeId = Integer.parseInt(currentNodeKey);
                if (currentNodeId % server.getNumServers() != server.getServerId()) {
                    continue;
                }
                batch.add(line);
                ctr++;
                if (ctr % BATCH_SIZE == 0) {
                    processBatch(batch);
                    batch.clear();
                }
            }
            //process the remaining ones
            processBatch(batch);

            input.close();
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("Failed to read file");
        }
    }

    private void processBatch(Vector<String> batch) {
        Vector<Node> node_batch = new Vector<Node>();
        for (String node_string : batch) {
            MSTNode node = processNode(node_string);
            node_batch.add(node);
        }
        try {
            server.loadNodes(node_batch);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private MSTNode processNode(String str) {
        StringTokenizer lineTok = new StringTokenizer(str, " ");

        String currentNodeKey = lineTok.nextToken();

        MSTNode currentNode = new MSTNode(currentNodeKey);

        if (!lineTok.hasMoreTokens()) {
            return currentNode;
        }
        String nbrList = lineTok.nextToken();

        StringTokenizer nbrTok = new StringTokenizer(nbrList, ",");

        while (nbrTok.hasMoreTokens()) {
            String nbrInfo = nbrTok.nextToken();

            StringTokenizer edgeTok = new StringTokenizer(nbrInfo, ":");

            String nbrKey = edgeTok.nextToken();
            int weight = Integer.parseInt(edgeTok.nextToken());

            MSTEdge toNbr = new MSTEdge(currentNodeKey, nbrKey);
            toNbr.weight = weight;

            currentNode.addNeighbor(nbrKey, toNbr);
        }

        return currentNode;
    }

}
