/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.MST.MSTOptimized;

import beehive.util.*;

import java.util.*;
import java.io.*;

public class MSTNode extends Node implements Externalizable {

    public String coreId;
    public int MSTCost;
    public Vector<String> clusterKeys;

    public MSTNode(String id) {
        super(id);
        this.coreId = id;
        this.MSTCost = 0;
        this.clusterKeys = new Vector<String>();
        this.clusterKeys.add(id);
    }

    public MSTNode() {
        super();
        this.coreId = "";
        this.MSTCost = 0;
        this.clusterKeys = new Vector<>();
    }

    public MSTNode(MSTNode n) {
        super(n.nodeId);
        this.coreId = n.coreId;
        this.MSTCost = n.MSTCost;
        this.clusterKeys = new Vector<>(n.clusterKeys);

        this.TS = n.TS;

        Enumeration edgeKeys = n.neighbors.keys();
        while (edgeKeys.hasMoreElements()) {
            String ekey = (String) edgeKeys.nextElement();
            MSTEdge e = (MSTEdge) n.neighbors.get(ekey);
            MSTEdge f = new MSTEdge(e);
            neighbors.put(f.nbrId, f);
        }

    }

    public void setCoreId(String coreId) {
        this.coreId = coreId;
    }

    public String printNode() {
        StringBuffer str = new StringBuffer();
        str.append("Node id: " + nodeId + "=" + " Cored id: " + coreId);
        return str.toString();
    }

    @Override
    public String toString() {
        return "MSTNode{" +
                "nodeId='" + nodeId + "\'" +
                "coreId='" + coreId + '\'' +
                '}';
    }

    public String printNodeEdges() {
        StringBuffer str = new StringBuffer();
        str.append("Node id: " + nodeId + "=" + " Cored id: " + coreId);
        Enumeration edges = neighbors.keys();
        while (edges.hasMoreElements()) {
            String key = (String) edges.nextElement();
            MSTEdge e = (MSTEdge) neighbors.get(key);
            str.append(e.toString() + " ");
        }
        return str.toString();
    }

    @Override
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal(in);

        this.coreId = (String) in.readObject();
        this.MSTCost = in.readInt();
        this.clusterKeys = (Vector<String>) in.readObject();
    }

    @Override
    public void writeExternal(ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeObject(this.coreId);
        out.writeInt(this.MSTCost);
        out.writeObject(this.clusterKeys);
    }
}





