package TestProgs.MaxClique;

import java.util.*;
import java.io.*;

import beehive.server.*;
import beehive.util.*;

public class InputFileReaderBucketed{

	String input_filename;
	String input_dir_path;
	HashtableServer server;
	int BATCH_SIZE = 100000;

	public InputFileReaderBucketed(String dirPath, HashtableServer server){
		input_dir_path = dirPath;
		this.server = server;
	}

	public void readAndLoadData() throws IOException, InterruptedException{
		try{
			input_filename = input_dir_path + "/" + "bucket_" + server.getServerId();
			BufferedReader input = new BufferedReader(new FileReader(input_filename));
			String line = "";

			Vector<String> batch = new Vector<String>();

			int ctr = 0;
			while( (line = input.readLine()) != null ){
				//add to batch only if this is local data to be loaded
				StringTokenizer lineTok = new StringTokenizer(line, " ");
				String currentNodeKey = lineTok.nextToken();
				int currentNodeId = Integer.parseInt(currentNodeKey);
				batch.add(line);
				ctr++;
				if(ctr % BATCH_SIZE == 0){
					processBatch(batch);
					batch.clear();
				}
			}
			//process the remaining ones
			processBatch(batch);

			input.close();
		} catch(Exception e){
			e.printStackTrace();
			System.err.println("Failed to read file");
		}
	}
	
	private void processBatch(Vector<String> batch) {
		Vector<Node> node_batch = new Vector<Node>();
		for(String node_string: batch){
			MCNode node = processNode(node_string);
			node_batch.add(node);
		}
		try {
			server.loadNodes(node_batch);
		}catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private MCNode processNode(String str){
		StringTokenizer lineTok = new StringTokenizer(str, " ");

		String currentNodeKey = lineTok.nextToken();
		MCNode currentNode = new MCNode(currentNodeKey);

		if(!lineTok.hasMoreTokens()){
			return currentNode;
		}
		String nbrList = lineTok.nextToken();

		StringTokenizer nbrTok = new StringTokenizer(nbrList, ",");

		while(nbrTok.hasMoreTokens()){
			String nbrInfo = nbrTok.nextToken();

			StringTokenizer edgeTok = new StringTokenizer(nbrInfo, ":");

			String nbrKey = edgeTok.nextToken();
			int weight = Integer.parseInt(edgeTok.nextToken());

			MCEdge toNbr = new MCEdge();
			toNbr.weight = weight;

			currentNode.addNeighbor(nbrKey, toNbr);
		}

		return currentNode;
	}

}
