package TestProgs.MaxClique;

import beehive.util.*;
import beehive.server.*;
import beehive.workpool.*;
import beehive.worker.*;

import java.rmi.RemoteException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

import java.io.*;
import java.nio.channels.*;

public class MCWorkerReflection extends Worker implements Runnable {
	
	long ioStartTime = 0;
	long ioTime = 0;
	long phaseStartTime[] = new long[6]; 
	long phaseEndTime[] = new long[6];

	boolean debug = false;
	int calls = 0;	 
	String myStorageURL;
	int numNodes = 0;
	public static Object mstobjLock = new Object();
	
	HashSet<String> computeCliqueNodesSet;
	HashSet<String> allComputeCliqueNodes;
	
	int recomputeCliqueMetaDataId = -(dataStorage.numServers);
	
	public MCWorkerReflection (String configFile, LocalWorkpoolImpl wpool, boolean master, StorageSystem datastorage, int workerId, String myStorageURL, int numOfNodes ,HashSet<String> computeCliqueNodesSet, HashSet<String> allComputeCliqueNodes )  throws java.rmi.RemoteException {
		super(configFile, wpool, workerId, datastorage, master);
		this.myStorageURL = myStorageURL;
		this.numNodes = numOfNodes;
		this.computeCliqueNodesSet = computeCliqueNodesSet;
		this.allComputeCliqueNodes = allComputeCliqueNodes;
	}
	
	public HashSet<String> calculateMaxClique(HashSet<String> hs, String nodeId){
		HashSet<String> result = new HashSet<String>();
		//HashSet oldHashSet = hs;
		boolean nocliques = false;
		int iter = 0;
		//System.out.println(nodeId+":: Size of initial hashset : "+hs.size());
		
		while(true){
			iter++;
			Iterator it = hs.iterator();
			while(it.hasNext()){
				String str = (String) it.next();
				String s = str.substring(0, (str.lastIndexOf(';')+1));	//all chars of string upto last comma
				String oldval = str.substring((str.lastIndexOf(';')+1), str.length());	//all chars of string after last comma
				//System.out.println(s);
				Iterator it2 = hs.iterator();
				while(it2.hasNext()){
					String t = (String)it2.next();
					if(t.equals(str))
						continue;
					if(t.indexOf(s)!=-1){
						//System.out.println("t : "+t+" str : "+str);
						String newval = t.substring((t.lastIndexOf(';')+1), t.length());
						//System.out.println(newval);
						String newclique = new String();
						//System.out.println("oldval : "+oldval+" newval : "+newval);
						if(Integer.parseInt(oldval)>Integer.parseInt(newval)){
							newclique = s+newval+";"+oldval;
						}
						else{
							newclique = s+oldval+";"+newval;
						}
						//System.out.println(newclique);
						if(result.contains(newclique))
							continue;
						
						String temp[] = newclique.split(";");
						int x[] = new int[2*temp.length-2]; //1,2,3,4 : to be stored as 1,2,3,4,1,2 to get 1,2,3;2,3,4;3,4,1;4,1,2 faster 
						for(int i=0;i<x.length;i++){
							x[i] = Integer.parseInt(temp[i%temp.length]);
						}
						
						boolean notpresent = false;
						for(int i=0;i<temp.length;i++){
							int[] y = Arrays.copyOfRange(x, i, i+temp.length-1);
							Arrays.sort(y);
							String strtemp = Arrays.toString(y).replaceAll("[\\[\\] ]", "");
							//System.out.println(strtemp);
							if(!hs.contains(strtemp)){
								notpresent = true;
								break;
							}						
						}
						if(notpresent==false){
							//System.out.println(nodeId+":: Got new clique : "+newclique);
							result.add(newclique);
						}
					}
				}
			}
			if(result.size()==0){
				/*if(iter==1){
					nocliques = true;
				}*/
				break;
			}
			else{
				//oldHashSet = hs;
				boolean notPresentInAnySet = true;
				Iterator it5 = result.iterator();
				boolean terminateWhileLoop = false;
				while(it5.hasNext()){
					String str = (String) it5.next();
					String[] ids = str.split(";");
					for(int i=0;i<ids.length;i++){
						if(ids[i].equals(nodeId)){
							notPresentInAnySet = false;
							terminateWhileLoop = true;
							break;
						}
					}
					if(terminateWhileLoop)
						break;
				}
				if(notPresentInAnySet==true){	//the new set result does not have this nodeId. So, older set was the largest set for this node.
					break;
				}
				else{	//new set result contains atleast one clique with this node
					hs = result;
					result = new HashSet<String>();
				}
			}
		}
		
		/*if(nocliques){
			return null;
		}
		else{*/
			//remove all the cliques which do not have this node in their clique set.
			Iterator it5 = hs.iterator();
			while(it5.hasNext()){
				String str = (String) it5.next();
				String[] ids = str.split(";");
				boolean notPresent = true;
				for(int i=0;i<ids.length;i++){
					if(ids[i].equals(nodeId)){
						notPresent = false;
						break;
					}
				}
				if(notPresent==true){
					it5.remove();
				}
			}
			if(hs.size()==0){
				return null;
			}
			else{
				return hs;
			}
		/*}*/
	}

	public void computeTriplets(Task t, Node node) {
		try{
			if((MCNode)node == null)
			{
				return;
			}
			MCNode u = new MCNode((MCNode)node);
			
			Vector<Node> localNeighborNodes = null;
			Vector<NodeData> remoteNodesData = null;
			HashSet<String> nbrKeys = null;
			HashSet<String> alreadyFetchedKeys = null;
			
			nbrKeys  = new HashSet<String>(getNeighborIds(node));
			//first get the local neighbour nodes (complete node)
			localNeighborNodes = getLocalNeighbors(node);
			alreadyFetchedKeys = new HashSet<String>();
			for(Node n : localNeighborNodes) {
				alreadyFetchedKeys.add(n.nodeId);
			}
			nbrKeys.removeAll(alreadyFetchedKeys);
			remoteNodesData = getRemoteNeighborsData(new Vector<String>(nbrKeys), u.getClass().getName());
			
			Vector<String> uNbr = new Vector<String>();
			Iterator it = u.neighbors.keySet().iterator();
			while(it.hasNext()){
				uNbr.add((String) it.next());
			}
			
			for(int i=0;i<uNbr.size();i++){
				//rwsetinfo.readSet.add(uNbr.get(i));
				
				for(int j=i+1;j<uNbr.size();j++){
					String b = uNbr.get(i);
					String c = uNbr.get(j);
					int bval = Integer.parseInt(b);
					int cval = Integer.parseInt(c);
					ConcurrentHashMap<String,Edge> ht=null;
					
					int present = -1;
					for(Node nd : localNeighborNodes){
						if(Integer.parseInt(nd.nodeId)==bval){
							ht = (ConcurrentHashMap<String,Edge>) nd.neighbors;
							present = 1;
							break;
						}
					}
					if(present == -1){
						for(NodeData nd : remoteNodesData){
							if(Integer.parseInt(nd.nodeId)==bval){
								ht = (ConcurrentHashMap<String,Edge>) nd.properties.get("neighbors");
								break;
							}
						}
					}
					if(ht==null){
						System.out.println("bval : "+bval+"localneighbors : "+localNeighborNodes+", remoteNodesData : "+remoteNodesData);
					}
					if(ht.containsKey(c)){
						int x[] = new int[3];
						x[0]=Integer.parseInt(u.nodeId);
						x[1]=bval;
						x[2]=cval;
						Arrays.sort(x);
						String strtemp = Arrays.toString(x).replaceAll("[\\[\\] ]", "").replaceAll(",", ";");
						//System.out.println("Added triplet for "+u.nodeId+" : "+strtemp);
						u.myTriplets.add(strtemp);
					}
				}
			}
			u.TS = u.TS + 1;
			updatedNodes.put(u.nodeId, u);
			//rwsetinfo.writeSet.add(u.nodeId);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	public void updateMyTriplets(Task t, Node node) {
		try{
			if((MCNode)node == null)
				return;
			MCNode mcnode = new MCNode((MCNode)node);
			UpdateTripletsTask tsk = (UpdateTripletsTask)t;
			
			if(tsk.taskType==MCTaskConstants.addToMyTriplets){
				mcnode.myTriplets.add(tsk.triple);
				mcnode.TS = mcnode.TS + 1;
				updatedNodes.put(mcnode.nodeId, mcnode);
			}
			else if(tsk.taskType==MCTaskConstants.deleteFromMyTriplets){
				mcnode.myTriplets.remove(tsk.triple);
				mcnode.TS = mcnode.TS + 1;
				updatedNodes.put(mcnode.nodeId, mcnode);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}
	}

	@Override
	public void doTask(Task tsk) throws Exception
	{
		boolean commit = false;
		tasksCreated.clear();
		updatedNodes.clear();
		//TransactionInfo txnInfo = null;
		//txnInfo = beginTransaction();
		//tsk.txnInfo = txnInfo;		
		
		Node origNode = dataStorage.getNode(tsk.getNodeId());
		
		Class c = tsk.getClass();
		String strclassnamearr[] = c.toString().split(" ")[1].split("\\.");
		String taskClassName = strclassnamearr[strclassnamearr.length-1];
		//System.out.println("Task class name : "+taskClassName);
		
		HashSet<String> tempRecomputeCliqueNodes = new HashSet<String>();
		
		while(!commit)
		{
			try{
				//System.out.println(taskClassName);
				if(taskClassName.equals("UpdateGraphTask")){
					tempRecomputeCliqueNodes = updateGraph(tsk, origNode);
				}
				else if(taskClassName.equals("UpdateTripletsTask")){
					updateMyTriplets(tsk, origNode);
				}
				else if(taskClassName.equals("CliqueComputeTask")){
					computeClique(tsk, origNode);
				}
				else{
					computeTriplets(tsk, origNode);
				}
				
				if(updatedNodes.size()>0){
					commit = commitTask(tsk);
					if(commit){
						workerStatus.status = WorkerStatus.REPORT_COMPLETION;
						workpool.reportCompletion(tsk, tasksCreated);
						break;
					}
				}
				else if(updatedNodes.size() == 0){
					commit = true;
					workerStatus.status = WorkerStatus.REPORT_COMPLETION;
					workpool.reportCompletion(tsk, tasksCreated);
				}
				
				if(commit == false){
					origNode = dataStorage.getNode(tsk.getNodeId());
					updatedNodes.clear();
					tasksCreated.clear();
				}
				
				//update STS is with latest STS
			}catch(Exception e){e.printStackTrace();}
		} //Transaction committed
		
		//for UpdateGraphTask, adding info to recomputeClique
		if(tempRecomputeCliqueNodes != null && tempRecomputeCliqueNodes.size()>0){
			//System.out.println("RecomputeCliqueMetadata ID : "+recomputeCliqueMetaDataId + ", mapdata : "+dataStorage.nodeClassNametoIDMap+", getting : "+(new MetaDataNode().getClass().getName()));
			NodeData nd = new NodeData(""+recomputeCliqueMetaDataId, dataStorage.nodeClassNametoIDMap.get(new MetaDataNode().getClass().getName()));
			nd.properties.put("addAllToSet", tempRecomputeCliqueNodes);
			nd.TS = nd.TS + 1;
			dataStorage.exec(nd);
		}
		//commitTask(tsk);
	}
	
	public void computeClique(Task t, Node node) {
		try{
			MCNode u = new MCNode((MCNode) node);
			//clearing allTriplets and maxClique
			HashSet<String> allTriplets = new HashSet<String>();
			u.maxClique = new HashSet<String>();
						
			Vector<Node> localNeighborNodes = null;
			Vector<NodeData> remoteNodesData = null;
			HashSet<String> nbrKeys = null;
			HashSet<String> alreadyFetchedKeys = null;
			nbrKeys  = new HashSet<String>(getNeighborIds(node));
			//first get the local neighbor nodes (complete node)
			localNeighborNodes = getLocalNeighbors(node);
			alreadyFetchedKeys = new HashSet<String>();
			for(Node n : localNeighborNodes) {
				alreadyFetchedKeys.add(n.nodeId);
			}
			nbrKeys.removeAll(alreadyFetchedKeys);
			remoteNodesData = getRemoteNeighborsData(new Vector<String>(nbrKeys), u.getClass().getName());
			
			//collecting all triplets from neighbor nodes
			for(Node n : localNeighborNodes){
				MCNode mc = (MCNode) n;
				allTriplets.addAll(mc.myTriplets);
			}
			for(NodeData nd : remoteNodesData){
				allTriplets.addAll((HashSet<String>) nd.properties.get("myTriplets"));
			}
			
			allTriplets.addAll(u.myTriplets);
			if(allTriplets.size()!=0){
				u.maxClique = calculateMaxClique(allTriplets, u.nodeId);
			}
			if(u.maxClique == null){
				u.maxClique = new HashSet<String>();
			}
			
			//System.out.println("Max-Clique for "+u.nodeId+" : "+u.maxClique.toString());
			u.TS = u.TS + 1;
			updatedNodes.put(u.nodeId, u);
			//rwsetinfo.writeSet.add(u.nodeId);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	private Vector<NodeData> getRemoteNeighborsData(Vector<String> nbrKeys, String className) throws Exception {

		Vector<NodeData> reqList = new Vector<NodeData>();
		NodeData nd;
		for(String key : nbrKeys) {
			try{
				Integer.parseInt(key);
			}
			catch(Exception e){
				System.out.println("Key : "+key);
			}
			nd = new NodeData(key, dataStorage.nodeClassNametoIDMap.get(className));
			nd.addProperty("neighbors", null);
			nd.addProperty("myTriplets", null);
			//nd.addProperty("marked", null);
			reqList.add(nd);
		}

		return dataStorage.getNodePropertiesParallel(reqList);
	}
	
	/*private Vector<NodeData> getRemoteNeighborsTriplets(Vector<String> nbrKeys) throws Exception {

		Vector<NodeData> reqList = new Vector<NodeData>();
		NodeData nd;
		for(String key : nbrKeys) {
			nd = new NodeData(Integer.parseInt(key));
			nd.addProperty("myTriplets", null);
			//nd.addProperty("marked", null);
			reqList.add(nd);
		}

		return dataStorage.getNodePropertiesParallel(reqList);
	}*/

	@Override
	public void run ()  {

		startTime  = System.currentTimeMillis();

		Task  myTask;
		WorkStatus ws;
		int phase = 0;
		int computephase = 0;

		boolean terminate = false;
		boolean relocated = false;
		String myURL = dataStorage.myURL;
		
		if(master) {
			try{
				MetaDataNode recomputeCliqueMetaData = new MetaDataNode(""+recomputeCliqueMetaDataId);
				recomputeCliqueMetaData.TS = recomputeCliqueMetaData.TS + 1;
				Vector<Node> node_batch = new Vector<Node>();
				node_batch.add(recomputeCliqueMetaData);
				HashtableServer htServer = dataStorage.getServer(""+recomputeCliqueMetaDataId);
				htServer.loadNodes(node_batch);
				
				Task t = new Task();
				t.phase = "STARTUP";
				t.nodeId="STARTUP_TASK";
				workpool.broadcast(t);
			} catch(Exception e) {
				e.printStackTrace();	
			}	
		}
		
		int maxphase=2;
		maxphase=configParams.getIntValue("MCWorker.MAX_PHASE")+1;
		int computeCliqueForNodesCount = 0;
		if(configParams.has("MCWorker.CALCULATE_CLIQUE_FOR_NODES_COUNT"))
			computeCliqueForNodesCount = configParams.getIntValue("MCWorker.CALCULATE_CLIQUE_FOR_NODES_COUNT");
		
		boolean writeresult=false;
		if(configParams.has("MCWorker.writeToFile"))
			writeresult=configParams.getBoolValue("MCWorker.writeToFile");
		

		while (true)  {
			try {
				ws = workpool.getTask();
				if (ws.status ==  StatusType.VALID_TASK) {
					myTask = ws.task;

					if (myTask == null) {
						System.out.println("Worker "+workerId+":  Error: Valid but Null task");
						continue;
					}
					else if(myTask.phase.equals("SHUTDOWN") ){
						phase = maxphase;
						workpool.progressStatusThread.stopPrinting = true;
						//System.out.println("Host "+dataStorage.myId+", Worker "+workerId+" received SHUTDOWN. Closing all operations after reaching phase "+phase);
						break;
					}
					else if(myTask.phase.equals("STARTUP")){
						if(master)
							System.out.println("Worker "+workerId+":  Startup task picked up by master itself");
						else
							System.out.println("Worker "+workerId+":  Startup task picked up by a non master.");
						
						createTasksAllVertices(myTask);
						System.out.println("Worker "+workerId+":  Created initial tasks");
					}
					else if(myTask.phase.equals("GENERIC")){
						doTask( myTask );
						//tasksCreated.clear();
					}
					else if(myTask.phase.equals("CREATE_MAX_CLIQUE_TASK")){
						if(computeCliqueForNodesCount==0)
							createCliqueComputeTaskAllLocalVertices(myTask);
						else{
							readCliqueComputeVerticesFromFile(computeCliqueForNodesCount);
							createCliqueComputeTaskForLocalVertices(myTask);
						}
					}
					else if (myTask.phase.equals("TERMINATION")) {
						//Stop task

						System.out.println("Worker "+workerId+":  Termination task");
						//System.out.println("Worker "+workerId+":  Calling verify()");
						//verify();
						//System.exit(0);
					}
					else{
						System.out.println("Worker "+workerId+":  Unknown phase");
					}


				}
				else if (ws.status == StatusType.NO_READY_TASK) {
					// Thread.sleep(configParams.getIntValue("Worker.THREADSLEEPTIME"));
				}
				else if (ws.status == StatusType.WORK_TERMINATED) {
					
					if(master&& (phase == maxphase) ) {
						//phaseEndTime[phase] = System.currentTimeMillis();
						//printResult(phase);
						
						//deleting the nodes that were to be deleted
						//System.out.println("Master removing the nodes to be deleted.");
						//deleteNodesToBeRemoved();
						
						System.out.println("Master Proceeding to cleanUp after reaching phase : " + phase);
						Task t = new Task();
						t.phase = "SHUTDOWN";
						t.nodeId = "STARTUP_TASK";
						for(int i=0;i<configParams.getIntValue("Worker.WORKER_COUNT");i++){
							workpool.broadcast(t);
						}
						cleanUp();
						workpool.progressStatusThread.stopPrinting = true;
						break;
					}
					else if( (master) && (phase < maxphase )){
						if(phase == computephase){
							computephase = computephase + 1;
							if(phase == 0){
								System.out.println("\nWorker "+workerId+":  Master creating COMPUTE_MAX_CLIQUE TASK\n");
								Task t = new Task();
								t.nodeId = "CREATE_MAX_CLIQUE_TASK";
								t.phase = "CREATE_MAX_CLIQUE_TASK";
								workpool.broadcast(t);
							}
							else{
								MetaDataNode recomputeCliqueMetaData = (MetaDataNode) dataStorage.getNode(""+recomputeCliqueMetaDataId);
								//System.out.println("recomputeCliqueMetaData.set.size() :: "+recomputeCliqueMetaData.set.size());
								if(recomputeCliqueMetaData.set.size()>0){	//check for recomputeClique
									System.out.println("\nWorker "+workerId+":: Master creating "+recomputeCliqueMetaData.set.size()+" recompute max_clique tasks...\n");
									Vector<Task> taskVector = new Vector<Task>();
									for(String nodeId : recomputeCliqueMetaData.set){
										CliqueComputeTask t1  = new CliqueComputeTask(nodeId);
										t1.phase = "GENERIC";
										String location = dataStorage.getLocation(nodeId);
										t1.targetLocation = location;
										t1.affinityLevel = TaskConstants.STRONG_AFFINITY;
										taskVector.add(t1);
									}
									
									NodeData nd = new NodeData(""+recomputeCliqueMetaDataId, dataStorage.nodeClassNametoIDMap.get(new MetaDataNode().getClass().getName()));
									nd.properties.put("clearSet", null);
									nd.TS = nd.TS + 1;
									dataStorage.exec(nd);
									
									workpool.addTask(taskVector);
								}
								else{
									System.out.println("Worker "+workerId+":: No node modified from Compute Clique Set. Master not creating recompute max_clique tasks.");
								}
							}
						}
						else{
							phaseEndTime[phase] = System.currentTimeMillis();
							if(writeresult)
								writeResult(phase);
							phase = phase + 1;
							if(phase<maxphase){
								Vector<UpdateGraphTask> vtasks = createGraphUpdateTasks( dataStorage.myId, phase );
								
								Vector<Task> finalVTasks = new Vector<Task>();
								for(int i=0;i<vtasks.size();i++){
									Task t = vtasks.get(i);
									if(t != null){
										finalVTasks.add(t);
									}
								}
								
								System.out.println("Master creating "+finalVTasks.size()+" new tasks...");
								phaseStartTime[phase] = System.currentTimeMillis();
								try {
									workpool.addTask(finalVTasks);
								} catch (RemoteException e) {
									e.printStackTrace();
								}
							}
							
						}
					}
				}				
			} catch (Exception e) {
				e.printStackTrace();
			}
		}   
	}
	
	public void cleanUp() {
		endTime = System.currentTimeMillis();
		long duration = endTime - startTime;
		
		long durationInitial = phaseEndTime[0] - startTime;
		long durationP1 = phaseEndTime[1] - phaseStartTime[1];
		long durationP2 = phaseEndTime[2] - phaseStartTime[2];
		long durationP3 = phaseEndTime[3] - phaseStartTime[3];
		long durationP4 = phaseEndTime[4] - phaseStartTime[4];
		

		System.out.println("*******************************");
		System.out.println("Total Execution Time taken : "+duration+" msec");
		System.out.println("Initial Computation Time taken : "+durationInitial+" msec");
		System.out.println("Phase 1 Time taken : "+durationP1+" msec");
		System.out.println("Phase 2 Time taken : "+durationP2+" msec");
		System.out.println("Phase 3 Time taken : "+durationP3+" msec");
		System.out.println("Phase 4 Time taken : "+durationP4+" msec");
		//String systemStats = "";
		try{
			//systemStats = dataStorage.getStats(true);
			FileOutputStream fout = new FileOutputStream("result.txt",true);
			FileLock lock = fout.getChannel().lock();    
			PrintWriter out = new PrintWriter(fout);
			out.println("==================================");
			out.println("Total Execution Time taken : "+duration+" msec");
			out.println("Initial Computation Time taken : "+durationInitial+" msec");
			out.println("Phase 1 Time taken : "+durationP1+" msec");
			out.println("Phase 2 Time taken : "+durationP2+" msec");
			out.println("Phase 3 Time taken : "+durationP3+" msec");
			out.println("Phase 4 Time taken : "+durationP4+" msec");
			out.println("==================================");
			//out.println(systemStats);
			//out.println("==================================");
			out.flush();
			lock.release();
			out.close();
		}catch(Exception e) {e.printStackTrace();}

		System.out.println("Worker "+workerId+":  Finished cleanup");

		System.out.println("*****************************");

		//System.out.println(systemStats);
	}
	
	public void writeResult(int phaseno){
		try{
			ConcurrentHashMap<Integer, HashtableServer> servers = dataStorage.servers;
			Collection<HashtableServer> serverCollection = servers.values();
			Vector<Node> l = new Vector<Node>();
			for(HashtableServer server:serverCollection){
				l.addAll(server.getAllNodes());
			}
			Collections.sort(l,new NodeComparator());
			
			String filename = "clique_result_"+phaseno+".txt";
			System.out.println("Writing results of clique computation to file "+filename+" ...");
			
			FileOutputStream fout = new FileOutputStream(filename,false);
			FileLock lock = fout.getChannel().lock();    
			PrintWriter out = new PrintWriter(fout);
			//out.println("==================================");
			
			StringBuffer sb = new StringBuffer();
			int i=0;
			for(Node n : l) {
				if(Integer.parseInt(n.nodeId)<0)
					continue;
				MCNode mc = (MCNode) n;
				synchronized(allComputeCliqueNodes){
					if(!allComputeCliqueNodes.contains(""+mc.nodeId))
						continue;
				}				
				sb.append(mc.nodeId);
				sb.append("  ");
				
				boolean first = true;
				Iterator it = mc.maxClique.iterator();
				while(it.hasNext()){
					String str = (String)it.next();
					if(first){
						String[] temp = str.split(",");
						sb.append(temp.length);
						sb.append("  ");
						first=false;
					}
					sb.append(str);
					sb.append(" ");
				}
				out.println(sb.toString());
				sb = new StringBuffer();
				i++;
				if(i%1000 == 0){
					out.flush();
				}
			}			
			
			//out.println("==================================");
			out.flush();
			lock.release();
			out.close();
			fout.close();
		}catch(Exception e) {e.printStackTrace();}

		System.out.println("Worker "+workerId+":  Finished writing to file");
		System.out.println("*****************************");

	}
	
	public void createCliqueComputeTaskAllLocalVertices(Task tsk) throws Exception{
		Iterator it = dataStorage.servers.keySet().iterator();
		while(it.hasNext()){
			int id = (Integer)it.next();
			Vector<String> myTempLocalKeys = dataStorage.servers.get(id).getAllKeys();
			synchronized(allComputeCliqueNodes){
				allComputeCliqueNodes.addAll(myTempLocalKeys);
			}
		}
		
		Vector<String> myLocalKeys = dataStorage.getAllKeys(myStorageURL);
		Vector<Task> taskVector = new Vector<Task>();
		int batch_size = 1000;
		for (String key: myLocalKeys) {
			if(key.equals(Integer.toString(numNodes)) || Integer.parseInt(key)<0)
				continue;
			synchronized (computeCliqueNodesSet) {
				computeCliqueNodesSet.add(key);
			}
			CliqueComputeTask t1  = new CliqueComputeTask(key);
			t1.phase = "GENERIC";
			String location = dataStorage.getLocation(key);
			t1.targetLocation = location;
			t1.affinityLevel = TaskConstants.STRONG_AFFINITY;
			taskVector.add(t1);
			
			if(taskVector.size() == batch_size) {
				workpool.addTask(taskVector);
				taskVector.clear();
			}
		}
		if(taskVector.size() > 0) {
			workpool.addTask(taskVector);
		}
		workpool.removeTask(tsk);
	}
	
	public void createCliqueComputeTaskForLocalVertices(Task tsk) throws Exception{

		Vector<Task> taskVector = new Vector<Task>();
		int batch_size = 1000;
		synchronized (computeCliqueNodesSet) {
			for (String key: computeCliqueNodesSet) {
				if(key.equals(Integer.toString(numNodes)) || Integer.parseInt(key)<0)
					continue;

				CliqueComputeTask t1  = new CliqueComputeTask(key);
				t1.phase = "GENERIC";
				String location = dataStorage.getLocation(key);
				t1.targetLocation = location;
				t1.affinityLevel = TaskConstants.STRONG_AFFINITY;
				taskVector.add(t1);
				if(taskVector.size() == batch_size) {
					workpool.addTask(taskVector);
					taskVector.clear();
				}
			}
		}
		if(taskVector.size() > 0) {
			workpool.addTask(taskVector);
		}
		workpool.removeTask(tsk);
	}
	
	public void readCliqueComputeVerticesFromFile(int count){
		try{
			String line = null;
			BufferedReader br = new BufferedReader(new FileReader(configParams.getStringValue("MCWorker.READ_CLIQUE_COMPUTE_VERTICES_FROM_FILE")));
			HashSet<String> nodeIdReadFromFile = new HashSet<String>();
			int i=0;
			while( (line = br.readLine()) != null) {
				if(!line.equals("")){
					nodeIdReadFromFile.add(line.trim());
					i++;
				}
				if(i==count)
					break;
			}
			br.close();
			if(master){
				synchronized(allComputeCliqueNodes){
					allComputeCliqueNodes.addAll(nodeIdReadFromFile);
				}
			}
				
			for(String nd : nodeIdReadFromFile){
				if(dataStorage.myHashTableServer.contains(nd)){
					synchronized (computeCliqueNodesSet) {
						computeCliqueNodesSet.add(nd);
					}
				}					
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	public void createCliqueComputeTaskForVertices(Task tsk) throws Exception{
		Vector<Task> taskVector = new Vector<Task>();
		int batch_size = 1000;
		synchronized (computeCliqueNodesSet) {
			for (String key: computeCliqueNodesSet) {
				CliqueComputeTask t1  = new CliqueComputeTask(key);
				t1.phase = "GENERIC";
				String location = dataStorage.getLocation(key);
				t1.targetLocation = location;
				t1.affinityLevel = TaskConstants.STRONG_AFFINITY;
				taskVector.add(t1);
				if(taskVector.size() == batch_size) {
					workpool.addTask(taskVector);
					taskVector.clear();
				}
			}
		}
		if(taskVector.size() > 0) {
			workpool.addTask(taskVector);
		}
		workpool.removeTask(tsk);
	}
	
	public void createTasksAllVertices(Task tsk) throws Exception{

		Vector<String> myLocalKeys = dataStorage.getAllKeys(myStorageURL);
		Vector<Task> taskVector = new Vector<Task>();
		int batch_size = 1000;
		for (String key: myLocalKeys) {
			if(key.equals(Integer.toString(numNodes)) || Integer.parseInt(key)<0)
				continue;
			Task t = new Task(key);
			t.phase = "GENERIC";
			taskVector.add(t);
			if(taskVector.size() == batch_size) {
				workpool.addTask(taskVector);
				taskVector.clear();
			}
			//Thread.sleep(1);
		}
		if(taskVector.size() > 0) {
			workpool.addTask(taskVector);
		}
		workpool.removeTask(tsk);
	}
	
	public Vector<UpdateGraphTask> createGraphUpdateTasks  ( int hostId, int phaseno ) {
		//printResult(phaseno-1);
		//only master will print this line
		if(master)
			System.out.println("\nAdding new changes in the graph...\n");
		//loadGenerator
		Vector<UpdateGraphTask> vtasks = new Vector<UpdateGraphTask>();
		int countAddEdgeOperations=0, countDeleteEdgeOperations=0;
		//int countUpdateEdgeOperations=0;
		//int countAddNodeOperations=0, countDeleteNodeOperations=0; 
		boolean deleteEdgeUsingFile = false;
		boolean showOutput=false;
		//int countMaxNeighbors=0;
		String deleteEdgeUsingFileName = "";
		int maxEdgeLength=100;
		int numServers = dataStorage.getNumberOfServers();
		countAddEdgeOperations = configParams.getIntValue("MCWorker.ADD_EDGE")/*/numServers*/;
		countDeleteEdgeOperations = configParams.getIntValue("MCWorker.DELETE_EDGE")/*/numServers*/;
		//countUpdateEdgeOperations = configParams.getIntValue("MCWorker.UPDATE_EDGE")/*/numServers*/;
		//countAddNodeOperations = configParams.getIntValue("MCWorker.ADD_NODE")/numServers;
		//countDeleteNodeOperations = configParams.getIntValue("MCWorker.DELETE_NODE")/numServers;
		deleteEdgeUsingFile = configParams.getBoolValue("MCWorker.DELETE_EDGE_USING_FILE");
		showOutput = configParams.getBoolValue("MCWorker.SHOW_OUTPUT");
		//countMaxNeighbors = configParams.getIntValue("MCWorker.MAX_NEIGHBORS");
		maxEdgeLength = configParams.getIntValue("MCWorker.EDGE_MAX_LENGTH");
		if(deleteEdgeUsingFile){
			deleteEdgeUsingFileName = configParams.getStringValue("MCWorker.DELETE_EDGE_USING_FILENAME");
		}
		
		Random r = new Random();
		StringBuffer strbuf = new StringBuffer();
		UpdateGraphTask t=null;
		
		switch(phaseno){
		case 1: //add Edge
			for(int i=0;i<countAddEdgeOperations;i++){
				int src = r.nextInt(this.numNodes);
				int dest = r.nextInt(this.numNodes);
				while(dest==src){
					dest = r.nextInt(this.numNodes);
				}
				int len=r.nextInt(maxEdgeLength)+1;
				t = addEdge(""+src,""+dest,len);
				if(t==null){
					i--;
				}
				else{
					vtasks.add(t);
					strbuf.append("Host "+dataStorage.myId+" :: Adding edge between Node "+src+" and "+dest+"\n");
				}
			}
			/*t = addEdge("0","9", 1);
			vtasks.add(t);
			t = addEdge("7","8", 1);
			vtasks.add(t);
			t = addEdge("11","12", 1);
			vtasks.add(t);
			t = addEdge("7","9");
			vtasks.add(t);
			t = addEdge("7","10");
			vtasks.add(t);
			t = addEdge("7","11");
			vtasks.add(t);
			t = addEdge("10","14");
			vtasks.add(t);
			t = addEdge("12","8");
			vtasks.add(t);
			t = addEdge("6","13");
			vtasks.add(t);
			t = addEdge("1","5");
			vtasks.add(t);*/
			break;
		case 2: //delete Edge
			if(deleteEdgeUsingFile && (countDeleteEdgeOperations!=0)){
				//delete Edges read from file upto the count value
				int i=0;
				String line = null;
				int src = 0, dest = 0;
				try{
					BufferedReader br = new BufferedReader(new FileReader(deleteEdgeUsingFileName));
					while( (line = br.readLine()) != null) {
						String str[] = line.split(",");
						src=Integer.parseInt(str[0]);
						/*if((src%numServers)!=dataStorage.myId)
							continue;*/
						dest=Integer.parseInt(str[1]);
						t = deleteEdge(""+src,""+dest);
						if(t==null){
							System.out.println("Task is null for delete edge scenario! Please check edge list file...");
							//i is not incremented
						}
						else{
							vtasks.add(t);
							i++;
							strbuf.append("Host "+dataStorage.myId+" :: Deleting edge between Node "+src+" and "+dest+"\n");
						}
						if(i>=countDeleteEdgeOperations){
							break;
						}
					}
					br.close();
				}
				catch(Exception e){
					e.printStackTrace();
				}			
			}
			else{
				//randomly delete edges
				for(int i=0;i<countDeleteEdgeOperations;i++){
					int src = r.nextInt(this.numNodes);
					int dest = r.nextInt(this.numNodes);
					while(dest==src){
						dest = r.nextInt(this.numNodes);
					}
					t = deleteEdge(""+src,""+dest);
					if(t==null){
						i--;
					}
					else{
						vtasks.add(t);
						strbuf.append("Host "+dataStorage.myId+" :: Deleting edge between Node "+src+" and "+dest+"\n");
					}
				}
			}
			/*t = deleteEdge("0","9");
			vtasks.add(t);
			t = deleteEdge("26","27");
			vtasks.add(t);
			t = deleteEdge("3","4");
			vtasks.add(t);
			t = deleteEdge("4","5");
			vtasks.add(t);
			t = deleteEdge("5","6");
			vtasks.add(t);
			t = deleteEdge("6","7");
			vtasks.add(t);
			t = deleteEdge("7","8");
			vtasks.add(t);
			t = deleteEdge("8","9");
			vtasks.add(t);
			t = deleteEdge("9","10");
			vtasks.add(t);
			t = deleteEdge("10","11");
			vtasks.add(t);
			t = deleteEdge("11","12");
			vtasks.add(t);
			t = deleteEdge("12","13");
			vtasks.add(t);
			t = deleteEdge("13","14");
			vtasks.add(t);
			t = deleteEdge("14","0");
			vtasks.add(t);*/
			break;
		}
		
		if(showOutput){
			System.out.println(strbuf);
		}
		
		return vtasks;
	}
	
	public UpdateGraphTask addEdge(String sourceNode, String destNode, int edgeLength){
		Vector<Object> params = new Vector<Object>();
		params.add(destNode);
		params.add(new Integer(edgeLength));
		UpdateGraphTask t = null;
		
		MCNode srcNode;
		try {
			srcNode = (MCNode) dataStorage.getNode(sourceNode);
			if(srcNode.neighbors.containsKey(destNode)){
				t = null;
			}
			else{
				t = new UpdateGraphTask(sourceNode, MCTaskConstants.addEdge, params);
				String location = dataStorage.getLocation(sourceNode);
				t.targetLocation = location;
				t.affinityLevel = TaskConstants.STRONG_AFFINITY;
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		return t;
	}
	
	public UpdateGraphTask deleteEdge(String sourceNode, String destNode){
		Vector<Object> params = new Vector<Object>();
		params.add(destNode);
		UpdateGraphTask t = null;
		
		MCNode srcNode;
		try {
			srcNode = (MCNode) dataStorage.getNode(sourceNode);
			if(srcNode.neighbors.containsKey(destNode)){
				t = new UpdateGraphTask(sourceNode, MCTaskConstants.deleteEdge, params);
				String location = dataStorage.getLocation(sourceNode);
				t.targetLocation = location;
				t.affinityLevel = TaskConstants.STRONG_AFFINITY;
			}
			else{
				t = null;
			}
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		return t;
	}
	
	public HashSet<String>  updateGraph(Task T, Node node){
		MCNode tgtNode=null;
		if(node != null)
			tgtNode = new MCNode((MCNode)node); 	//you need a copy of orig node
		
		UpdateGraphTask t = (UpdateGraphTask)T;
		HashSet<String> tempRecomputeCliqueNodes = new HashSet<String>();
		
		try{
			switch(t.opname){
			case MCTaskConstants.addEdge:
				{
					String nbrid = (String) t.params.get(0);
					Integer edge = (Integer)t.params.get(1);
					int edgelength = edge.intValue();
					//System.out.println("inside addEdge:: node : "+node.nodeId+", nbrid : "+nbrid);
					MCNode nbrNode = new MCNode((MCNode) dataStorage.getNode(nbrid));
					
					MCEdge edgeToNbr = new MCEdge(edgelength);
					nbrNode.addNeighbor(tgtNode.nodeId, edgeToNbr);
					MCEdge edgeToTgt = new MCEdge(edgelength);
					tgtNode.addNeighbor(nbrNode.nodeId, edgeToTgt);
					//System.out.println("addEdge("+tgtNode.nodeId+","+nbrNode.nodeId+") :: after adding edge, neighbors for tgtNode and nbrNode : "+tgtNode.neighbors.keySet().toString()+", "+nbrNode.neighbors.keySet().toString());
					
					tgtNode.TS = tgtNode.TS + 1;
					nbrNode.TS = nbrNode.TS + 1;
					
					updatedNodes.put(tgtNode.nodeId, tgtNode);
					updatedNodes.put(nbrNode.nodeId, nbrNode);
					
					HashMap<String, String> newTriples = new HashMap<String, String>();
					HashSet<String> neighborsToRead = new HashSet<String>();
					try{
						//calculating new triplets
						//System.out.println("addEdge("+tgtNode.nodeId+","+nbrNode.nodeId+") :: neighbors for tgtNode and nbrNode : "+tgtNode.neighbors.keySet().toString()+", "+nbrNode.neighbors.keySet().toString());
						Iterator it = tgtNode.neighbors.keySet().iterator();
						while(it.hasNext()){
							String tempNbr = (String)it.next();
							if(nbrNode.neighbors.containsKey(tempNbr)){
								//System.out.println("addEdge("+tgtNode.nodeId+","+nbrNode.nodeId+") :: common neighbor : "+tempNbr);
								int x[] = new int[3];
								x[0]=Integer.parseInt(tgtNode.nodeId);
								x[1]=Integer.parseInt(nbrNode.nodeId);
								x[2]=Integer.parseInt(tempNbr);
								Arrays.sort(x);
								String strtemp = Arrays.toString(x).replaceAll("[\\[\\] ]", "").replaceAll(",", ";");
								newTriples.put(tempNbr,strtemp);
								neighborsToRead.add(tempNbr);
								//adding new triplets to tgtNode and nbrNode
								tgtNode.myTriplets.add(strtemp);
								nbrNode.myTriplets.add(strtemp);
							}
						}
						
						//creating task for neighbos to add the new triple to their myTriples set
						Iterator<String> iter = neighborsToRead.iterator();
						while(iter.hasNext()){
							String nbrId = (String) iter.next();
							String tripleToAdd = newTriples.get(nbrId);
							
							UpdateTripletsTask t1  = new UpdateTripletsTask(nbrId,tripleToAdd,MCTaskConstants.addToMyTriplets);
							String location = dataStorage.getLocation(nbrId);
							t1.targetLocation = location;
							t1.affinityLevel = TaskConstants.STRONG_AFFINITY;
							tasksCreated.add(t1);
						}
						
						//the triplets for a node can change only if the node is a part of the new triplet formed.
						//So, checking if any of the three nodes is a part of allComputeCliqueNodes, then a task should be created to recompute the clique for the required node.
						synchronized (allComputeCliqueNodes) {
							if(allComputeCliqueNodes.contains(tgtNode.nodeId)){
								tempRecomputeCliqueNodes.add(tgtNode.nodeId);
							}
							if(allComputeCliqueNodes.contains(nbrNode.nodeId)){
								tempRecomputeCliqueNodes.add(nbrNode.nodeId);
							}
							for(String nodeId : neighborsToRead){
								if(allComputeCliqueNodes.contains(nodeId)){
									tempRecomputeCliqueNodes.add(nodeId);
								}
							}
						}
					}
					catch(Exception e)
					{
						e.printStackTrace();
						System.exit(1);
					}
				}
				break;
			case MCTaskConstants.deleteEdge:
				{
					String nbrid = (String) t.params.get(0);
					//System.out.println("inside deleteEdge:: node : "+node.nodeId+", nbrid : "+nbrid);
					MCNode nbrNode = new MCNode((MCNode) dataStorage.getNode(nbrid));
					//System.out.println("deleteEdge("+tgtNode.nodeId+","+nbrNode.nodeId+") :: before remove, neighbors for tgtNode and nbrNode : "+tgtNode.neighbors.keySet().toString()+", "+nbrNode.neighbors.keySet().toString());
					
					nbrNode.neighbors.remove(""+tgtNode.nodeId);
					tgtNode.neighbors.remove(""+nbrNode.nodeId);
					
					//System.out.println("deleteEdge("+tgtNode.nodeId+","+nbrNode.nodeId+") :: after remove, neighbors for tgtNode and nbrNode : "+tgtNode.neighbors.keySet().toString()+", "+nbrNode.neighbors.keySet().toString());
					
					tgtNode.TS = tgtNode.TS + 1;
					nbrNode.TS = nbrNode.TS + 1;
					
					updatedNodes.put(tgtNode.nodeId, tgtNode);
					updatedNodes.put(nbrNode.nodeId, nbrNode);
					
					HashMap<String, String> affectedTriples = new HashMap<String, String>();
					HashSet<String> neighborsToRead = new HashSet<String>();
					try{
						//calculating new triplets
						Iterator it = tgtNode.neighbors.keySet().iterator();
						while(it.hasNext()){
							String tempNbr = (String)it.next();
							if(nbrNode.neighbors.containsKey(tempNbr)){
								//System.out.println("deleteEdge("+tgtNode.nodeId+","+nbrNode.nodeId+") :: common neighbor : "+tempNbr);
								int x[] = new int[3];
								x[0]=Integer.parseInt(tgtNode.nodeId);
								x[1]=Integer.parseInt(nbrNode.nodeId);
								x[2]=Integer.parseInt(tempNbr);
								Arrays.sort(x);
								String strtemp = Arrays.toString(x).replaceAll("[\\[\\] ]", "").replaceAll(",", ";");
								affectedTriples.put(tempNbr,strtemp);
								neighborsToRead.add(tempNbr);
								//adding new triplets to tgtNode and nbrNode
								//System.out.println("deleteEdge("+tgtNode.nodeId+","+nbrNode.nodeId+") :: removing "+strtemp+" : before removing : "+tgtNode.myTriplets+", "+nbrNode.myTriplets);
								tgtNode.myTriplets.remove(strtemp);
								nbrNode.myTriplets.remove(strtemp);
								//System.out.println("deleteEdge("+tgtNode.nodeId+","+nbrNode.nodeId+") :: removing "+strtemp+" : after removing : "+tgtNode.myTriplets+", "+nbrNode.myTriplets);
								
							}
						}
						
						//creating task for neighbors to remove the triple from their myTriples set
						Iterator<String> iter = neighborsToRead.iterator();
						while(iter.hasNext()){
							String nbrId = (String) iter.next();
							String tripleToRemove = affectedTriples.get(nbrId);
							
							UpdateTripletsTask t1  = new UpdateTripletsTask(nbrId,tripleToRemove,MCTaskConstants.deleteFromMyTriplets);
							String location = dataStorage.getLocation(nbrId);
							t1.targetLocation = location;
							t1.affinityLevel = TaskConstants.STRONG_AFFINITY;
							tasksCreated.add(t1);
						}
						
						//the triplets for a node can change only if the node is a part of the new triplet formed.
						//So, checking if any of the three nodes is a part of allComputeCliqueNodes, then a task should be created to recompute the clique for the required node.
						synchronized (allComputeCliqueNodes) {
							if(allComputeCliqueNodes.contains(tgtNode.nodeId)){
								tempRecomputeCliqueNodes.add(tgtNode.nodeId);
							}
							if(allComputeCliqueNodes.contains(nbrNode.nodeId)){
								tempRecomputeCliqueNodes.add(nbrNode.nodeId);
							}
							for(String nodeId : neighborsToRead){
								if(allComputeCliqueNodes.contains(nodeId)){
									tempRecomputeCliqueNodes.add(nodeId);
								}
							}
						}
					}
					catch(Exception e)
					{
						e.printStackTrace();
						System.exit(1);
					}
				}
				break;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}
		return tempRecomputeCliqueNodes;
	}

	public class NodeComparator implements Comparator<Node>{
		public int compare(Node n1, Node n2){
			if(Integer.parseInt(n1.nodeId) > Integer.parseInt(n2.nodeId)) return 1;
			else return -1;
		}
	}
}