package TestProgs.PageRankV2;

import java.util.*;
import java.io.*;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;

import beehive.util.*;

public class InputFileReader {

	StorageSystem storage;
	String fileName;	
	int numNodes;
	public Vector<String> totalNodes;
	public Vector<String> nodesCreated;
	Hashtable<String, Node> table;
	boolean debug = false;

	public InputFileReader (String filename, int numNodes, StorageSystem storage) {
		this.fileName = filename;
		this.storage = storage;
		this.numNodes = numNodes;
		double tau = (double)1/numNodes;
		double rank = (double)1/numNodes;
        table = new Hashtable<String, Node>();
        for(int i=0; i< numNodes; i++) {
            PRNode n = new PRNode(""+i,rank,tau);
            String nodeKey  =  Integer.toString(i);
            table.put(nodeKey, n);
        }
	}		

	public void readAndLoadData() throws IOException {
		totalNodes = new Vector<String>();
		nodesCreated = new Vector<String>();
		
		BufferedReader br = new BufferedReader(new FileReader(fileName));
		String line = null;
		double tau = (double)1/numNodes;
		double rank = (double)1/numNodes;
	
		while( (line = br.readLine()) != null) {
			StringTokenizer tok = new StringTokenizer(line," ");
			String currentNodeKey = tok.nextToken();  
			nodesCreated.add(currentNodeKey);	
			if (!totalNodes.contains(currentNodeKey)) {
				totalNodes.add(currentNodeKey);	
			}
			PRNode currentNode  = new PRNode(currentNodeKey,rank, tau);            
			if (table.containsKey(currentNodeKey)) {
				currentNode = (PRNode)table.get(currentNodeKey);

            }
			//update the neighbor list for this node;
			String nbrList = "";
			if (tok.hasMoreTokens()){ 
				nbrList = tok.nextToken();
			} else
				continue;
			
			StringTokenizer tok2 = new StringTokenizer(nbrList,",");
			// loop over the list of neighbours
			while(tok2.hasMoreTokens()) {
				String neighborInfo = tok2.nextToken();
				StringTokenizer tok3 = new StringTokenizer(neighborInfo,":");
				int nbrId = Integer.parseInt(tok3.nextToken());  /* Neighbor Id */
                		
				String nbrKey = Integer.toString( nbrId );
				if (!totalNodes.contains(currentNodeKey)) {
					totalNodes.add(currentNodeKey);	
				}
					
				
				//int cap = Integer.parseInt(tok3.nextToken());
				//forward edge
	            Edge toNbr =  new Edge();
	            //toNbr.capacity = cap; 
           		//toNbr.flow  = 0;
				currentNode.addNeighbor(nbrKey, toNbr);

				/* If the neighbor alread has an edge for the current node */				
				PRNode neighborNode = (PRNode) table.get(nbrKey);
				Edge fromNbr = new Edge();
                neighborNode.addBackNeighbor(currentNodeKey, fromNbr);
                table.put(nbrKey, neighborNode);
				
			}
            table.put(currentNodeKey, currentNode);				
		    if (debug) 	
			  System.out.println("Added node: "+ currentNodeKey);
		}

		int ctr=0;
        Enumeration<String> enumKeys = table.keys();
        Vector<Node> batch = new Vector<Node>();
        while (enumKeys.hasMoreElements()) {
            String key = enumKeys.nextElement();
			PRNode node = (PRNode)table.get(key);
            batch.add(node);
            if (batch.size() == 1000) {
                storage.putNodesParallel(batch);
                batch = new Vector<Node>();
            }
            ctr++;
            if (ctr % 1000==0) {
              System.out.println(new Date()+" total nodes put:"+ctr);
            }
        }
        if (batch.size() > 0) {
            storage.putNodesParallel(batch);
        }
         System.out.println(new Date()+" total nodes put:"+ctr);

	}

    

	public void printTrace() {
		for (String nodeId : nodesCreated) {
			try {
			   Node node = storage.getNode(nodeId);
			   System.out.println("Node "+ node.printNodeEdges());
			} catch(Exception e) {
			   System.out.println("Exception encountered while fetching node "+nodeId);
			   e.printStackTrace();	
			}
		}
	}	
	
}

