package TestProgs.PageRankV2;

import java.util.*;
import java.io.*;
import java.rmi.RMISecurityManager;
import java.rmi.Naming;
import java.net.*;

import beehive.util.*;
import beehive.server.*;	

public class InputFileReaderBatchV2 {

	StorageSystem storage;
	String fileName;	
	// public Vector<String> totalNodes;
	// public Vector<String> nodesCreated;
	Hashtable<String, Node> table;
	boolean debug = false;
	int lastFlushCtr = 0;
	static int BATCH_SIZE = 10;
	int numNodes;
	public InputFileReaderBatchV2 (String filename, int numNodes, StorageSystem storage) {
		this.fileName = filename;
		this.storage = storage;
        table = new Hashtable<String, Node>();
		this.numNodes = numNodes;
	}		

	public void readAndLoadData() throws IOException {

		double tau = (double)1/numNodes;
		double rank = (double)1/numNodes;
		
		BufferedReader fwdReader = new BufferedReader(new FileReader(fileName));
		BufferedReader backReader = new BufferedReader(new FileReader(fileName+".bp"));

		String line = null;
        String backline = null;

        backline = backReader.readLine();
		StringTokenizer baktok = new StringTokenizer(backline," ");
		int currentBackNode = Integer.parseInt(baktok.nextToken());
        
        
		int ctr = 0;
		while( (line = fwdReader.readLine()) != null) {
		  ctr++;
		  if ((ctr-1) % BATCH_SIZE==0 && ctr!=1 ) {
			System.out.println(new Date()+" total nodes processed:"+ (ctr-1));
			flushTable();
		  }
			StringTokenizer tok = new StringTokenizer(line," ");
			int currentNodeId = Integer.parseInt(tok.nextToken());
            String currentNodeKey = Integer.toString( currentNodeId);  
				
			PRNode currentNode  = new PRNode(currentNodeKey, rank, tau);            
			// first add all the fwd edges if any
			String nbrList = "";
			if (tok.hasMoreTokens()) { 
				nbrList = tok.nextToken(); 
				StringTokenizer tok2 = new StringTokenizer(nbrList,",");
				// loop over the list of neighbours
				while(tok2.hasMoreTokens()) {
					String neighborInfo = tok2.nextToken();
					StringTokenizer tok3 = new StringTokenizer(neighborInfo,":");
					String nbrId = tok3.nextToken();  /* Neighbor Id */
	            	Edge fwdEdge =  new Edge();
					currentNode.addNeighbor(nbrId, fwdEdge);				
				}
			}

            /* Check if the currentBackNode is the same as the curentNode.      */
            /*  In that case add back edges from the current node.          */
		
			if (currentNodeId > currentBackNode) {	// advance back reader pointer if its behind
				backline = backReader.readLine();
				if (backline != null) {
					baktok = new StringTokenizer(backline," ");
					currentBackNode = Integer.parseInt(baktok.nextToken());				
				}
			} 
			if (currentNodeId == currentBackNode) {			
				String backNbrList = "";
				if (baktok.hasMoreTokens()) {
					backNbrList = baktok.nextToken();
					StringTokenizer baktok2 = new StringTokenizer(backNbrList,",");
					while(baktok2.hasMoreTokens()) {
						String neighborInfo = baktok2.nextToken();
						StringTokenizer baktok3 = new StringTokenizer(neighborInfo,":");
						int nbrId = Integer.parseInt(baktok3.nextToken());  /* Back Neighbor Id */               				
						Edge backEdge = new Edge();
						currentNode.addBackNeighbor(""+nbrId, backEdge);	
					}	
				}	        	
			}

            table.put(currentNodeKey, currentNode);				
		    if (debug) 	
			  System.out.println("Added node: "+ currentNodeKey);
			
		}

		System.out.println(new Date()+ " All nodes processed total nodes:"+ ctr);
		if (!table.isEmpty()) {
			Vector<Node> batch = new Vector<Node>();
			batch.addAll(table.values());
			System.out.println("Flushing the last batch of "+batch.size()+" nodes");
			try {
				storage.putNodesParallel(batch);
			} catch(Exception e) {e.printStackTrace();}
		}
	}	
    

	private void flushTable() {			
        Enumeration<String> enumKeys = table.keys();
		Vector<Node> batch = new Vector<Node>();
		int ctr = lastFlushCtr;	
		for(int i = 0; i<BATCH_SIZE; i++) {
 			String key = Integer.toString(ctr);
			Node node = table.remove(key);
			batch.add(node);
		    ctr++;
        }
		System.out.println("Flushing from "+lastFlushCtr+" to "+ (ctr-1));
		if (batch.size() > 0) {
			try {
				storage.putNodesParallel(batch);			
			} catch(Exception e) { e.printStackTrace();}
		}
		System.out.println(new Date()+" total nodes put till now:"+ctr);
		lastFlushCtr = ctr;
	}

	public void printTrace() {
        for (int i=0; i<numNodes;i++) {
			String nodeId = Integer.toString(i);
            try {
               Node node = storage.getNode(nodeId);
               System.out.println("Node "+ node.printNodeEdges());
            } catch(Exception e) {
               System.out.println("Exception encountered while fetching node "+nodeId);
               e.printStackTrace();
            }
        }
    }

	/*public static void main(String args[]) throws Exception {	// just for testing
		System.setSecurityManager (new RMISecurityManager ());
    	//String myHostname = InetAddress.getLocalHost().getCanonicalHostName();	
    	String myHostname = InetAddress.getLocalHost().getHostName();	

		String location = "//"+myHostname+"/HashtableServer";
    	HashtableServerImpl HashtableNode = new HashtableServerImpl(location);
		Vector<String> hostList = new Vector<String>();
		hostList.add(myHostname);
		StorageSystem dataStorage = new StorageSystem(hostList, HashtableNode, configParams);
		String inputFile = args[0];
		int nodes = Integer.parseInt(args[1]);
		InputFileReaderBatchV2 reader = new InputFileReaderBatchV2(inputFile, nodes, dataStorage);
		reader.readAndLoadData();
		reader.printTrace();	
	} 	*/
}

