package TestProgs.PageRankV2;

import beehive.util.*;
import java.util.*;
import java.io.*;

public class PRNode  extends Node implements Serializable  {

	double[] rank;
	double tau;
	double ssRank;

	public Hashtable<String, Edge> backNeighbors;	

	public PRNode(String id, double tau, double rank) {
		super( id );
		backNeighbors = new Hashtable<String,Edge>();
		this.rank = new double[2];
		this.rank[0] = rank;
		this.rank[1] = rank;
		this.ssRank = rank;
		this.tau = tau;

	}

	public PRNode(PRNode n) {
		super(n.nodeId);
		this.tau = n.tau;
		this.rank = new double[2];
		this.rank[0]=n.rank[0];
		this.rank[1]=n.rank[1];
		this.ssRank = n.ssRank;
		backNeighbors = new Hashtable<String, Edge>();
		Enumeration edgeKeys = n.neighbors.keys();
		while(edgeKeys.hasMoreElements())
		{
			String ekey = (String)edgeKeys.nextElement();
			Edge e = (Edge)n.neighbors.get(ekey);
			Edge f = new Edge();
			neighbors.put(ekey,f);	
		}

		edgeKeys = n.backNeighbors.keys();
		while(edgeKeys.hasMoreElements())
		{
			String ekey = (String)edgeKeys.nextElement();
			Edge e = (Edge)n.backNeighbors.get(ekey);
			Edge f = new Edge();
			backNeighbors.put(ekey,f);	
		}

	}	

	public void addBackNeighbor(String key, Edge edge) {
		this.backNeighbors.put(key, edge);
	}

	public String printNode () {
		StringBuffer str =  new StringBuffer();
		str.append("Node id: "+ nodeId +" Rank 0: "+rank[0]+" Rank 1: "+rank[1]+" SSRank: "+ssRank);
		return  str.toString();
	}

	public String toString() {
		return printNode();
	}

	public String printNodeEdges () {
		StringBuffer str =  new StringBuffer();
		str.append("Node id: "+ nodeId +" Rank 0:"+rank[0]+" Rank 1:"+rank[1] +" SSRank: "+ssRank);
		Enumeration edges = neighbors.keys();
		str.append(" Outgoing Edges: ");
		while (edges.hasMoreElements()) {
			String key = (String) edges.nextElement();
			Edge e = (Edge) neighbors.get(key);
			str.append(e.toString()+" ");
		}
		str.append(" Incoming Edges: ");
		edges = backNeighbors.keys();
		while (edges.hasMoreElements()) {
			String key = (String) edges.nextElement();
			Edge e = (Edge) backNeighbors.get(key);
			str.append(e.toString()+" ");
		}


		return  str.toString();
	}

	public Double fractionalRank(Integer phase) {
		int degree = neighbors.size();
		
		double neighborRank;
		
		if(phase.intValue()==0) {
			neighborRank = rank[1];
		}else {
			neighborRank = rank[0];
		}
			
		double frac = (neighborRank/(double)degree);
		
		return new Double(frac);
	}

}





