package TestProgs.PageRankV2;

import  beehive.server.*;
import  beehive.util.*;
import beehive.worker.*;
import beehive.workpool.*;

import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.Naming;
import java.rmi.registry.Registry;
import java.util.*;

import java.io.*;
import java.net.*;

public class PageRankTest  {

	public static void main (String args[]) throws Exception {
		int myId = 0;
		if (args.length != 6) {
			throw new RuntimeException ("Syntax: PageRankTest <machine list> <config params file> <workpoolserver hostname>  <input filename> <num_nodes> <Exp Desc File>");
		}
		String machineList = args[0];
		System.setSecurityManager (new RMISecurityManager ());
		String myHostname = InetAddress.getLocalHost().getCanonicalHostName();	
		//String myHostname = InetAddress.getLocalHost().getHostName();	

		/***  Code added for porting to 3.7.7.2   */
		Registry localRegistry = null;
		try {
			localRegistry = LocateRegistry.createRegistry(1099);
		} catch (RemoteException e) {
			System.out.println("Couldn't Create Registry");
			e.printStackTrace();
		}

		System.setSecurityManager (new RMISecurityManager ());

		/******* */

		//Initializing Config Parameters
		String configParamsFile = args[1];
		Config configParams = new Config(configParamsFile);

		//get the caching flag
		boolean useCaching = configParams.getBoolValue("Server.useCaching");
		boolean recordStats = configParams.getBoolValue("StorageSystem.recordStats");
		//get the reflection flag for property get/update
		boolean useReflection = configParams.getBoolValue("StorageSystem.useReflection");
		boolean writeThroughCache = configParams.getBoolValue("StorageSystem.WriteThroughCache");

		boolean master = false;
		boolean first = true;
		Vector<String> hostList = new Vector<String>();
		try {
			BufferedReader bufferedReader = new BufferedReader(new FileReader(machineList));
			String host;
			int hostCount = 0;
			while ((host = bufferedReader.readLine()) != null) {
				if(first){
					if(host.equals(myHostname))
					{	
						master = true;
						System.out.println("Master machine");
					}
					first = false;
				}
				hostList.add(host);
				if (host.equals(myHostname)) {
					myId = hostCount;
				}
				hostCount++;
			}
		} catch(Exception e){
			e.printStackTrace();
			System.out.println("Error reading config file");
			System.exit(1);
		}

		//All inherited Node classes used
		HashMap<Byte, String> nodeClassNameMap = new HashMap<Byte, String>();
		HashMap<Byte, Class> nodeClassMap = new HashMap<Byte, Class>();
		nodeClassNameMap.put((byte) 1, PRNode.class.getName());
		nodeClassMap.put((byte) 1, PRNode.class);
				
		String location = "//"+myHostname+"/HashtableServer";
		HashtableServerImpl HashtableNode = new HashtableServerImpl(location, myId, configParams, useCaching, nodeClassNameMap, nodeClassMap);
		

		if(recordStats) HashtableNode.enableStats();

		HashtableNode.setNumServers(hostList.size());

		int numNodes = Integer.parseInt(args[4]);

		localRegistry.bind ("HashtableServer", HashtableNode);
		System.out.println("Hashtable server started");
		Thread.sleep(10000);

		//configure this server with handles of other servers while others are also doing the same
		HashtableNode.configureServer(hostList);
		System.out.println("Server configure complete");

		//storage system does not need to know about the useCaching flag since we have separate APIs
		StorageSystem  dataStorage;
		if(useCaching) {
			dataStorage =  new StorageSystem(hostList, HashtableNode, writeThroughCache, configParams, nodeClassNameMap, nodeClassMap);
		}else {
			dataStorage = new StorageSystem(hostList, HashtableNode, configParams, nodeClassNameMap, nodeClassMap);
		}
		if(recordStats) dataStorage.enableStats();

		/* Mar 29, 2013: read the worker count from config file */
		int numThreads = configParams.getIntValue("Worker.WORKER_COUNT");
		System.out.println("Number of workers:"+numThreads);  
		int count =0;
		if(master){
			FileWriter fw = new FileWriter("result.txt",true);
			BufferedWriter out = new BufferedWriter(fw);

			BufferedReader br = new BufferedReader(new FileReader(args[0]));
			String line = null;
			out.write("\n============================================================================\n");
			out.write("New Run: "+new Date().toString()+"\n");

			out.write("PageRankV2-Barrier model Test "+" "+Arrays.toString(args)+"\n");
			out.write("Number of Machines used");
			while((line= br.readLine())!= null)
			{
				count++;
			}
			out.write(" "+Integer.toString(count)+"\n");

			out.write("Number of threads:"+Integer.toString(numThreads)+"\n");



			out.write("Experiment settings description: \n");
			br = new BufferedReader(new FileReader(args[5]));
			line = null;
			while((line =br.readLine())!=null)
			{
				out.write(line+"\n");
			}
			out.close();
		}

		LocalWorkpoolImpl workpool = new LocalWorkpoolImpl(args[2], numThreads, new int[1]); 
		Naming.bind ("WorkpoolServer", workpool);
		Thread.sleep(20000);
		workpool.lookupPeers(hostList);


		if (master) {	
			InputFileReaderBatchV2 reader = new InputFileReaderBatchV2(args[3], numNodes, dataStorage);	
			reader.readAndLoadData();
			//reader.printTrace();
		} else {
			System.out.println("Running in slave mode..");
		}

		boolean master_thread = master;
		for (int i=0; i< numThreads; i++) {    
			if (i> 0) {	
				master_thread = false;
			}	
			PageRankWorker worker;
			if(useReflection) {
				worker = new PageRankWorkerReflection(machineList, workpool, master_thread, dataStorage, i, numNodes, location);
			}else {
				worker = new PageRankWorker(machineList, workpool, master_thread, dataStorage, i, numNodes, location);
			}

			Thread workerThread = new Thread(worker);
			workerThread.start();
		}
	}

}
