package TestProgs.PageRankV2;

import beehive.util.*;
import beehive.server.*;
import beehive.workpool.*;
import beehive.worker.*;
import java.rmi.server.UnicastRemoteObject;
import java.rmi.RMISecurityManager;
import java.rmi.RemoteException;
import java.rmi.Naming;
import java.rmi.registry.*;
import java.util.*;
import java.io.*;
import java.nio.channels.*;


public class PageRankWorkerReflection extends PageRankWorker implements Runnable {

	public PageRankWorkerReflection (String configFile, LocalWorkpoolImpl wpool, boolean master, StorageSystem datastorage, int workerId, int numNodes, String myStorageURL )  throws java.rmi.RemoteException {
		super(configFile, wpool, master, datastorage, workerId, numNodes, myStorageURL);
	}

	public Vector<NodeData> getBackNeighborsData(Node u, int phase) throws Exception {

		PRNode pr = (PRNode)u;
		Vector<NodeData> neighborNodesData = new Vector<NodeData>();
		Enumeration edges = pr.backNeighbors.keys();

		//for these, create the node data request object
		Vector<NodeData> request = new Vector<NodeData>();

		while ( edges.hasMoreElements() ){
			String key = (String) edges.nextElement();
			request.add(createRequestNodeData(key, phase));
		}

		neighborNodesData.addAll(dataStorage.execParallel(request));
		return neighborNodesData;
	}

	private NodeData createRequestNodeData(String nodeId, int phase) {
		NodeData rqND = new NodeData(nodeId, dataStorage.nodeClassNametoIDMap.get(PRNode.class.getName()));
		rqND.addProperty("fractionalRank", phase);
		return rqND;
	}

	public void calculateRankWithNodeData(PRNode u,int phase, Vector<NodeData> backNeighborNodeList) throws Exception {

		// sanity check
		double total=0;

		double tolerance;
		double tau = u.tau;
		double rank = 0;

		for(NodeData n: backNeighborNodeList) {
			rank += ((Double)n.getProperty("fractionalRank")).doubleValue();
		}


		rank  = (0.85*rank) + (0.15*tau);

		//updating our rank	
		u.rank[phase]=rank;

		//System.out.println("Computed Curr rank: "+u.rank[phase]+" Prev rank"+u.rank[(phase+1)%2]);

	}

	@Override
	public void doTask(Task tsk) throws Exception{

		Node origNode = dataStorage.getNode(tsk.getNodeId());
		int itrCount = ((PRTask)tsk).itrCount; 
		Vector<NodeData> backNeighborsData = getBackNeighborsData(origNode, itrCount%2);
		computeWithNodeData(tsk, origNode,	backNeighborsData);
		Vector<Node> updatedNodeList = new Vector<Node>();
		updatedNodeList.add(origNode);
		dataStorage.putNodesParallel(updatedNodeList);
		workpool.removeTask(tsk);
	}


	public void  computeWithNodeData(Task t, Node node, Vector<NodeData> backNeighborNodeList) { 
		try{
			//dont need a copy 
			PRNode u = (PRNode)node; 	
			PRTask prt = (PRTask)t;
			int itrCount = prt.itrCount;
			//System.out.println("Computing rank for node: "+ u.printNode()+" Iteration :"+itrCount+ " Phase: "+(itrCount%2));

			calculateRankWithNodeData(u, itrCount%2, backNeighborNodeList); 

		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.exit(1);
		}


	}
}