/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ShortestPathIncremental;


import beehive.util.*;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.io.*;

public class SPNode  extends Node implements Externalizable  {
	// public int nodeId;
	public int distFromSrc = Integer.MAX_VALUE;
	public String predecessorId = "-1";
	
	public int getDistFromSrc() {
		return distFromSrc;
	}

	public void setDistFromSrc(Integer distFromSrc) {
		this.distFromSrc = distFromSrc;
		//System.out.println("==> Node : "+this.nodeId+", TS : "+this.TS+", distFromSrc : "+this.distFromSrc);
		
	}

	public String getPredecessorId() {
		return predecessorId;
	}

	public void setPredecessorId(String predecessorId) {
		this.predecessorId = predecessorId;
		//System.out.println("==> Node : "+this.nodeId+", TS : "+this.TS+", predecessorId : "+this.predecessorId);
		
	}

	public SPNode(String id) {
		super( id );
	}

	public SPNode(SPNode n) {
		super(n, false);

		//reseting in order to add edges of type SPEdge
		this.neighbors = new ConcurrentHashMap<String, Edge>();
		
		Enumeration edgeKeys = n.neighbors.keys();
		while(edgeKeys.hasMoreElements())
		{
			String ekey = (String)edgeKeys.nextElement();
			SPEdge e = (SPEdge)n.neighbors.get(ekey);
			SPEdge f = new SPEdge(e);
			neighbors.put(new String(ekey),f);	
		}

		this.distFromSrc = n.distFromSrc;
		this.predecessorId  = new String(n.predecessorId);
	}

	public String printNode () {
		StringBuffer str =  new StringBuffer();
		str.append("Node id: "+ nodeId +"=" + " Distance from source="+distFromSrc+" Predecessor="+predecessorId);
		return  str.toString();
	}

	public String toString() {
		return printNode();
	}

	public String printNodeEdges () {
		StringBuffer str =  new StringBuffer();
		str.append("Node id: "+ nodeId +"=" + " Distance from source="+distFromSrc+" Predecessor="+predecessorId);
		Enumeration edges = neighbors.keys();
		while (edges.hasMoreElements()) {
			String key = (String) edges.nextElement();
			SPEdge e = (SPEdge) neighbors.get(key);
			str.append(e.toString());
		}
		return  str.toString();
	}
	
    @Override  
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {  
    	super.readExternal(in);
    	this.distFromSrc = in.readInt();
    	this.predecessorId = (String) in.readObject();
	}  

	@Override  
	public void writeExternal(ObjectOutput out) throws IOException {  
		super.writeExternal(out);
		out.writeInt(this.distFromSrc);
		out.writeObject(this.predecessorId);
	}
	
	public SPNode() {
		this.nodeId="-1";
		this.TS=0;
		this.neighbors = new ConcurrentHashMap<>();
	}
	
	public void addNeighbor(String index, SPEdge edge) {
    	this.neighbors.put(index, edge);
    }
}





