/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ShortestPathIncremental;


import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import beehive.util.*;

public class SPTask extends Task {

    public int distFromSrc;
    public int edgeWeight;
    public String senderId;
    public long TS;

    public SPTask(String nbrId, int distFromSrc, int edgeWeight, String senderId, long senderTS) {
        super(nbrId);
        this.distFromSrc = distFromSrc;
        this.edgeWeight = edgeWeight;
        this.senderId = senderId;
        this.TS = senderTS;
    }

    public SPTask() {
        super();
        this.distFromSrc = 0;
        this.edgeWeight = 0;
        this.senderId = "";
        this.TS = 0L;
    }

    public SPTask(String targetNodeId) {
        super(targetNodeId);
    }

    @Override
    public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
        super.readExternal(in);
        this.distFromSrc = in.readInt();
        this.edgeWeight = in.readInt();
        this.senderId = (String) in.readObject();
        this.TS = in.readLong();
    }

    @Override
    public void writeExternal(ObjectOutput out) throws IOException {
        super.writeExternal(out);
        out.writeInt(this.distFromSrc);
        out.writeInt(this.edgeWeight);
        out.writeObject(this.senderId);
        out.writeLong(this.TS);
    }
}