/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ShortestPathIncremental;

import beehive.util.Task;
import beehive.workpool.TaskComparator;

public class SPTaskComparator extends TaskComparator {
	
	@Override
	public int combine(Task oldTask, Task newTask) {
		Class c = newTask.getClass();
		String strclassnamearr[] = c.toString().split(" ")[1].split("\\.");
		String newTaskClassName = strclassnamearr[strclassnamearr.length-1];
		
		if(newTaskClassName.equals("SPTask")){
			if(oldTask == null){
				return ADD_TO_TASKPOOL_AND_UPDATE_TASKMAP;
			}
			else{
				SPTask oldSPTask = (SPTask) oldTask;
				SPTask newSPTask = (SPTask) newTask;
				replace(oldSPTask, newSPTask);
				return IGNORE_NEW_TASK;
			}
		}
		else{
			return ADD_TO_TASKPOOL_AND_DONT_UPDATE_TASKMAP;
		}
	}
	
	public void replace(SPTask oldSPTask, SPTask newSPTask){
		if( (oldSPTask.distFromSrc + oldSPTask.edgeWeight) > (newSPTask.distFromSrc + newSPTask.edgeWeight) ){
			//copy the content of newTask to the contents of oldTask
			oldSPTask.distFromSrc = newSPTask.distFromSrc;
			oldSPTask.edgeWeight = newSPTask.edgeWeight;
			oldSPTask.senderId = newSPTask.senderId;
			oldSPTask.TS = newSPTask.TS;
			
			oldSPTask.nodeId = newSPTask.nodeId;
			oldSPTask.txnInfo = newSPTask.txnInfo;
			oldSPTask.phase = newSPTask.phase;
			oldSPTask.affinityLevel = newSPTask.affinityLevel;
			oldSPTask.targetLocation = newSPTask.targetLocation;
			oldSPTask.status = newSPTask.status;
		}
	}
}
