/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.ShortestPathIncremental;

import beehive.server.*;
import beehive.util.*;
import beehive.worker.*;

import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

import TestProgs.ShortestPathIncremental.SPWorker.NeighborInfo;
import TestProgs.util.BeehiveAppLoader;

public class SPTest extends BeehiveAppLoader {

    public static void main(String args[]) throws Exception {
        if (args.length != 7) {
            throw new RuntimeException("Syntax: SPTest <hostListFile> <config params file> <workpoolserver hostname> <input filename> <num_nodes> <MsgFile> <scrNode>");
        }

        //All inherited Node classes used
        HashMap<Byte, String> nodeClassNameMap = new HashMap<Byte, String>();
        HashMap<Byte, Class> nodeClassMap = new HashMap<Byte, Class>();
        nodeClassNameMap.put((byte) 1, new SPNode().getClass().getName());
        nodeClassMap.put((byte) 1, new SPNode().getClass());

        SPTest loader = new SPTest();
        loader.init(args, "SPTest", nodeClassNameMap, nodeClassMap);

        loader.workpool.taskComparator = new SPTaskComparator();

        System.out.println(loader.workpool.myHost + " completed Initialization...");
        Thread.sleep(10000);

        ConcurrentHashMap<String, NeighborInfo> neighborDistances = new ConcurrentHashMap<String, NeighborInfo>();
        boolean master_thread = loader.isMaster();
        for (int i = 0; i < loader.getNumThreads(); i++) {
            if (i > 0) {
                master_thread = false;
            }
            Worker worker = new SPWorker(loader.getHostlistFile(), loader.getWorkpool(), master_thread, loader.getDataStorage(), i, loader.getLocation(), args[6], loader.getNumNodes(), neighborDistances);
            Thread workerThread = new Thread(worker);
            workerThread.start();
        }
    }

    public void loadGraphData(String input_file, HashtableServer server, int numNodes, StorageSystem dataStorage) {
        try {
            InputFileReaderDistributed fileReader = new InputFileReaderDistributed(input_file, server);
            fileReader.readAndLoadData();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
