/*
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 * % Beehive Parallel Programming Framework
 * % © Regents of the University of Minnesota.
 * % This software is licensed under GPL v3.0 (https://www.gnu.org/licenses/gpl-3.0.en.html) open source license.
 * %     For an alternative license please contact the Office for Technology Commercialization at the University of Minnesota.
 * %
 * % How to cite:
 * %     If you use this software for your work, please use the following citation:
 * %   Anand Tripathi, Vinit Padhye, Tara Sasank Sunkara, Jeremy Tucker, Bhagavathi Dhass Thirunavukarasu, Varun Pandey, Rahul R. Sharma,
 * %   "A Transactional Model for Parallel Programming of Graph Applications on Computing Clusters,"
 * %   2017 IEEE 10th International Conference on Cloud Computing (CLOUD), Honolulu, HI, USA, June 25-30, 2017.
 * %
 * %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 */

package TestProgs.util;

import java.rmi.RemoteException;
import java.util.HashSet;

public class DistributedSynchronizerImpl implements DistributedSynchronizer {	
	//not extending UnicastRemoteObject. Use UnicastRemoteObject.exportObject() call with the custom client and server socket factory configs 
	int totalMembers=0;
	boolean isCoordinator = false;
	HashSet<String> members = new HashSet<String>();
	
	public DistributedSynchronizerImpl() throws RemoteException {}
	
	public DistributedSynchronizerImpl(int totalMembers, boolean isCoordinator) throws RemoteException {
		this.totalMembers = totalMembers;
		this.isCoordinator = isCoordinator;
	}
	
	public synchronized void synchronize(String host) throws RemoteException, InterruptedException{
		this.members.add(host);
		
		if ( members.size() == totalMembers && isCoordinator) {
                notifyAll();
        }
        else{
        		wait();
    	}
	}
	
    public synchronized void reset() throws RemoteException {
    	if(isCoordinator){
    		members = new HashSet<String>();
    	}
    	else{
    		System.out.println("Error :: Reset operation not supported! Only Coordinator can reset the barrier.");
    	}
    	
    }

}